;===============================================================================
;;; 8 --> CRATION DE FONCTIONS : primitive defun et lambda-listes -- aperu des
;;;       macro-fonctions -- primitives trace et untrace
;$$ 8 --> FUNKTIONEN DEFINIEREN: das defun-Primitive und lambda-Listen --
;$$       Vorschau auf Makro-Funktionen -- die Primitive trace und untrace zur
;$$       Ablaufverfolgung                                            [d..H.B.]
;++ 8 --> CREATING FUNCTIONS: defun Primitive and Lambda-List--Macro-Functions
;++       Preview--trace and untrace Primitives
;===============================================================================

;;   Premiers exemples...
;$   Erste Beispiele...
;+   First examples...

(defun 3liste (a1 a2 a3)                                             ;[Wertz:17]
  (cons a1 (cons a2 (cons a3 ()))))
(defun truc (qqc)                                                    ;[Wertz:18]
  (cons qqc (cons qqc (cons qqc ()))))
(defun echange (couple)
  (list (cadr couple) (car couple)))

;;   Exemples arithmtiques...
;$   Arithmetische Beispiele...
;+   Arithmetic examples...

(defparameter temp 'contenu_global_de_temp)      ;temp = variable GLOBALE
                                                 ;N.B. : setq est aussi possible
(defun f-a-c1 (temp)
  (/ (- temp 32.) 1.8))
(defun pc-augm (x y)
  (/ (* 100.0 (- y x)) x))

;;   Exemples d'un corps compos d'une squence de plusieurs formes.
;; Dmonstration de l'indpendance d'une variable locale (interne) d'une
;; fonction, par rapport  une variable globale, et contre-exemple.
;$   Beispiel eines Funktionsrumpfes bestehend aus einer Reihe von Anweisungen.
;$ Demonstration der Unabhngigkeit der lokalen (internen) Variable einer
;$ Funktion von einer globalen Variable, und Gegenbeispiel.
;+   Example of a body composed of a sequence of many forms. Demonstration of
;+ the independence of a local (internal) function variable from a global
;+ variable, and counterexample.

(defun f-a-c2 (temp)                             ;BONNE programmation :
  (setq temp (- temp 32.))                       ;temp = variable LOCALE
  (princ "temp de f-a-c2 = ")(princ temp)        ;  "       "       "
  (/ temp 1.8))                                  ;  "       "       "
(defun f-a-c3 (fare)                             ;MAUVAISE programmation :
  (setq temp (- fare 32.))                       ;temp = variable GLOBALE
  (princ "temp de f-a-c3 = ")(princ temp)        ;  "       "       "
  (/ temp 1.8))                                  ;  "       "       "

;;   Exemples abords au Chapitre 7 (certaines primitives de Le_Lisp) :
;;    firstn <liste> <n>      => liste des <n> premiers lments de <liste>
;;    lastn  <liste> <n>      => liste des <n> derniers lments de <liste>.
;;                               quivalente LISP : last avec second argument
;;                               optionnel
;;    append1 <liste> <atome> => insre <atome> en fin de <liste>, en
;;                               conservant une structure normale de <liste>. Se
;;                               comporte comme un cons en fin de <liste>
;$   Beispiele aus Kapitel 7 (ausgewhlte Le_Lisp Primitive):
;$    firstn <liste> <n>      => Liste der ersten <n> Glieder der <liste>
;$    lastn  <liste> <n>      => Liste der letzten <n> Glieder der <liste>.
;$                               LISP-Equivalent: last mit zweitem optionalen
;$                               Argument
;$    append1 <liste> <atome> => fgt ein <atome> am Ende der <liste> ein und
;$                               behlt eine normale Listenstruktur bei. Verhlt
;$                               sich wie ein cons am Ende einer Liste
;+   Examples shown in Chapter 7 (certain Le_Lisp primitives):
;+    firstn <liste> <n>      => list of the <n> first elements of <liste>
;+    lastn  <liste> <n>      => list of the <n> last elements of <liste>. LISP
;+                               equivalent: last with second optional argument
;+    append1 <liste> <atome> => inserts <atome> at the end of <liste> while
;+                               preserving a normal list structure. Behaves as
;+                               an end of list cons

(defun firstn1 (liste n)
  (butlast liste (- (list-length liste) n)))
(defun lastn1 (liste n)
  (reverse (butlast (reverse liste) (- (list-length liste) n))))
(defun append1 (liste atome)
  (append liste (list atome)))

;;   Dmonstration du passage des valeurs des arguments d'appel comme valeurs
;; des paramtres, ou variables locales (internes),  l'intrieur d'une
;; fonction.
;$   Demonstration der Weitergabe der Werte der aufgerufenen Argumente als
;$ Parameterwerte, oder lokale (interne) Variablen, innerhalb einer Funktion. 
;+   Demonstration of the passing of the values of the calling arguments as
;+ values of parameters, or local (internal) variables, inside a function.

(defun echo1 (p) (format *debug-io* "p = ~S" p)       p )
(defun echo2 (p) (format *debug-io* "p = ~S" p) (eval p))

;;   Aperu des macro-fonctions. Fonctions sans valuation de leurs arguments
;; d'appel, suivies de l'valuation de leur propre valeur : effet d'valuation
;; retarde des arguments d'appel.
;$   Vorschau auf Makrofunktionen. Funktionen ohne Evaluierung ihrer Argumente,
;$ gefolgt von der Evaluierung ihrer eigenen Werte: verzgern der Evaluierung
;$ der Aufrufargumente.
;+   Preview of macro-functions. Functions without evaluation of their calling
;+ arguments, followed by the evaluation of their own value : delaying the
;+ evaluation of the calling arguments.

(defmacro echo3 (p) (format *debug-io* "p = ~S" p)       p )
(defmacro echo4 (p) (format *debug-io* "p = ~S" p) (eval p))

;;   Exemples d'arguments optionnel avec valeur par dfaut, et de documentation
;; intgre sous forme de documentation-string.
;;   En Macintosh Common LISP (MCL), ces dernires ne sont rellement
;; sauvegardes avec les dfinitions de fonctions que si la variable
;; d'environnement *save-doc-strings* a la valeur t (cf. fichier +clinit).
;;   En LispWorks, les documentation-strings sont sauvegardes par dfaut.
;$   Beispiel eines optionalen Arguments mit Standardwert und integrierter
;$ Hilfemeldung.
;$   In Macintosh Common Lisp (MCL) wird letzteres nicht wirklich mit der
;$ Funktionsdefinition gespeichert, auer wenn die Umgebungsvariable
;$ *save-doc-strings* auf t gesetzt wird (z. B. in der Datei +clinit).
;$   In LispWorks werden die documentation-strings standardmig gespeichert.
;+   Examples of optional arguments with default value, and integrated
;+ documentation-strings.
;+   In Macintosh Common LISP (MCL), the latter are not really saved with
;+ function definitions unless the *save-doc-strings* environment variable has
;+ the value t (cf. +clinit file).
;+   In LispWorks, the documentation-strings are by default saved.

;;    lastn2 <liste> &optional (<n> 1) => liste du(des) <n> dernier(s)
;;                                        lment(s) de <liste>. quivalente
;;                                        LISP : last
;$    lastn2 <liste> &optional (<n> 1) => Liste der letzten <n> Glieder der
;$                                        <liste>. LISP-Equivalent: last
;+    lastn2 <liste> &optional (<n> 1) => list of the last <n> element(s) of
;+                                        <liste>. LISP equivalent: last

(defun lastn2 (liste &optional (n 1))
"
;   Equivalente de la primitive last [CLtL2:416-417] :
;    lastn2 <liste> &optional (<n> 1) => <n> dernier(s) element(s) de <liste>
"
  (reverse (butlast (reverse liste) (- (list-length liste) n))))

;;   Firstn, ici avec <n> optionnel : meilleure version que firstn1 ci-dessus.
;;    firstn2 <liste> &optional (<n> 1) => liste du(des) <n> premier(s)
;;                                         lment(s) de <liste>
;$   Firstn, hier mit optionalem <n>: bessere Version als firstn1 oben.
;$    firstn2 <Liste> &optional (<n> 1) => Liste der ersten <n> Glieder der
;$                                         <liste>
;+   Firstn, here with optional <n>: better version than firstn1 above.
;+    firstn2 <liste> &optional (<n> 1) => list of the <n> first element(s) of
;+                                         <liste>

(defun firstn2 (li &optional (n 1))
"
;    firstn2 <liste> &optional (<n> 1) => <n> premier(s) element(s) de <liste>
"
  (butlast li (- (list-length li) n)))

;;   Pour obtenir la documentation-string de la fonction <fff> [CLtL2:695],
;; faire :
;$   Fhre folgendes aus um den documentation-string der Funktion <fff>
;$ [CLtL2:695] zu erhalten:
;+   To obtain the documentation-string of function <fff> [CLtL2:695], do:
;-    (documentation '<fff> 'function)
;; ou, pour se faciliter la vie, utiliser la macro-fonction suivante... Par
;; souci de commodit, la macro-fonction dispense l'utilisateur de quoter
;; l'argument d'appel, et ralise une "expansion" de la forme d'appel abrge.
;$ Oder, um sich das Leben einfacher zu machen, benutze das folgende Makro...
;$ Aus Grnden der Bequemlichkeit befreit das Makro den Benutzer von der
;$ Pflicht, die Aufrufargumente zu zitieren und erstellt eine "Erweiterung" des
;$ abgekrzten Aufrufs
;+ or, to make life easier, use the following macro-function... For the sake of
;+ convenience, the macro-function exempts the user from quoting the calling
;+ arguments, and realizes an "expansion" of the abbreviated calling form.

(defmacro dof1 (nom)
  (list 'documentation
        (list 'quote nom)
        (list 'quote 'function)))

;;   Exemples de manipulations simples...
;$   Beispiele simpler Manipulationen...
;+   Examples of simple manipulations...

;; Rotations, ou permutations circulaires :
;$ Rotationen, oder zyklische Permutationen:
;+ Rotations, or circular permutations:

;- <==G==\                                  " gauche" / "links" / "to the left"
;- (*    *    *    *    *    *    *)
;-  \----------------------------->
(defun rotag1 (li)
  (append (cdr li) (list (car li))))

;-                           /==D==>      " droite" / "rechts" / "to the right"
;- (*    *    *    *    *    *    *)
;-  <-----------------------------/
(defun rotad1a (li)                         ;version la plus simple
  (append (last li) (butlast li)))
(defun rotad1b (li)                         ;mme chose, mais plus complique...
  (append (last li) (firstn2 li (- (list-length li) 1)))) ;firstn2 : ci-dessus
(defun rotad1c (li)                         ;idem
  (append (last li) (reverse (cdr (reverse li)))))
(defun rotad1d (li)  ;Danger : remove peut enlever + d'une occurrence du dernier
  (append (last li) (remove (car (last li)) li)))    
(defun rotad1e (li)  ;Celle-ci corrige, mais n'est pas belle  voir !
  (append (last li) (remove (car (last li)) li :from-end t :count 1)))

;- Palindromes : / Palindrome: / Palindromes:

(defun palindroA (li)
  (append li (reverse li)))
(defun palindroB (li)
  (append li (cdr (reverse li))))

;;   Cration de carrs latins et magiques d'ordre 3. Application simplifie de
;; la rgle de E. Lucas [cf. Stewart, Ian, 1997 : "Les carrs alphamagiques",
;; <Pour la science>, 232:96]. Constante ou somme magique du carr = 3 * premier
;; argument.
;$   Erstellen lateinischer und magischer Quadraten 3. Ordnung. Vereinfachte
;$ Anwendung der Regel von E. Lucas [cf. Stewart, Ian, 1997 : "Les carrs
;$ alphamagiques", <Pour la science>, 232:96]. Konstante oder magische Summe
;$ des Quadrates = 3 * erstes Argument.
;+   Creation of latin and magical squares of order 3. Simplified application of
;+ the rule of E. Lucas [cf. Stewart, Ian, 1997 : "Les carrs alphamagiques",
;+ <Pour la science>, 232:96]. Constant or magic sum of the square = 3 * first
;+ argument.
;- Exemples : / Beispiele: / Examples:
;-    (carre1  4 1) => ((4 3 5) (5 4 3) (3 5 4))   =   4 3 5
;-                                                     5 4 3
;-                                                     3 5 4
;-    (carre2  2 3) => ((2 -1 5) (5 2 -1) (-1 5 2))
;-    (carre1 11 6) =>
;-        ((11 5 17) (17 11 5) (5 17 11)), cf. ci-dessous / unten / below
;;   Pour simplifier, une seule interprtation de la rgle est systmatiquement
;; applique. Deux ralisations diffrentes donnent le mme rsultat.
;$   Zur Vereinfachung wird eine einzige Interpretation der Regel systematisch
;$ angewendet. Zwei verschiedene Umsetzungen ergeben dasselbe Ergebnis.
;+   To simplifiy, a single interpretation of the rule is systematically
;+ applied. Two different realizations give the same result.

(defun carre1 (centre diff)
  (list (list centre            (- centre diff)   (+ centre diff)   )
        (list (+ centre diff)   centre            (- centre diff)   )
        (list (- centre diff)   (+ centre diff)   centre            )))
(defun carre2 (centre diff)
  (carre-auxiliaire (list centre (- centre diff) (+ centre diff))))
(defun carre-auxiliaire (liste)
  (list liste (rotad1a liste) (rotad1a (rotad1a liste))))

;;   Avec application de la rgle de Lucas complte, on obtient davantage de
;; varit des valeurs du carr [ibid.], mais il n'est latin que si <b> = 0.
;$   Bei vollstndiger Anwendung der Lucas-Regel erhlt man eine grere
;$ Abwechslung der Werte des Quadrats [ebd.], jedoch ist es nur lateinisch
;$ wenn <b> = 0.
;+   When applying the complete rule of Lucas, one obtains a greater variety in
;+ the values of the square [ibid.], but it is latin only if <b> = 0.
;- Exemples : / Beispiele: / Examples:
;-    (carre3  7 11 -2) => ((18 -2  5) (-6  7 20) ( 9 16 -4))
;-    (carre3  5 -1 -3) => (( 4  9  2) ( 3  5  7) ( 8  1  6)) cf. Fu Xi -XXIIIe
;-    (carre3  5  1  3) => (( 6  1  8) ( 7  5  3) ( 2  9  4)) variante du mme
;-    (carre3 11  0  6) => ((11  5 17) (17 11  5) ( 5 17 11)) = (carre1 11 6)
;; Dans tous les cas, la constante du carr est toujours 3 * premier argument.
;; Cf. aussi : carrs hypermagiques, Chapitre 16.3.
;$ Dennoch ist in jedem Fall die Konstante des Quadrats immer 3 * das erste
;$ Argument. Siehe auch: Hypermagische Quadrate, Kapitel 16.3.
;+ In all cases, the squares's constant is still always 3 * first argument. See
;+ also: hypermagic squares, Chapter 16.3.

(defun carre3 (a b c)
  (list (list (+ a b)         (- a b c)   (+ a c)         )
        (list (+ (- a b) c)   a           (+ a (- b c))   )
        (list (- a c)         (+ a b c)   (- a b)         )))

;;   Exemples d'utilisation des primitives trace et untrace. Faire des essais
;; avec rotad1b et lastn2, aprs valuation des formes suivantes :
;$   Beispiel des Gebrauchs der Primitive trace und untrace. Fhre Tests mit
;$ rotad1b und lastn2 nach folgendem Ablauf durch:
;+   Example of use of the trace and untrace primitives. Run tests with rotad1b
;+ and lastn2, after evaluating the following forms:
;-    (trace rotad1b butlast firstn2 reverse), puis /dann / then
;-    (untrace rotad1b), puis / dann / then
;-    (untrace) etc.

;;; Remarques sur certaines caractristiques de la rgle d'douard Lucas pour
;;; la cration de carrs magiques d'ordre 3 (fonction carre3)
;$$ Bemerkungen ber bestimmte Eigenschaften von douard Lucas's Regel zur
;$$ Erzeugung von magischen Quadraten dritter Ordnung (Funktion carre3).
;++ Remarks about certain characteristics of douard Lucas's rule for the
;++ creation of magical squares of order 3 (carre3 function)

;;   On peut obtenir des variantes intressantes d'un carr magique obtenu grce
;;  la rgle de Lucas complte avec trois arguments <a>, <b> et <c>, en
;; changeant le(s) signe(s) de certains de ces arguments. Les proprits
;; constates refltent la structure mme de la rgle de Lucas.
;;   Afin de mettre en vidence les symtries entre les variantes obtenues, tous
;; les exemples suivants utilisent les mmes arguments : <a> = 5, <b> = 1 et
;; <c> = 4. Bien entendu, cette restriction n'a pour but que de mieux illustrer
;; le propos, mais des rsultats similaires seraient obtenus avec des valeurs
;; quelconques.
;;   La somme magique des carrs est toujours gale  3 fois le premier argument
;; d'appel de la fonction carre3. Dans les exemples ci-dessous, elle sera donc
;; 15 ou -15.
;;   Il existe sept possibilits.
;$   Bei Verwendung der ganzen Lucas-Regel mit drei Argumenten <a>, <b> und <c>
;$ sind interessante Varianten der magischen Quadrate durch ndern der
;$ Vorzeichen von manchen dieser Argumente mglich. Die beobachteten Ergebnisse
;$ spiegeln die eigentliche Struktur der Lucas-Regel wider.
;$   Um die Symmetrien zwischen den sich ergebenden Varianten zu betonen,
;$ verwenden die folgenden Beispiele dieselben Argumente: <a> = 5, <b> = 1
;$ und <c> = 4. Natrlich ist diese Einschrnkung nur dazu da, den Zweck besser
;$ zu verdeutlichen, jedoch wrden hnliche Ergebnisse auch mit beliebigen
;$ anderen Werten erzielt werden.
;$   Die magische Summe der Quadrate entspricht immer 3 mal dem ersten Argument
;$ der carre3-Funktion. Im folgenden Beispiel wird sie deshalb 15 oder -15 sein.
;$   Es gibt sieben Mglichkeiten.
;+   Interesting variants of a magic square are possible when using the
;+ complete rule of Lucas with three arguments <a>, <b> and <c>, by changing the
;+ sign(s) of certain of these arguments. The results observed reflect the very
;+ structure of Lucas's rule.
;+   In order to highlight the symmetries between the resulting variants, all
;+ the following examples use the same arguments: <a> = 5, <b> = 1 and <c> = 4.
;+ Of course, this restriction is only intended to give a better illustration of
;+ the purpose, but similar results would be obtained with any values.
;+   The magical sum of the squares is always equal to 3 times the first calling
;+ argument of the carre3 function. In the examples below, it will thus be 15 or
;+ -15.
;+   There are seven possibilities.

;- 1. -<a>  <b>  <c>
;- Exemples : / Beispiele: / Examples:
;-    (carre3  5  1  4) => (( 6   0  9) (  8  5   2) ( 1  10  4))
;-    (carre3 -5  1  4) => ((-4 -10 -1) ( -2 -5  -8) (-9   0 -6))
;-
;;                   6   0   9          -4 -10  -1
;; soit les carrs   8   5   2    et    -2  -5  -8
;;                   1  10   4          -9   0  -6
;$                      6   0   9          -4 -10  -1
;$ ergibt die Quadrate  8   5   2   und    -2  -5  -8
;$                      1  10   4          -9   0  -6
;+                      6   0   9          -4 -10  -1
;+ namely the squares   8   5   2   and    -2  -5  -8
;+                      1  10   4          -9   0  -6
;-
;; Exception faite de ce que tous leurs nombres sont de signes opposs, ces
;; carrs magiques sont symtriques et forment un palindrome en deux dimensions.
;; Ceci se visualise le mieux comme suit :
;$ Mit der Ausnahme, dass alle ihre Zahlen unterschiedliche Vorzeichen haben,
;$ sind diese magischen Quadrate symmetrisch und bilden ein zweidimensionales
;$ Palindrom. Am besten kann dies folgendermaen veranschaulicht werden:
;+ Except that all their numbers are opposite in sign, these magic squares are
;+ symmetrical and form a two dimensional palindrome. This can be best
;+ visualized as follows:
;-    (append (carre3 5 1 4) (carre3 -5 1 4))
;-        => ((6 0 9) (8 5 2) (1 10 4) (-4 -10 -1) (-2 -5 -8) (-9 0 -6))
;-    (append (apply #'append (carre3 5 1 4)) (apply #'append (carre3 -5 1 4)))
;-        => (6 0 9 8 5 2 1 10 4 -4 -10 -1 -2 -5 -8 -9 0 -6)

;- 2.  <a> -<b>  <c>
;- Exemples : / Beispiele: / Examples:
;-    (carre3  5  1  4) => (( 6   0  9) (  8  5   2) ( 1  10  4))
;-    (carre3  5 -1  4) => (( 4   2  9) ( 10  5   0) ( 1   8  6))
;-
;;                   6   0   9           4   2   9
;; soit les carrs   8   5   2    et    10   5   0
;;                   1  10   4           1   8   6
;$                        6   0   9           4   2   9
;$ ergibt die Quadrate    8   5   2   und    10   5   0
;$                        1  10   4           1   8   6
;+                      6   0   9           4   2   9
;+ namely the squares   8   5   2   and    10   5   0
;+                      1  10   4           1   8   6
;-
;; On note les interversions : les lignes de l'un sont les colonnes de l'autre
;; en sens de lecture contraires.
;$ Man beachte die Vertauschungen: die Zeilen des einen sind die Spalten des
;$ anderen in umgekehrter Richtung.
;+ Note the interchanges: the lines of one are the columns of the other in
;+ opposite direction

;- 3.  <a>  <b> -<c>
;- Exemples : / Beispiele: / Examples:
;-    (carre3  5  1  4) => (( 6   0  9) (  8  5   2) ( 1  10  4))
;-    (carre3  5  1 -4) => (( 6   8  1) (  0  5  10) ( 9   2  4))
;-
;;                   6   0   9           6   8   1
;; soit les carrs   8   5   2    et     0   5  10
;;                   1  10   4           9   2   4
;$                       6   0   9           6   8   1
;$ ergibt die Quadrate   8   5   2   und     0   5  10
;$                       1  10   4           9   2   4
;+                      6   0   9           6   8   1
;+ namely the squares   8   5   2   and     0   5  10
;+                      1  10   4           9   2   4
;-
;; Mme type d'interversions de lignes et colonnes, symtriques des prcdentes.
;$ Dieselbe Vertauschung der Zeilen und Spalten wie beim vorangegangenen.
;+ Same type of interchanges in lines and columns, symmetrical of the preceding.

;- 4. -<a> -<b>  <c>
;- Exemples : / Beispiele: / Examples:
;-    (carre3  5  1  4) => (( 6   0  9) (  8  5   2) ( 1  10  4))
;-    (carre3 -5 -1  4) => ((-6  -8 -1) (  0 -5 -10) (-9  -2 -4))
;-
;;                   6   0   9          -6  -8  -1
;; soit les carrs   8   5   2    et     0  -5 -10
;;                   1  10   4          -9  -2  -4
;$                      6   0   9          -6  -8  -1
;$ ergibt die Quadrate  8   5   2   und     0  -5 -10
;$                      1  10   4          -9  -2  -4
;+                      6   0   9          -6  -8  -1
;+ namely the squares   8   5   2   and     0  -5 -10
;+                      1  10   4          -9  -2  -4
;-
;; Mmes cas que 3. ci-dessus avec changements de signes.
;$ Gleich dem drittletzten mit Vorzeichennderung.
;+ Same case as 3. above with sign changes.

;- 5. -<a>  <b> -<c>
;- Exemples : / Beispiele: / Examples:
;-    (carre3  5  1  4) => (( 6   0  9) (  8  5   2) ( 1  10  4))
;-    (carre3 -5  1 -4) => ((-4  -2 -9) (-10 -5   0) (-1  -8 -6))
;-
;;                   6   0   9          -4  -2  -9
;; soit les carrs   8   5   2    et   -10  -5   0
;;                   1  10   4          -1  -8  -6
;$                      6   0   9          -4  -2  -9
;$ ergibt die Quadrate  8   5   2   und   -10  -5   0
;$                      1  10   4          -1  -8  -6
;+                      6   0   9          -4  -2  -9
;+ namely the squares   8   5   2   and   -10  -5   0
;+                      1  10   4          -1  -8  -6
;-
;; Mmes cas que 2. ci-dessus avec changements de signes.
;$ Gleich dem zweitletzten mit Vorzeichennderung.
;+ Same case as 2. above with sign changes.

;- 6.  <a> -<b> -<c>
;- Exemples : / Beispiele: / Examples:
;-    (carre3  5  1  4) => (( 6   0  9) (  8  5   2) ( 1  10  4))
;-    (carre3  5 -1 -4) => (( 4  10  1) (  2  5   8) ( 9   0  6))
;-
;;                   6   0   9           4  10   1
;; soit les carrs   8   5   2    et     2   5   8
;;                   1  10   4           9   0   6
;$                      6   0   9           4  10   1
;$ ergibt die Quadrate  8   5   2   und     2   5   8
;$                      1  10   4           9   0   6
;+                      6   0   9           4  10   1
;+ namely the squares   8   5   2   and     2   5   8
;+                      1  10   4           9   0   6
;-
;; Cf. 1. ci-dessus, mais sans changements de signes : ces carrs sont
;; symtriques et forment un palindrome en deux dimensions. Ceci se visualise le
;; mieux comme suit :
;$ Vgl. das vorletzte Beispiel, aber ohne Vorzeichennderung: diese Quadrate
;$ sind symmetrisch und bilden ein zweidimensionales Palindrom. Dies kann am
;$ besten folgendermaen veranschaulicht werden:
;+ Cf. 1. above, but without sign changes: these squares are symmetrical and
;+ form a two dimensional palindrome. This can be best visualized as follows:
;-    (append (carre3 5 1 4) (carre3 5 -1 -4))
;-        => ((6 0 9) (8 5 2) (1 10 4) (4 10 1) (2 5 8) (9 0 6))
;-    (append (apply #'append (carre3 5 1 4))(apply #'append (carre3 5 -1 -4)))
;-        => (6 0 9 8 5 2 1 10 4 4 10 1 2 5 8 9 0 6)

;- 7. -<a> -<b> -<c>
;- Exemples : / Beispiele: / Examples:
;-    (carre3  5  1  4) => (( 6   0  9) (  8  5   2) ( 1  10  4))
;-    (carre3 -5 -1 -4) => ((-6   0 -9) ( -8 -5  -2) (-1 -10 -4))
;-
;;                   6   0   9          -6   0  -9
;; soit les carrs   8   5   2    et    -8  -5  -2
;;                   1  10   4          -1 -10  -4
;$                      6   0   9          -6   0  -9
;$ ergibt die Quadrate  8   5   2   und    -8  -5  -2
;$                      1  10   4          -1 -10  -4
;+                      6   0   9          -6   0  -9
;+ namely the squares   8   5   2   and    -8  -5  -2
;+                      1  10   4          -1 -10  -4
;-
;; On obtient deux carrs magiques de valeurs absolues identiques.
;$ Man erhlt zwei magische Quadrate mit gleichen Absolutwerten.
;+ One obtains two magic squares of identical absolute values.