;===============================================================================
;;; 9 --> LOGIQUE, CONDITIONS, CONTRLE : primitives if, when et cond
;$$ 9 --> LOGIK, BEDINGUNGEN, KONTROLLE: if-, when- und cond-Primitive[d..H.B.]
;++ 9 --> LOGIC, CONDITIONS, CONTROL: if, when and cond Primitives
;===============================================================================

;;; Calcul de propositions logiques, logique formelle
;;; Cf. [WITTGENSTEIN, Ludwig, 1961(1922) : <Tractatus logico-philosophicus> :
;;; 64-66 5.101, 92]. Les citations reprises ci-dessous corrigent une
;;; vingtaine d'erreurs typographiques d'une dition franaise.
;$$ Berechnung logischer Stze, formale Logik
;$$ Vgl. [WITTGENSTEIN, Ludwig, 1961(1922) : <Tractatus logico-philosophicus> :
;$$ 64-66 5.101, 92]. Die nachstehenden Zitate korregieren ungefhr zwanzig
;$$ Druckfehler der franzsischen Ausgabe.
;++ Calculus of Logical Propositions, Formal Logic
;++ Cf. [WITTGENSTEIN, Ludwig, 1961(1922) : <Tractatus logico-philosophicus> :
;++ 64-66 5.101, 92]. The quotations below correct about twenty misprints of
;++ a French edition.

;;   Fonctions ralisant les 16 propositions  deux termes de L. Wittgenstein,
;; avec ajout de deux propositions  un seul terme (tautot1 et contra1).
;;   Avec la macrofonction tv, dfinie dans le fichier ped.09-tv, on peut
;; obtenir la table de vrit de n'importe quelle fonction <fonc> ci-dessous, en
;; valuant
;$   Funktionen, die L. Wittgenstein's 16 zweiteilige logische Folgerungen
;$ umsetzen, zuzglich zweier einteiliger Folgerungen (tautot1 und contra1).
;$   Mittels der Makro-Funktion tv, definiert in Datei ped.09-tv, kann man eine
;$ Wahrheitstabelle fr eine beliebige der folgenden <fonc>-funktionen erhalten
;$ indem man folgendes ausfhrt:
;+   Functions realizing L. Wittgenstein's 16 two term logical propositions,
;+ with addition of two one term propositions (tautot1 and contra1).
;+   With the tv macrofunction, defined if file ped.09-tv, one can obtain the
;+ truth table of any <fonc> function below, by evaluating
;-    (tv <fonc>)
;; Exemple, cf. W-2 ci-dessous :
;$ Beispiel, Vgl. W-2 unten:
;+ Example, cf. W-2 below:
;-    (tv implic) =>    T <=  (IMPLIC  NIL NIL)
;-                      T <=  (IMPLIC  NIL   T)
;-                    NIL <=  (IMPLIC    T NIL)
;-                      T <=  (IMPLIC    T   T)
;; La macrofonction tv dresse aussi les tables de vrit des primitives
;; (logiques et LISP) NOT, OR et AND.
;$ Die Makro-Funktion tv erstellt auch eine Wahrheitstabelle fr die (logischen
;$ und LISP) Primitive NOT, OR und AND.
;+ The tv macrofunction also composes the truth tables of the (logical and LISP)
;+ primitives NOT, OR and AND.

;- W-0
;;   Tautologie,  une et deux propositions lmentaires (ou termes) : rsultats
;; toujours T. Exemples, que <p> et <q> soient indiffremment T ou NIL :
;$   Tautologie, mit einen oder zwei elementaren Stzen (oder Ausdrcken):
;$ Ergebnis ist immer T. Beispiele, bei denen <p> und <q> T oder NIL sein
;$ knnen: 
;+   Tautology, with one and two elementary propositions (or terms): results
;+ always T. Examples, <p> and <q> being indifferently T or NIL:
;-    (tautot1 <p>) => T ; (tautot2 <p> <q>) => T

(defun tautot1 (2b)
  (or 2b (not 2b)))                                      ;To be, OR not to be...

;$ "Wenn p, so p; und wenn q, so q" [WITTGENSTEIN: ibid.]
;+ "If p, so p; and if q, so q" [WITTGENSTEIN: ibid.]
(defun tautot2 (p q)       ;"si p, soit p ; si q, soit q" [WITTGENSTEIN : ibid.]
  (and (implic p p) (implic q q)))                    ;(( p -> p )  ( q -> q ))

;- W-1
;;   Ngation de ET.
;$   Negation von UND. "Nicht beides p und q" [ibid.]
;+   Negation of AND. "Not both p and q" [ibid.]

(defun W-1 (p q)                             ;"non pas tous deux p et q" [ibid.]
  (not (and p q)))                                                ;( ( p  q ))

;- W-2
;;   Implication de Philon.
;$   Philo's Folgerung. "Wenn p, so q" [ibid.]
;+   Philo's implication. "If p, so q" [ibid.]

(defun implic (p q)                                      ;"si p, soit q" [ibid.]
  (not (and p (not q))))                           ;( p -> q ) = ( ( p   q ))

;- W-3
;;   Rciproque de l'implication de Philon.
;$   Kehrwert von Philo's Folgerung. "Wenn q, so p" [ibid.]
;+   Reciprocal of Philo's implication. "If q, so p" [ibid.]

(defun W-3 (p q)                                         ;"si q, soit p" [ibid.]
  (not (and q (not p))))                           ;( q -> p ) = ( ( q   p ))

;- W-4
;;   Ou, primitive OR.
;$   Oder, OR Primitive. "p oder q" [ibid.]
;+   OR primitive. "p or q" [ibid.]

(defun ou (p q)                                                ;"p ou q" [ibid.]
  (or p q))                                                           ;( p v q )

;- W-5
;$ "Nicht p" [ibid.]
;+ "Not p" [ibid.]

(defun W-5 (p q) (declare (ignore q))                       ;"non pas p" [ibid.]
  (not p))                                                              ;(  p )

;- W-6
;$ "Nicht q" [ibid.]
;+ "Not q" [ibid.]

(defun W-6 (p q) (declare (ignore p))                       ;"non pas q" [ibid.]
  (not q))                                                              ;(  q )

;- W-7
;;   OU exclusif (XOR).
;$   Exklusives ODER (XOR). "p, oder q, aber nicht beide" [ibid.]
;+   Exclusive OR (XOR). "p, or q, but not both" [ibid.]

(defun xor (p q)                         ;"p ou q mais non pas les deux" [ibid.]
  (or (and p (not q));                               (( p   q ) v ( q   p ))
      (and q (not p))))

;- W-8
;$ "Wenn p, so q, und wenn q, so p" [ibid.]
;+ "If p, so q, and if q, so p" [ibid.]

(defun W-8 (p q)                     ;"si p, ainsi q ; et si q, ainsi p" [ibid.]
  (eq p q))                                                           ;( p = q )

;- W-9
;$ "q" [ibid.]
;+ "q" [ibid.]

(defun W-9 (p q) (declare (ignore p))                               ;"q" [ibid.]
  q)                                                                      ;( q )

;- W-A
;$ "p" [ibid.]
;+ "p" [ibid.]

(defun W-A (p q) (declare (ignore q))                               ;"p" [ibid.]
  p)                                                                      ;( p )

;- W-B
;$ "Weder p noch q" [ibid.]
;+ "Neither p nor q" [ibid.]

(defun W-B (p q)                                            ;"ni p ni q" [ibid.]
  (and (not p) (not q)))                                      ;((  p)  (  q))

;- W-C
;$ "q und nicht p" [ibid.]
;+ "q and not p" [ibid.]

(defun W-C (p q)                                       ;"q et non pas p" [ibid.]
  (and q (not p)))                                                ;( q  (  p))

;- W-D
;$ "p and not q" [ibid.]
;+ "p and not q" [ibid.]

(defun W-D (p q)                                       ;"p et non pas q" [ibid.]
  (and p (not q)))                                                ;( p  (  q))

;- W-E
;;   Et, primitive AND.
;$   Und, AND Primitive. "q und p" [ibid.]
;+   AND primitive. "q and p" [ibid.]

(defun et (p q)                                                ;"q et p" [ibid.]
  (and q p))                                              ;( q  p ) = ( p  q )

;- W-F
;;   Contradiction,  une et deux propositions lmentaires (ou termes) :
;; rsultats toujours NIL. Exemples, que <p> et <q> soient indiffremment T ou
;; NIL :
;$   Widerspruch, mit einen oder zwei elementaren Stzen (oder Ausdrcken):
;$ Ergebnis ist immer NIL. Beispiele, bei denen <p> und <q> T oder
;$ NIL sein knnen:
;+   Contradiction, with one or two elementary propositions (or terms): results
;+ always NIL. Examples, <p> and <q> being indifferently T or NIL:
;-    (contra1 <p>) => NIL ; (contra2 <p> <q>) => NIL

(defun contra1 (2b)
  (and 2b (not 2b)))                                    ;To be, AND not to be...

;$ "p und nicht p; und q und nicht q" [ibid.]
;+ "p and not p; and q and not q" [ibid.]
(defun contra2 (p q)               ;"p et non pas p ; et q et non pas q" [ibid.]
  (and (contra1 p) (contra1 q)))                      ;(( p   p)  ( q   q))

;;; Lois de De Morgan
;$$ Gesetze von De Morgan
;++ De Morgan's Laws

;- p  q =  (( p) v ( q)) = (not (or (not p) (not q)))

(defun Morgan-1 (p q)
  (not (or (not p) (not q))))

;- p v q =   (( p)  ( q)) = (not (and (not p) (not q)))

(defun Morgan-2 (p q)
  (not (and (not p) (not q))))

;;; Primitive de contrle : if
;$$ Kontrolprimitive: if
;++ Control Primitive: if

;;   Prdicat rendant <n> ou NIL selon que sont argument d'appel <n> est un
;; nombre ou non. Un cho de <n> peut tre plus utile que T.
;$   Prdikat, das gibt <n> oder NIL zurck, je nachdem ob sein Aufrufargument
;$ <n> eine Zahl ist oder nicht. Die Rckgabe von <n> drfte ntzlicher sein
;$ als T.
;+   Predicate giving <n> or NIL according to whether its calling argument <n>
;+ is a number or not. An echo of <n> may more useful than T.

(defun nbr? (n)
  (if (numberp n) n
  nil))

;;   Prdicat examinant si le premier lment d'une liste est ou non un nombre.
;; Premire version inutilement verbeuse... au point de se priver de sa fonction
;; logique de prdicat !
;$   Prdikat, das prft, ob das erste Element einer Liste eine Zahl ist
;$ oder nicht. Erste Version, unntig viel Rckmeldung... so viel, dass es
;$ die logische Verwendung des Prdikats verhindert!
;+   Predicate testing whether the first element of a list is a number or not.
;+ First version, uselessly verbose... to the point of preventing its logical
;+ use as a predicate!

(defun KopfNummer (li)                                  ;verbeux et peu utile...
  (if (numberp (car li))
    (append '(Jawohl) (list (car li)) '(ist doch eine schne Nummer...))
  (cons (car li) '(ist leider keine Nummer... Es tut mir leid...))))

;;   Seconde version, concise et logiquement utile : rend le premier nombre de
;; <li> ou NIL.
;$   Zweite Version, kurz und als logischer Ausdruck brauchbar: gibt die erste
;$ Zahl von <li> oder NIL zurck.
;+   Second version, terse and logically useful: outputs the first number of
;+ <li> or NIL.

(defun KNum (li)                                       ;style prdicat, sobre et
  (if (numberp (car li)) (car li)                      ;plus "professionnel"
  nil))

;;; Imbrication de deux ou trois if, et variations
;$$ Verschachtelung von zwei oder mehr if, und Variationen
;++ Nesting of Two or More if, and Variations

;;   Exemple testant la position d'un argument <a> dans l'arbre syntaxique LISP.
;; Cf. Chap. 2 et [Wertz:26]. Lisibilit moyenne.
;$   Das Beispiel prft die Position des Arguments <a> in einem LISP Syntaxbaum.
;$ Vglt Kap. 2 und [Wertz:26]. Mittelprchtige Lesbarkeit.
;+   Example testing the position of an argument <a> in the LISP syntactic tree.
;+ Cf. Chap. 2 and [Wertz:26]. Moderate legibility.

(defun type? (a)
  (if (numberp a)  'nombre
      (if (atom a) 'symbole
      'liste)))

;;   Prdicat examinant si les trois premiers lments d'une liste sont ou non
;; des nombres. Il s'agit d'un enchanement de trois clauses obligatoires. Une
;; premire version, suivie de quatre variations possibles. Cf. [Wertz:26,267].
;$   Prdikat, das untersucht, ob die ersten drei Elemente einer Liste Zahlen
;$ sind oder nicht. Besteht aus einer Folge dreier obligatorischer
;$ Bedingungen. Eine erste Version, gefolgt von vier mglichen Variationen.
;$ Vgl. [Wertz:26,267].
;+   Predicate examining whether the first three elements of a list are numbers
;+ or not. Consists in a sequence of three compulsory clauses. A first version,
;+ followed by four possible variations. Cf. [Wertz:26,267].

(defun 3nbrp1 (li)                                                ;moins lisible
  (if (numberp (car li))
    (if (numberp (cadr li))
      (if (numberp (caddr li)) li
      nil)
    nil)
  nil))

(defun 3nbrp2 (li)                                      ;plus lisible, mais plus
  (if (not (numberp (car li)))   nil                    ;lourd. Cf. loi de
  (if (not (numberp (cadr li)))  nil                    ;De Morgan
  (if (not (numberp (caddr li))) nil
  li))))

(defun 3nbrp3 (li)                                     ;lisible et conomique...
  (if (and (numberp (car li))
           (numberp (cadr li))
           (numberp (caddr li)))
      li
  nil))

(defun 3nbrp3a (li)                                   ;... et mme davantage !
  (and (numberp (car li))
       (numberp (cadr li))
       (numberp (caddr li))
       li))

(defun 3nbrp4 (li)                                    ;symtrique de la version
  (if (or (not (numberp (car li)))                    ;3nbrp3, sans doute un peu
          (not (numberp (cadr li)))                   ;trop alambique... Cf.
          (not (numberp (caddr li))))                 ;loi de De Morgan
      nil
  li))

;;   Rtrograde trs restreinte des trois premiers lments d'une liste de zro,
;; un, deux ou trois lments. Imbrication de deux if. Cf. [Wertz:28].
;; Lisibilit moyenne.
;$   Sehr eingeschrnkte Umkehrung der ersten drei Elemente einer Liste mit
;$ 0-3 Elementen. Zwei verschachtelte if. Vgl. [Wertz:28]. Mittelprchtige
;$ Lesbarkeit.
;+   Very restricted retrograde of the first three elements of a list of zero,
;+ one, two or three elements. Two nested if. Cf. [Wertz:28]. Moderate
;+ legibility.

(defun retro1 (li)
  (if (endp (cdr li)) li
      (if (endp (cddr li)) (cons (cadr li) (cons (car li) ()))
      (cons (caddr li) (cons (cadr li) (cons (car li) ()))))))

;;   Ralisation d'un tat de connaissance embryonnaire : la variable nomme
;; pour le paramtre <kumul> accumulera graduellement des "connaissances" au
;; fur et  mesure de sa confrontation avec des valeurs de l'argument <item>.
;; Exemple de squence d'appels :
;$   Umsetzung eines embrionalen Wissensstandes: die Variable, die fr den
;$ <kumul>-Parameter bestimmt wurde, wird schrittweise etwas "Wissen" ansammeln
;$ whrend sie mit den Werten des Arguments <item> konfrontiert wird. 
;$ Beispiel einer Aufrufsequenz:
;+   Realization of an embryonic knowledge state: the variable named for the
;+ <kumul> parameter will gradually cumulate some "knowledge" as it will be
;+ confronted to the values of the argument <item>. Example of calling sequence:
;-    (setq bekannt nil) => nil
;-    (kum1 36 'bekannt) => (36)
;-    (kum1 55 'bekannt) => (55 36)
;-    (kum1 36 'bekannt) => (55 36) ...

(defun schon (item kumul)                                    ;fonction effective
  (if (member item kumul :test #'equal) kumul
  (cons item kumul)))
(defun kum1 (it kk)                                          ;fonction interface
  (set kk (schon it (eval kk))))

;;; Primitive de contrle : when
;$$ Kontrollprimitive: when
;++ Control Primitive: when

;;   Seconde version de l'tat de connaissance prcdent, en un style prdicat
;; plus efficace. Exemple de squence d'appels :
;$   Zweite Version des vorherigen Wissensstandes, mit effizienterem Aufrufstil.
;$ Beispiel einer Aufrufsequenz:
;+   Second version of the preceding knowledge state, in a more efficient
;+ predicate style. Example of calling sequence:
;-    (setq bekannt nil) => nil
;-    (kum2 36 'bekannt) => (36)
;-    (kum2 55 'bekannt) => (55 36)
;-    (kum2 36 'bekannt) => NIL ...

(defun noch-nicht (item kumul)                              ;prdicat spcialis
  (not (member item kumul :test #'equal)))
(defun kum2 (it kk)                                          ;fonction interface
  (when (noch-nicht it (eval kk)) (set kk (cons it (eval kk)))))

;;; Primitive de contrle : cond
;$$ Kontrollprimitive: cond
;++ Control Primitive: cond

;;   Autre version de l'exemple type? ci-dessus, avec ajout d'une distinction
;; entre deux catgories de nombres : entier ou autre. Lisibilit suprieure.
;$   Andere Version des obigen type?-Beispiels mit zustzlicher Unterscheidung
;$ zwischen zwei Zahlenarten: Ganzzahlen und anderen. Verbesserte Lesbarkeit.
;+   Other version of the type? example above, with additional distinction
;+ between two number categories: interger or other. Improved legibility.

(defun type?? (a)
  (cond ((numberp a) (if (integerp a) 'entier
                     'autre_nombre))
        ((atom a)    'symbole)
        (t           'liste)))

;;   Autre variation de l'exemple 3nbrp ci-dessus, sans avantage particulier.
;$   Weitere Variante des obigen 3nbrp-Beispiels; kein besonderer Vorteil.
;+   Further variation of the 3nbrp example above; no particular improvement.

(defun 3nbrp5 (li)
  (cond ((not (numberp (car li)))   nil)
        ((not (numberp (cadr li)))  nil)
        ((not (numberp (caddr li))) nil)
        (t                          li )))

;;   Seconde version de l'algorithme de rtrogradation ci-dessus, avec ajout de
;; deux clauses de rejet ventuel de la liste <li>. La lisibilit reste
;; parfaite, malgr l'enchanement de cinq clauses alternatives.
;$   Zweite Version des obigen Umkehrungsalgorithmus mit zwei zustzlichen
;$ Abfragen um die <li> Liste eventuell abzulehnen. Perfekte Lesbarkeit wurde
;$ erhalten -- trotz einer Folge fnf unterschiedlicher Bedingungen.
;+   Second version of the retrogradation algorithm above, with two additional
;+ clauses eventually rejecting the <li> list. Perfect legibility is preserved,
;+ in spite of a sequence of five alternate clauses.

(defun retro2 (li) (cond
  ((not (listp li))       'pas_une_liste)
  ((> (list-length li) 3) 'trop_longue_liste)
  ((endp (cdr li))        li)
  ((endp (cddr li))       (list (cadr li) (car li)))
  (t                      (list (caddr li) (cadr li) (car li)))))
