;-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;-
;- Fichier ped.10.cl, NON DISTRIBU : pour servir d'exemples
;-                     UTILISER MANUELLEMENT...
;-
;-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;- Cette fonction imprime une *  la position horizontale donne par l'argument
;- d'appel <num>, dans [-1,1]. Par exemple :
;-    (ligne -1.0 :max 46) => "*                                           "
;-    (ligne  0.0 :max 46) => "                    *                       "
;-    (ligne  1.0 :max 46) => "                                           *"
;- La valeur par dfaut 79 du mot-clef :max est prvue pour une fentre de 80
;- caractres de largeur.

;Premire dfinition : deux inconvnients. [1] Les lignes courtes (avec "*"
;plutt  gauche) sont imprimes plus vite que les longues (avec "*" plutt
; droite) => impression irrgulire des fonctions chantillonnes...
;[2] Plus lente que la version ci-dessous.
;(defun ligne (num &key (max 79))
;  (dotimes (x (round (+ (* (+ num 1.0) (/ max 2.0)) (/ (- 79.0 max) 2.0)))
;              (format t "*~%"))
;    (format t " ")))
;Deuxime dfinition, mieux que ci-dessus.
(defparameter chaine (make-string 80 :initial-element #\ ))
(defparameter bullules (make-array 12 :initial-contents '(#\ #\. #\o #\O #\ #\ #\ #\O #\ #\ #\@ #\:)))
(defun ligne (num &key (max 79) (bulles nil) &aux indx)
  (setq indx (round (+ (* (+ num 1.0) (/ max 2.0)) (/ (- 79.0 max) 2.0))))
  (if (> indx 79) (incf indx -79)
      (when (minusp indx) (incf indx 79)))
  (setf (aref chaine 40) #\)
  (setf (aref chaine indx) (if bulles (aref bullules (random 12)) #\*))
  (format t "~A~%" chaine)
  (mp::process-wait-with-timeout "pour ralentir...secs" 0.003)     ;<--LispWorks
  ;(dotimes (n 2000) (eval t))            ;perte de temps... <-- RGLAGE VITESSE
  (setf (aref chaine indx) #\ ) 'ok)

;- Bon pour exemples de trigo, mais surtout du principe d'chantillonnage...
;- Exemples suivants avec (sin <radians>), mise en oeuvre sous la forme
;- (sin (incf <angle> <pas>)), ou (sin (incf <angle> (freq <Hz>))).
;- N.B. : 360 degrs = 2*pi radians.

;- Calcul des <pas> angulaires  partir d'une frquence <f>, par rapport  un
;- taux d'chantillonnage de 32000 ch./sec. (plus rond que 44100...).
(defun freq (f)
  (/ (* 2.0 pi f) 32000.0))
;- N.B. : priode, en chantillons = (/ (* 2 pi) <pasN>)
;-                                 = (/ (* 2 pi) (freq <Hz>))

;- N.B. : cliquer pour figer le dfilement.

;; Utilisation type, version complte et laborieuse :
(defparameter angle (- (freq 1230)))                 ;=> environ 26 ch./priode
;    (continu (ligne (sin (incf angle (freq 1230))) :max 40))

;; Ou encore, avec modulation d'amplitude ou ring-modulation :
(defparameter pas1 (freq 100))                    ;modulante lente
(defparameter pas2 (freq 1140))                   ;porteuse plus rapide
(defparameter ang1 (- pas1))
(defparameter ang2 (- pas2))
;2 sinus multiplis
;    (continu (ligne (* (sin (incf ang1 pas1))(sin (incf ang2 pas2)))))

;- On peut redfinir ensuite des valeurs, par exemple <pas1>, et recommencer
;- avec de nouveaux appels de (continu... ci-dessus :
;    (defparameter pas1 (freq 20))       ;modulante trs lente
;    (defparameter pas1 (freq 440))      ;    "    assez rapide
;    (defparameter pas1 (freq 2345))     ;    "    rapide
;    (defparameter pas1 (freq 5399))     ;    "    encore plus rapide

;- On peut aussi faire des exemples de sous-chantillonnage, repliement, etc.
;- Formule simplifie pour exemples du principe d'chantillonnage, avec
;- macro fonction g :
;-    => g <freq> &optional (maxi 40), et c'est tout !!
(defmacro g (f &optional (maxi 40))
  (setq chaine (make-string 80 :initial-element #\ ))
  `(continu (ligne (sin (incf angle (freq ,f))) :max ,maxi)))
;- ATTENTION : affichages trompeurs quand on est plutt prs de 16000 (moiti
;- du taux d'chantillonnage) : effets de dphasages divers entre la frquence
;-  reprsenter et celle du taux d'chantillonnage. Mais a marche.
;    (g 15000)                   ;un peu plus de 2 ch./pr.
;- ou
;    (g 16000)                   ;2 ch./pr. : pas = pi
;    (g 32000)                   ;1 pr./ch. : pas = 2*pi
;    (g 32205)                   ;0.9968 ch./pr., pas = (2*pi)+0.04

;- 3 frquences "alias" : 1140, 30860, 33140
;    (g 1140)                    ;frquence = 1140 = Te/2 - 14860
;    (g 30860)                   ;frquence = Te/2 + 14860
;    (g 33140)                   ;frquence = Te + 1140
;- etc...

;- Idem, exemples d'crtage/dbordement, avec mot-clef :max > 79 :
;    (g 1645 85)                                   ;crtage/overflow
;N.B. :  cause d'adresses hors bornes pour la primitive aref, dans la fonction
;ligne, ceci ne marche qu'avec max <= 238 = (3*79)+1.

;- Exemples mathmatiques aussi possibles avec (abs... (cos...
;- Idem, exemples avec des canons stochastiques :
(load (merge-pathnames "canons_tous.lisp" common-lisp::*path-stochastique*))
;    (continu (ligne (gauss 0.0 0.15) :bulles t)) ;diffrents verres de champagne...
;    (continu (ligne (gauss 0.0 0.33) :bulles t))
;    (continu (ligne (plapla 0.0 0.15) :bulles t))
;    (continu (ligne (corect -1.0 1.0) :bulles t));plutt Perrier (verre  fond plat)
;    (continu (ligne (corect 0.1 0.6) :bulles t));plutt Perrier localis !
;    (continu (ligne (beta 0.25 0.25) :bulles t));"1/2 verre" + bulles prs parois !
;    (continu (ligne (cobeta 0.25 0.25 -1.0 1.0) :bulles t)) ;plein verre idem.
;- etc...

;- Squences quasi-alatoires (cf. stoch.d/QuAlea)
;    (dolist (le (QuAleaCorput 10 -1 1 :permut t) 'ok) (ligne le))
;    (dolist (le (QuAleaCorput 10 -1 1) 'ok) (ligne le))
;    (dolist (le (QuAleaEq 10 -1 1) 'ok) (ligne le))
;    (dolist (le (QuAleaEq 10 -1 1 :permut nil) 'ok) (ligne le))