;===============================================================================
;;; 11 --> DE LA RCURSIVIT, PAR L'EXEMPLE :
;;;        11.1 - Quelques cas mathmatiques simples
;$$ 11 --> REKURSIVITT MIT BEISPIELE:                                [d..H.B.]
;$$        11.1 - Einige einfache mathematische Flle
;++ 11 --> OF RECURSIVITY THROUGH EXAMPLES:
;++        11.1 - A Few Simple Mathematical Cases
;===============================================================================

;;   Remarque : selon les implmentations de Common LISP, la forme
;$   Anmerkung: in bestimmten Common LISP Implementierungen ist der Ausdruck
;+   Note: in certain Common LISP implementations, the form
;-    (declare (notinline <fonc>))
;; est parfois ncessaire afin d'obtenir une trace de tous les niveaux
;; d'appels rcursifs d'une fonction <fonc> [CLtL2:229-230].
;;   Cette forme est intgres aux dfinitions des exemples uniquement dans le
;; prsent fichier. En cas de besoin, elle devrait tre ajoute aux dfinitions
;; contenues dans d'autres fichiers.
;$ ntig, um eine Ablaufverfolgung (engl. trace) aller Rekursionsebenen einer
;$ Funktion <fonc> zu erhalten [CLtL2:229-230].
;$   Dieser Ausdruck ist in den Beispieldefinitionen in dieser Datei enthalten.
;$ Er sollte falls ntig den Definitionen in anderen Dateien hinzugefgt
;$ werden.
;+ must be included to obtain a trace of all levels of recursive calls of a
;+ <fonc> function [CLtL2:229-230].
;+   This form is included in the example definitions only in the present
;+ file. If necessary, it should be added to the definitions included in other
;+ files.

;;   Deux dfinitions itratives (non-rcursives) possibles de <m>**<n> :
;; <m> exposant <n>.
;$   Zwei mgliche iterative (nicht-rekursive) Definitionen von <m>**<n>:
;$ <m> Exponent <n>.
;+   Two possible iterative (non-recursive) definitions of <m>**<n>:
;$ <m> exponent <n>.

(defun pow-1 (n m &aux (Ergb 1))
  (dotimes (x m Ergb)
    (setq Ergb (* Ergb n))))

(defun pow-2 (n m)
  (eval (cons '* (make-list m :initial-element n))))

;;   <m>**<n> : <m> exposant <n>. Dfinition rcursive. quivalente LISP :
;; (expt <m> <n>).
;$   <m>**<n>: <m> Exponent <n>. Rekursive Definition. LISP-Equivalent:
;$ (expt <m> <n>).
;+   <m>**<n>: <m> exponent <n>. Recursive definition. LISP equivalent:
;+ (expt <m> <n>).

(defun pow1 (m n) (declare (notinline pow1))
  (cond ((zerop n) 1)
        (t         (* m (pow1 m (- n 1))))))

(defun pow2 (m n) (declare (notinline pow2))
  (if (zerop n) 1
  (* m (pow2 m (1- n)))))

(defun pow3 (m n)              ;Interface de pow2, avec vrification de <n>,
    (if (integerp n) (pow2 m n);pour la scurit de la condition terminale zerop
    (error "pow3 : ~A : l'exposant n'est pas entier." n)))

;;   Deux dfinitions itratives (non-rcursives) possibles de <n>! :
;; <n> factorielle.
;; N.B. : pour fact-2, la fonction range doit tre dfinie selon l'environnement
;; DLO. Par exemple :
;$   Zwei mgliche iterative (nicht-rekursive) Definitionen von <n>!:
;$ <n> Fakultt.
;$ N.B.: fr fact-2 die range Funktion nach der DLO Umgebung definiert werden
;$ muss. Zum Beispiel:
;+   Two possible iterative (non-recursive) definitions of <n>!: <n> factorial.
;+ N.B.: for fact-2, the range function must be defined according to the DLO
;+ environment. For instance:
;-    (range 1 6) => (1 2 3 4 5 6)

(defun fact-1 (n &aux (Ergb 1))
  (dotimes (x n Ergb)
    (setq Ergb (* Ergb (1+ x)))))

(defun fact-2 (n)
  (eval (cons '* (range 1 n))))

;;   <n>! : <n> factorielle. Dfinition rcursive.
;$   <n>!: <n> Fakultt. Rekursive Definition.
;+   <n>!: <n> factorial. Recursive definition.

(defun fact1 (n) (declare (notinline fact1))
  (cond ((zerop n) 1)
        (t         (* n (fact1 (- n 1))))))

(defun fact2 (n) (declare (notinline fact2))
  (if (zerop n) 1
  (* n (fact2 (1- n)))))

(defun fact3 (n)               ;Interface de fact2, avec vrification de <n>,
  (if (integerp n) (fact2 n)   ;pour la scurit de la condition terminale zerop
  (error "fact3 : ~A : n'est pas entier." n)))

;;   Srie de Fibonacci :
;;    fibo1 <n> => <n>-ime terme de la srie de Fibonacci
;$   Fibonacci-Folge:
;$    fibo1 <n> => <n>-tes Folgeglied der Fibonacci-Folge
;+   Fibonacci series:
;+    fibo1 <n> => <n>-th term of the Fibonacci series

(defun fibo1 (n) (declare (notinline fibo1))
  (cond ((zerop n) 1)
        ((= n 1)   1)
        (t         (+ (fibo1 (- n 1))
                      (fibo1 (- n 2))))))

(defun fibo2 (n)               ;Interface de fibo1, avec vrification de <n>,
  (if (integerp n) (fibo1 n)   ;pour la scurit de la condition terminale zerop
  (error "fibo2 : ~A : n'est pas entier." n)))

;;   Srie de Padovan [Stewart, 2000 : <L'univers des nombres> : 19].
;;    pado1 <n> => <n>-ime terme de la srie de Padovan
;$   Padovan-Reihe [Stewart, 2000: <L'univers des nombres>: 19].
;$    pado1 <n> => <n>-tes Folgeglied der Padovan-Reihe
;+   Padovan series [Stewart, 2000: <L'univers des nombres>: 19].
;+    pado1 <n> => <n>-th term of the Padovan series

(defun pado1 (n) (declare (notinline pado1))
  (cond ((zerop n) 1)
        ((= n 1)   1)
        ((= n 2)   1)
        (t         (+ (pado1 (- n 2))
                      (pado1 (- n 3))))))

(defun pado2 (n)               ;Interface de pado1, avec vrification de <n>,
  (if (integerp n) (pado1 n)   ;pour la scurit de la condition terminale zerop
  (error "pado2 : ~A : n'est pas entier." n)))

;;   "[...] En premier lieu, il a spar du mlange total une portion. Ensuite
;; il a pris une seconde portion double de celle-l ; puis une troisime portion
;; egale  une fois et demie la seconde et  trois fois la premire ; une
;; quatrime double de la seconde ; une cinquime triple de la troisime ; une
;; sixime gale  huit fois la premire ; une septime gale  vingt-sept fois
;; la premire. [...]" [Platon : Time ; in Verdet, J.-P., 1993 : Astronomie et
;; astrophysique, Larousse : 25-26].
;;   Srie 1, 2, 3, 4, 9, 8, 27, 16... alternant des puissances de 2 et de 3.
;; Exploite seulement jusqu' 27 -- (timee 7) -- dans la cosmogonie de Platon,
;; bien qu'il mentionne aussi le rapport 256/243 = (/ (timee 16) (timee 11)).
;; D'autre part :
;;    (+ (timee 1) (timee 2) ... (timee 6)) = (timee 7) = 27
;$   "[...] Einen Teil nahm er als erstes vom Ganzen fort, nach diesem entnahm
;$ er das Doppelte davon, den dritten anderthalb mal so gro wie den zweiten --
;$ also dreimal so gro wie der erste --, den vierten doppelt so gro wie den
;$ zweiten, den fnften dreimal so gro wie den dritten, den sechsten achtmal so
;$ gro wie den ersten, den siebenten 27mal so gro wie den ersten. [...]"
;$ [Platon: Timaios; ZEKL, Hans Gnter (Herausgabe, bersetzung usw.), 1992,
;$ Philosophische Bibliothek B. 444, Hamburg, F. Meiner Verlag: 43].
;$   Reihe 1, 2, 3, 4, 9, 8, 27, 16... alternierende 2er- und 3er-Potenzen.
;$ In seiner Kosmogonie verwendet Platon sie nur bis 27 -- (timee 7) --, obwohl
;$ er auch das Verhltnis 256/243 = (/ (timee 16) (timee 11)) erwhnt. Auerdem:
;$    (+ (timee 1) (timee 2) ... (timee 6)) = (timee 7) = 27
;+   "[...] And he proceeded to divide after this manner: - First of all, he
;+ took away one part of the whole (1), and then he separated a second part
;+ which was double the first (2), and then he took away a third part which was
;+ half as much again as the second and three times as much as the first (3),
;+ and then he took a fourth part which was twice as much as the second (4), and
;+ a fifth part which was three times the third (9), and a sixth part which was
;+ eight times the first (8), and a seventh part which was twenty-seven times
;+ the first (27)." [...]" [Plato: Timaeus, translated by Benjamin Jowett, cf.
;+ http://classics.mit.edu/Plato/timaeus.html].
;+   Series 1, 2, 3, 4, 9, 8, 27, 16... alternating powers of 2 and of 3. Used
;+ only through 27--(timee 7)-- in Plato's cosmogony, although he also mentions
;+ the ratio 256/243 = (/ (timee 16) (timee 11)). Moreover:
;+    (+ (timee 1) (timee 2) ... (timee 6)) = (timee 7) = 27
;-   Pour obtenir la srie : / Um die Reihe zu erhalten: / To obtain the Series:
;-    (let ((n 0)) (pasapas (print (timee (incf n)))))

(defun timee (n) (declare (notinline timee))
  (cond ((not (integerp n)) (error "timee : ~A : n'est pas entier." n))
        ((< n 3)            n)
        (t                  (* (timee (- n 2)) (if (evenp n) 2 3)))))

;;   Nombres figurs pythagoriciens. "Ces nombres polygonaux sont obtenus en
;; sommant les termes d'une progression arithmtique dont le premier terme vaut
;; 1. La raison de cette progression est 1 pour les triangles, 2 pour les
;; carrs, 3 pour les pentagones, etc." [DAHAN-DALMEDICO, Amy, & PEIFFER,
;; Jeanne, 1986 : Une histoire des mathmatiques, Seuil, Coll. Points sciences :
;; 49].
;;   On a toujours, quel que soit <r>,
;$   Pythagoreische figurale Zahlen. "Diese Polygonalzahlen erhlt man durch
;$ Addition der Elemente einer arithmetischen Folge, bei welcher das erste
;$ Element 1 ist. Die Konstante dieser Folge ist 1 fr Dreiecke, 2 fr
;$ Quadrate, 3 fr Fnfecke etc." [DAHAN-DALMEDICO, Amy, & PEIFFER, Jeanne,
;$ 1986: Une histoire des mathmatiques, Seuil, Coll. Points sciences: 49].
;$   Es gilt immer, fr jedes <r>,
;+   Pythagorean figured numbers. "These polygonal numbers are obtained by
;+ summing the terms of an arithmetic progression of which the first term is 1.
;+ The constant of this progression is 1 for the triangles, 2 for the squares, 3
;+ for the pentagons etc." [DAHAN-DALMEDICO, Amy, & PEIFFER, Jeanne, 1986: Une
;+ histoire des mathmatiques, Seuil, Coll. Points sciences: 49].
;+   One has always, for any <r>,
;-    (NF 0 <r>) = 1
;;   La fonction suivante rend le <n>-ime nombre de la srie dtermine par la
;; raison <r>, c'est--dire d'un polygone de <r>+2 cts. Donc, selon <r> :
;;    1  => triangulaires
;;    2  => carrs, possdant deux caractristiques particulires :
;;            - un nombre carr est la somme de deux nombres triangulaires :
;$   Die folgende Funktion gibt das <n>-te Element der Folge zurck, welche
;$ durch die Konstante <r> bestimmt wurde, also eines Polygon mit <r>+2 Seiten.
;$ Daher gilt in Bezug auf <r>:
;$    1  => Dreiecke
;$    2  => Quadrate, die die folgenden beiden Merkmale aufweisen:
;$            - eine Quadratzahl ist die Summe zweier Dreieckszahlen:
;+   The following function returns the <n>-th term of the series determined by
;+ the constant <r>, namely of a polygon of <r>+2 sides. Thus, according to <r>:
;+    1  => triangulars
;+    2  => squares, displaying two particular characteristics:
;+            - a square number is the sum of two triangular numbers:
;-                (NF <n> 2) = (+ (NF <n> 1) (NF (1- <n>) 1))
;;            - le nombre carr <n> est la somme du nombre carr prcdent et
;;              d'un "gnomon" (querre) de 1+2<n> :
;$            - die Quadratzahl <n> ist die Summe der vorhergehenden Quadratzahl
;$              und eines "gnomon" (Winkelma) von 1+2<n>:
;+            - the square number <n> is the sum of the preceding square number
;+              and of a "gnomon" (setsquare) of 1+2<n>:
;-                (NF <n> 2) = (+ (NF (1- <n>) 2) 1 (* 2 <n>))
;;    3  => pentagonaux
;;    4  => hexagonaux
;;    5  => heptagonaux
;;    6  => octogonaux
;; et en gnral
;;    <r> => "<r>+2"-gonaux
;;   Exemple d'valuation test pour obtenir la srie de nombres "<r>+2"-gonaux :
;$    3  => Fnfecke (Pentagon)
;$    4  => Sechsecke (Hexagon)
;$    5  => Siebenecke (Heptagon)
;$    6  => Achtecke (Oktogon)
;$ und im allgemeinen
;$    <r> => "<r>+2"-gon
;$   Anwendungsbeispiel um eine Folge "<r>+2"-gonaler Zahlen zu erhalten:
;+    3  => pentagonals
;+    4  => hexagonals
;+    5  => heptagonals
;+    6  => octogonals
;+ and generally
;+    <r> => "<r>+2"-gonals
;+   Example test evaluation to obtain the series of "<r>+2"-gonal numbers:
;-    (progn (setq n -1) (pasapas (print (NF (incf n) <r>))))

(defun NF (n r)
  (if (= 0 n) 1
  (+ (NF (1- n) r) 1 (* r n))))

;;   Axiomatique des nombres entiers naturels... Fonction inutile, s'il en fut,
;; mais  propos dmonstratif.
;;    0   := 0         := "quand 'ya pas, 'ya pas"
;;    <n> := (<n>-1)+1 := "quand 'ya, c'est un de plus qu'un de moins"
;; Exemple, avec trace active :
;$   Axiomatik der natrlichen Ganzzahlen... Nutzlose aber anschauliche
;$ Funktion.
;$    0   := 0         := "nichts ist nichts"
;$    <n> := (<n>-1)+1 := "etwas ist eins mehr als eins weniger"
;$ Beispiel, mit Trace aktiviert:
;+   Axiomatics of natural integer numbers... Useless, albeit demontrative
;+ function.
;+    0   := 0         := "naught's naught"
;+    <n> := (<n>-1)+1 := "if there's something, it's one more than one less"
;+ Example, with trace on:
;-    (nen (+ (nen 6) (nen 7)))

(defun nen (n) (declare (notinline nen))
  (if (zerop n) 0
  (1+ (nen (1- n)))))