;===============================================================================
;;; 11 --> DE LA RCURSIVIT, PAR L'EXEMPLE :
;;;        11.2a - Analyses de listes
;$$ 11 --> REKURSIVITT MIT BEISPIELEN:
;$$        11.2a - Analyse von Listen                                 [d..H.B.]
;++ 11 --> OF RECURSIVITY THROUGH EXAMPLES:
;++        11.2a - List Analysis
;===============================================================================

;;    atomlist <li> => t si, et seulement si <li> est constitue d'atomes.
;$    atomlist <li> => t wenn, und nur wenn <li> aus Atomen besteht.
;+    atomlist <li> => t if, and only if <li> is made out of atoms.

(defun atomlist (li)
  (cond ((endp li)       t)                              ;cas trivial
        ((atom (car li)) (atomlist (cdr li)))            ;confirmer la suite...
        (t               nil)))                          ;FAUX immdiat

;;    lmidip <li> => t si, et seulement si <li> est constitue de MIDI pitches.
;; N.B. : suppose la dfinition de midip (cf. fichier exo.10.0, qui contient
;; aussi une dfinition plus concise nomme lmidip2).
;$    lmidip <li> => t wenn, und nur wenn <li> aus MIDI-Tonhhen besteht.
;$ N.B.: setzt die Definition von midip voraus (Vgl. Datei exo.10.0, die auch
;$ eine krzere Definition namens lmidip2 enthlt).
;+    lmidip <li> => t if, and only if <li> is made out of MIDI pitches.
;+ N.B.: presumes the definition of midip (cf. exo.10.0 file, which also
;+ includes a shorter definition named lmidip2).

(defun lmidip1 (li)
  (cond ((endp li)        t)                             ;cas trivial
        ((midip (car li)) (lmidip1 (cdr li)))            ;confirmer la suite...
        (t                nil)))                         ;FAUX immdiat

;;    membre <it> <li> => rend le cdr de <li> commenant par <it> s'il existe,
;;                        ou nil autrement. quivalente LISP : member, avec
;;                        :test #'equal
;$    membre <it> <li> => gibt das cdr von <li> beginnend mit <it> zurck, falls
;$                        es existiert, sonst nil. LISP-Equivalent: member mit
;$                        :test #'equal
;+    membre <it> <li> => returns the cdr of <li> beginning by <it>, if it
;+                        exists, or nil otherwise. LISP equivalent: member,
;+                        with :test #'equal

(defun membre1 (it li)
  (cond ((endp li)           nil)                        ;cas trivial
        ((equal it (car li)) li)                         ;VRAI immdiat
        (t                   (membre1 it (cdr li)))))    ;confirmer la suite...

(defun membre2 (it li)                                   ;autre version, plus
  (if (or (endp li) (equal it (car li))) li              ;concise, mais beaucoup
  (membre2 it (cdr li))))                                ;moins lisible

;;    queue <s> <li> => rend <s> s'il est l'un des cdr de <li> ; nil autrement
;;                      Apparente LISP : tailp, plus restreinte
;$    queue <s> <li> => gibt <s> zurck, falls es eines der cdr von <li> ist,
;$                      sonst nil. hnlich in LISP: tailp, eingeschrnkter
;+    queue <s> <li> => retruns <s> if it is one of the cdr of <li>; nil
;+                      otherwise. Related LISP: tailp, more restricted

(defun queue (s li)
  (cond ((endp li)    nil)                               ;cas trivial
        ((equal s li) li)                                ;VRAI immdiat
        (t            (queue s (cdr li)))))              ;confirmer la suite...

;;    dernier <li> => rend le dernier lment de <li>. quivalent LISP :
;;                    (car (last <li>))
;$    dernier <li> => gibt das letzte Element von <li> zurck. LISP-Equivalent:
;$                    (car (last <li>))
;+    dernier <li> => returns the last element of <li>. LISP equivalent:
;+                    (car (last <li>))

(defun dernier1 (li)
  (cond ((endp li)       li)                       ;cas trivial (pas essentiel)
        ((endp (cdr li)) (car li))                 ;cas terminal
        (t (dernier1     (cdr li)))))              ;voir la suite...

(defun dernier2 (li)                               ;autre version, plus concise
  (if (endp (cdr li)) (car li)                     ;cas terminal
  (dernier2 (cdr li))))                            ;voir la suite...

;;    longueur <li> => nombre d'lments de premier niveau de <li>. quivalente
;;                     LISP : list-length
;$    longueur <li> => Anzahl der Elemente auf der ersten Ebene von <li>. 
;$                     LISP-Equivalent: list-length 
;+    longueur <li> => number of elements at the first level of <li>. Lisp
;+                     equivalent: list-length

(defun longueur (li)
  (if (atom li) 0                                  ;cas trivial
      (1+ (longueur (cdr li)))))                   ;le faire... !!!

;;    nieme <n> <li> => rend le <n>-ime lment de <li>, s'il existe.
;;                      quivalente LISP : nth
;$    nieme <n> <li> => gibt das <n>-te Element von <li> zurck, falls es
;$                      existiert. LISP-Equivalent: nth
;+    nieme <n> <li> => returns the <n>-th element of <li>, if it exists.
;+                      LISP eqivalent: nth

(defun nieme1 (n li)
  (cond ((<= n 0) (car li))                        ;cas terminal
        (t        (nieme1 (- n 1) (cdr li)))))     ;continuer...

(defun nieme2 (n li)                               ;autre rdaction, plus claire
  (if (<= n 0) (car li)                            ;cas terminal
  (nieme2 (1- n) (cdr li))))                       ;continuer...

;;    egal <a> <b> => t si, et seulement si <a> et <b> sont des objets
;;                    identiques. quivalente LISP : equal
;$    egal <a> <b> => t wenn, und nur wenn <a> und <b> identische Objekte sind.
;$                    LISP-Equivalent: equal
;+    egal <a> <b> => t if, and only if <a> and <b> are identical objects.
;+                    LISP equivalent: equal

(defun egal (a b) (declare (notinline egal))            ;DCOMPOSITION D'ARBRE :
  (cond ((atom a) (eq a b))                             ;cas terminal
        ((atom b) nil)                                  ;cas trivial
        ((egal (car a) (car b))                         ;confirmer...
          (egal (cdr a) (cdr b)))                       ;... la suite...
        (t nil)))                                       ;FAUX immdiat

;;   Sur le modle de egal :
;;    arbrmidip <a> => t si, et seulement si <a> est un arbre ne contenant que
;;                     des MIDI pitches
;; Bug : accepte un nil parmi les MIDI pitches... La seconde clause devrait donc
;; grer spcifiquement un tel cas.
;; N.B. : suppose la dfinition de midip (cf. fichier exo.10.0.lisp).
;$   Basierend auf obiger "egal"-Funktion:
;$    arbrmidip <a> => t wenn, und nur wenn <a> ein Baum ist, der nur
;$                     MIDI-Pitches enthlt
;$ Bug: akzeptiert nil inmitten der MIDI-Pitches ... Die zweite Anweisung sollte
;$ diese Ausnahme entsprechend abfangen.
;$ N.B.: setzt die Definition von midip voraus (Vgl. Datei exo.10.0.lisp).
;+   Based on the above egal:
;+    arbrmidip <a> => t if, and only if <a> is a tree containing only MIDI
;+                     pitches
;+ Bug: accepts nil among the MIDI pitches... The second clause should deal with
;+ this specific eventuality.
;+ N.B.: presumes the definition of midip (cf. exo.10.0.lisp file).

(defun arbrmidip (a) (declare (notinline arbrmidip))    ;DCOMPOSITION D'ARBRE :
  (cond ((and a (atom a)) (midip a))                    ;cas terminal
        ((endp a) t)                                    ;poursuivre
        ((arbrmidip (car a))                            ;confirmer...
          (arbrmidip (cdr a)))                          ;... la suite...
        (t nil)))                                       ;FAUX immdiat

;;    nbratoms <li> => nombre d'atomes prsents,  tous les niveaux de <li>,
;;                     c'est--dire le nombre de feuilles de l'arbre <li>
;; Bug : ne compte pas un atome nil parmi les feuilles de <li>...
;$    nbratoms <li> => Atomanzahl auf allen Ebenen von <li>, das heit
;$                     die Anzahl der Bltter des Baums <li>
;$ Bug: zhlt keine nil-Atome inmitten der Bltter von <li>...
;+    nbratoms <li> => number of atoms at all levels of <li>, which means the
;+                     number of leaves of the <li> tree
;+ Bug: does not count a nil atom among the leaves of <li>...

(defun nbratoms1 (li) (declare (notinline nbratoms1))   ;DCOMPOSITION D'ARBRE :
  (cond ((null li) 0)                                   ;cas trivial
        ((atom li) 1)                                   ;cas terminal
        (t         (+ (nbratoms1 (car li))              ;le faire...
                      (nbratoms1 (cdr li))))))          ;... tout simplement !

;;   Diffrences entre les termes successifs d'une liste de nombres <li>. Il
;; s'agit de la drive d'une fonction engendrant les termes de <li>. Exemple,
;; appliqu  une mlodie en MIDI pitches :
;$   Unterschied zwischen den aufeinanderfolgenden Gliedern einer Liste
;$ <li>. Die Funktion besteht aus der Ableitung einer Funktion zur Erzeugung
;$ der Glieder von <li>. Beispiel anhand einer Melodie in MIDI-Pitches:
;+   Differences between the successive terms of a list of numbers <li>. It
;+ consists in the derivative of a function generating the terms of <li>.
;+ Example, applied to a melody in MIDI pitches: 
;-    (deriv '(64 64 65 67 67 65 64 62 60 60 62 64 64 62 62))
;-        => (0 1 2 0 -2 -1 -2 -2 0 2 2 0 -2 0)
;; On obtient donc la liste des intervalles sparant les notes successives.
;$ Man erhlt eine Liste der Intervalle zwischen aufeinanderfolgenden Noten.
;+ One obtains the list of the intervals joining the successive notes.

(defun deriv (li)
  (if (endp (cddr li)) (cons (- (cadr li) (car li)) nil)
      (cons (- (cadr li) (car li)) (deriv (cdr li)))))
