;;; Ce fichier peut tre charg. Il comporte des exemple  valuer manuellement.
;;; Version LispWorks CAPI, 05.2009

;;   Exemple de dessin fractal du genre croissance vgtale - espce de liane.
;; Sur mme base que ped.11.3.1-CAPI.lisp.

;; PwrBkG4 : la fentre maximale est :view-size #@(1150 720) (horiz. - verti.)
;; MacBookPro 2009 : la fentre maximale est 1920 x 1200 (horiz. - verti.)

;; LispWorks : un point est repsent par une paire pointe de coordonnes
;; (x . y)

;;   Utilisations type, avec dimensions de fentres fixes au maximum. L'origine
;; de tous les tracs est invariable  (0 . 0).
;;   La valeur de lon_min affecte la finesse du dtail de la liane obtenue et
;; est inversement proportionnelle au temps ncessaire pour effectuer le trac.
;; Chaque appel produit une croissance alatoirement diffrente.
;; (defparameter lon_min 20.0)
;; (defparameter lon_min 15.0)
;; (defparameter lon_min 10.0)
;; (defparameter lon_min  8.0)
;; (defparameter lon_min  5.0)
;; (defparameter lon_min  3.0)

;; N.B. LispWorks Personal 5.1.1 (2009, 2.6 GHz Intel Core 2 Duo) :
;; a du mal avec les gros dessins... Pour rcuprer de la place mmoire...
;;    voir avant:  (room)
;;    en faire :   (mark-and-sweep 3)
;;    voir aprs : (room)
;; ... LispWorks a du mal pour lon_min < 10...

;; (DesLiane  50 pis4) oder (DesLiane  50 pis4 :print t)
;; (DesLiane  90 pis4) oder (DesLiane  90 pis4 :print t)
;; (DesLiane 110 pis4) oder (DesLiane 110 pis4 :print t)
;; (DesLiane 150 pis4) oder (DesLiane 150 pis4 :print t)

(defparameter pos-h-orig 0)           ;Coordonnes rectangulaires de l'origine :
(defparameter pos-v-orig 0)           ;point #@(0 0)
(defparameter pos-h nil)              ;Coordonnes courantes en cours de trac
(defparameter pos-v nil)

(defparameter pis2    (/ pi 2.0))     ;Dfinitions prleminaires
(defparameter pis4    (/ pi 4.0))
(defparameter pis8    (/ pi 8.0))

(defparameter lon_min 10.0      )     ;pas 1.0... pour soulager LispWorks !

(defparameter dimH   1750       )     ;dimensions fentre
(defparameter dimV    950       )

(defparameter lesco nil)

;;   Tous les mots-clef concernent l'application en notes MIDI par CLCE. Cf.
;; fichier cl.d/clce.d/ped.d/xmpls_CoursLISP/11.3-Beispiele.clce.lisp.  cause
;; d'un parcours non squentiel du temps, un score midiShare est cr.
;;    clce      := activation de l'interface avec CLCE (cration du score etc.)
;;                 [NIL par dfaut]
;;    print     := impression des coordonnes (pixels) des points d'inflexion de
;;                 la liane [NIL par dfaut]
;;    max-date  := date d'attaque (msecs) correspondant  la limite droite de la
;;                 fentre (cf. dimH ci-dessus) [10000 par dfaut]. La date
;;                 d'attaque correspondant  la limite gauche de la fentre sera
;;                 videmment toujours 0
;;    min-dur   := dure minimale (msecs) des notes [5 par dfaut]
;;    delta-dur := delta d'une distribution exponentielle dterminant un
;;                 supplment de dure ajout  min-dur [0.5 par dfaut]
;;    min-pitch := MIDI pitch correspondant  la limite infrieure de la fentre
;;                 [12 par dfaut]
;;    max-pitch := MIDI pitch correspondant  la limite suprieure de la fentre
;;                 [95 par dfaut]

(defun DesLiane (lon ang &key clce print
                              (max-date 10000)
                              (min-dur 5) (delta-dur 0.5)
                              (min-pitch 12) (max-pitch 95)
                         &aux *fen* l1 l2 a1)
  (setq *fen* (capi::contain
               (make-instance 'capi::output-pane)
               :best-width dimH :best-height dimV
               :title (string (gensym "Dessin-Liane-"))))
  (setq pos-h pos-h-orig)
  (setq pos-v pos-v-orig)
  ;cration du midiShare score si ncessaire
  (when clce (setq lesco (new 'midi-score :name "lesco")))
  (labels ((interne (l a)
             (if (< l lon_min) (tracer l a *fen* 1 :clce clce :print print
                                                   :max-date max-date
                                                   :min-dur min-dur
                                                   :delta-dur delta-dur
                                                   :min-pitch min-pitch
                                                   :max-pitch max-pitch    )
                 (progn (interne (setq l1 (* l (corect 0.3 0.5)))      ; 1
                                 a)
                        (interne (setq l2 (* l (corect 0.3 0.5)))      ; 2
                                 (setq a1 (+ a (corect pis8 pis4))))
                        (interne l2                                    ; 3
                                 (+ a1 pi))
                        (interne (* l (corect 0.0 0.2))                ; 4
                                 (+ a (if (zerop (random 2)) 0 pi)))
                        (interne (setq l2 (* l (corect 0.3 0.5)))      ; 5
                                 (setq a1 (- a (corect pis8 pis4))))
                        (interne l2                                    ; 6
                                 (+ a1 pi))
                        (interne (- l l1)                              ; 7
                                 a)))))
    (interne lon ang))
  ;pour imprimer les positions finales...  
  (when print (format t "~&FIN~%pos-h = ~10F~25Tpos-v = ~10F~%" pos-h pos-v))
  ;pour crire la note finale dans le midiShare score
  (when (and clce lesco)
    (midi-write-ev lesco (note :pitch (round
                                       (+ max-pitch (* (- min-pitch max-pitch)
                                                       (/ pos-v dimV))))
                               :dur   (round (+ min-dur (expon delta-dur))))
                   :date (round (* pos-h (/ max-date dimH))))
    (when print (format t "date  =~8D~18Tduree =~8D~36Tpitch =~5D~%"
                        (round (* pos-h (/ max-date dimH)))
                        (round (+ min-dur (expon delta-dur)))
                        (round (+ max-pitch (* (- min-pitch max-pitch)
                                               (/ pos-v dimV))))))))

;;    corect <r1> <r2>, o <r1> < <r2>, et tous deux sont des rels
;;	=> un rel dans [<r1>,<r2>[.
;; Cf. librairie stochastique.

(defun corect (r1 r2)
	(+ r1 (random (- r2 r1))))

;;   Conversion de vecteurs en coordonnes rectangulaires de l'extrmit 
;; atteindre. N'EST PLUS UTILE EN LispWorks : la conversion de vecteurs en
;; coordonnes cartsiennes est intgre  la fonction tracer ci-dessous.
;(defun next-pt (lon ang)
;  (let ((h (* lon (sin ang)))   
;        (v (* lon (cos ang))))
;    (incf pos-h h)
;    (incf pos-v v)
;    (values (round pos-h) (round pos-v))))

;;   Trac rel d'un segment de droite dans <fenetre> := un vecteur de longueur
;; <lon> et d'angle <ang>, commenant au point courant (<pos-h> . <pos-v>).
;; L'extrmit du vecteur trac devient ensuite le nouveau point courant.
;; Possible de modifier l'paisseur [<epais> = 1 par dfaut].

(defun tracer (lon ang fenetre &optional (epais 1)
                               &key clce print
                                    max-date
                                    min-dur delta-dur
                                    min-pitch max-pitch)
  ;pour imprimer les positions  chaque tape...  
  (when print (format t "~&pos-h = ~10F~25Tpos-v = ~10F~%" pos-h pos-v))
  ;pour crire les notes dans le midiShare score
  (when (and clce lesco)
    (midi-write-ev lesco (note :pitch (round
                                       (+ max-pitch (* (- min-pitch max-pitch)
                                                       (/ pos-v dimV))))
                               :dur   (round (+ min-dur (expon delta-dur))))
                   :date (round (* pos-h (/ max-date dimH))))
    (when print (format t "date  =~8D~18Tduree =~8D~36Tpitch =~5D~%"
                        (round (* pos-h (/ max-date dimH)))
                        (round (+ min-dur (expon delta-dur)))
                        (round (+ max-pitch (* (- min-pitch max-pitch)
                                               (/ pos-v dimV)))))))
  ;pour graphique
  (capi::apply-in-pane-process fenetre 'gp::draw-line fenetre
                               pos-h
                               pos-v
                               (incf pos-h (* lon (sin ang)))
                               (incf pos-v (* lon (cos ang)))
                               :thickness epais))

;;   Petit utilitaire : cre un point aux coordonnes courantes
;; (<pos-h> . <pos-v>).

(defun pos () (cons pos-h pos-v))
