;;; Modles de croissance vgtale (avec/sans ge du bois)
;;; Version LispWorks CAPI, 05.2009

;; Cf. [GODIN, Christophe, SINOQUET, Herv, COSTES, velyne & TRAAS, Jan, 2006 :
;; "Des plantes virtuelles pour comprendre les plantes relles", Dossier Pour la
;; science n52, La modlisation informatique, exploration du rel : 76-81].

;; Charger cl.d/+clinit.cl pour pasapas

;;   Si la variable globale *age* est t, l'paisseur des traits est
;; proportionnelle  l'ge du bois : plus un rameau est profond dans la
;; structure (i.e. "jeune"), plus il est mince, et vice-versa.
;;   Cette prise en compte des paisseurs des bois d'ges diffrents produit un
;; graphisme plus satisfaisant... jusqu' "un certain nombre" de niveaux,
;; au-del duquel l'paisseur des "vieux" bois cache les niveaux plus profonds
;; de la structure : l'apparence est plus raliste, mais moins claire quant  la
;; structure fine.
(defparameter *age* t) ;=> Tenir compte de l'ge du bois
;; (setq *age* nil) ;Graphisme "sans ge" : tous traits d'paisseur 1
;;   En croissance ramifie (fonction croiss-ramifiee), aprs <N> appels, on a
;; un arbre de profondeur <N> comportant (expt 2 <N>) entrenoeuds et mristmes.
;; Ces nombres de niveaux et d'lments atomiques sont videmment moindres dans
;; le cas de l'utilisation de la fonction croissance, car celle-ci laisse une
;; certaine probabilit  de la croissance linaire (cf. ci-dessous).

;;; Croissance linaire
;; Dpart : M (Mristme initial)
;; Rgle  : M -> E M
;;          (Mristme -> Entrenoeud, Mristme apical)
;; Dmonstration symbolique pas  pas...
;; (let ((h '(M))) (pasapas (progn (print h) (setq h (croiss-lineaire h)))) h)
;; Reprsentation graphique d'un arbre donn
;; (graph1 '<arbre>) ;N.B. : gestion d'ge marche pas dans ce cas...
;; Dmonstration graphique (inclut procdure pas  pas...)
;; (graphe 'croiss-lineaire '(M))

(defun croiss-lineaire (arbre)            ;(declare (notinline croiss-lineaire))
  (if (eq 'M (car arbre)) '(E M)
  (cons (car arbre) (croiss-lineaire (cdr arbre)))))

;;; Croissance ramifie
;; Dpart : M (Mristme initial)
;; Rgle  : M -> E (M) M
;;          (Mristme -> Entrenoeud, Mristme axilaire, Mristme apical)
;; Dmonstration symbolique pas  pas...
;; (let ((h '(M))) (pasapas (progn (print h) (setq h (croiss-ramifiee h)))) h)
;; Reprsentation graphique d'un arbre donn
;; (graph1 '<arbre>)
;; Dmonstration graphique (inclut procdure pas  pas...)
;; (graphe 'croiss-ramifiee '(M))

(defun croiss-ramifiee (arbre)            ;(declare (notinline croiss-ramifiee))
  (cond ((eq 'M (car arbre)) '(E (M) M))
        ((eq 'E (car arbre)) (cons 'E (croiss-ramifiee (cdr arbre))))
        (t                   (cons (croiss-ramifiee (car arbre))
                                   (croiss-ramifiee (cdr arbre))))))

;;; Croissance mixte, ramifie ou parfois linaire
;; Dpart : M (Mristme initial)
;; Rgles : 1] M -> E (M) M avec probabilit *prob-ramification*
;;             (Mristme -> Entrenoeud, Mristme axilaire, Mristme apical)
;;          2] M -> E M avec probabilit (1 - *prob-ramification*)
;;             (Mristme -> Entrenoeud, Mristme apical)
(defparameter *prob-ramification* 0.85)
;; Dmonstration symbolique pas  pas...
;; (let ((h '(M))) (pasapas (progn (print h) (setq h (croissance h)))) h)
;; Reprsentation graphique d'un arbre donn
;; (graph1 '<arbre>)
;; Dmonstration graphique (inclut procdure pas  pas...)
;; (graphe 'croissance '(M))

(defun croissance (arbre)                      ;(declare (notinline croissance))
  (cond ((eq 'M (car arbre)) (if (< (random 1.0) *prob-ramification*)
                               '(E (M) M)
                             '(E M)))
        ((eq 'E (car arbre)) (cons 'E (croissance (cdr arbre))))
        (t                   (cons (croissance (car arbre))
                                   (croissance (cdr arbre))))))

;;; Interface graphique

;; MacBookPro 2009 : la fentre maximale est 1920 x 1200 (horiz. - verti.)

;; LispWorks : un point est repsent par une paire pointe de coordonnes
;; (x . y)

;;; Variables globales de rglage, lies aux modles

;; Rduction de taille applique  chaque sous-niveau
(defparameter *reduction-taille* 0.938)
;; Taille au premier niveau (pixels)
(defparameter *taille-unite* 100)
;; cart type des variations gaussiennes des tailles, en proportion des tailles
(defparameter *ecart-type* 0.03)
;; Valeurs absolues min et max des angles des ramifications (degrs -> radians)
(defparameter *angle-min* ((lambda (x) (* (/ x 360.0) 2.0 pi)) 36))
(defparameter *angle-max* ((lambda (x) (* (/ x 360.0) 2.0 pi)) 44))

;;; Variables globales purement graphiques

;; Il faut 24 pixels de plus verticalement pour une fentre CAPI (sans doute 
;; cause de la barre de titre).

(defparameter taille-fen '(1301 . 824));Fentre : (largeur . hauteur) pixels
(defparameter pos-h-orig 650)          ;Coordonnes rectangulaires de l'origine=
(defparameter pos-v-orig 797)          ;point (650 . 797) = milieu bas fentre
(defparameter *rayon-meristeme* 5)     ;Rayon extrmit d'un mristme (pixels)
(defparameter *couleur-meristeme* 'blanc);couleur = 'blanc ou 'noir

(defparameter pos-orig (cons pos-h-orig pos-v-orig))
(defparameter pos-h nil)              ;Coordonnes courantes en cours de trac
(defparameter pos-v nil)

;; Affichage d'un arbre <erbra> donn

(defun graph1 (erbra &aux *fen* profondeur)
  (setq profondeur (profond erbra))
  (setq *fen* (capi::contain
               (make-instance 'capi::output-pane)
               :best-width (car taille-fen) :best-height (cdr taille-fen)
               :title (string (gensym "erbra-"))))
  (capi::apply-in-pane-process *fen* 'gp::draw-string *fen*
                               (format nil "~2D" profondeur)
                               3 (- pos-v-orig 3))
  (setq pos-h pos-h-orig)
  (setq pos-v pos-v-orig)
  (arborer *fen* erbra pi 1.0 pos-orig pos-orig (age? profondeur)))

;; Gestion de l'affichage ritr pas  pas...

(defun graphe (fonction harb &aux *fen* (num 0))
  (setq *fen* (capi::contain
               (make-instance 'capi::output-pane)
               :best-width (car taille-fen) :best-height (cdr taille-fen)
               :title (string (gensym "arbre-"))))
  (format t "Taper quelque chose, par ex. <RETURN>, pour repeter, ~
             et <q> pour finir.")
  ;boucle pour gnrations successives
  (pasapas (progn (print harb)
             (capi::apply-in-pane-process *fen*            ;effacer la fentre
                        'gp::draw-rectangle *fen*
                        0 0 (car taille-fen) (cdr taille-fen)
                        :foreground :white
                        :filled t)
             (capi::apply-in-pane-process *fen* 'gp::draw-string *fen*
                        (format nil "~2D" (setq num (1+ num)));pasapas =/=> incf
                        3 (- pos-v-orig 3))
             (setq pos-h pos-h-orig)
             (setq pos-v pos-v-orig)
             (arborer *fen* harb pi 1.0  pos-orig pos-orig (age? num))
             (setq harb (funcall fonction harb))))
  ;dessiner la dernire gnration endue aprs "q" d'arrt de pasapas
  (capi::apply-in-pane-process *fen*                       ;effacer la fentre
             'gp::draw-rectangle *fen*
             0 0 (car taille-fen) (cdr taille-fen)
             :foreground :white
             :filled t)
  (capi::apply-in-pane-process *fen* 'gp::draw-string *fen*
             (format nil "~2D" (incf num))
             3 (- pos-v-orig 3))
  (setq pos-h pos-h-orig)
  (setq pos-v pos-v-orig)
  (arborer *fen* harb pi 1.0  pos-orig pos-orig (age? num))
  harb)

;; Fonction graphique effective, rcursive
;; N.B. : Structure rcursive...
;;        Actions graphiques proprement dites...

(defun arborer (fenetre arbre angle coef-taille orig-abs orig-rel age)
  (labels ((interne (arb ang cft ora orr age) (cond
            ((endp arb) (meristeme fenetre (pospt) *couleur-meristeme*)
                        (setq pos-h (car ora))
                        (setq pos-v (cdr ora)))
            ((consp (car arb)) (interne (car arb)
                                        (choix-angle ang)
                                        (* cft *reduction-taille*)
                                        orr orr (max 1 (1- age)))
                               (interne (cdr arb)
                                        ang
                                        (* cft *reduction-taille*)
                                        ora ora age))
            (t (let ((taille (* *taille-unite* cft)))
                (tracer (gauss taille (* taille *ecart-type*)) ang fenetre age))
               (interne (cdr arb)
                        ang
                        (* cft *reduction-taille*)
                        ora (pospt) (max 1 (1- age)))))))
    (interne arbre angle coef-taille orig-abs orig-rel age)))

;;; Auxiliaires, dont cf. canons stochastiques

;; Tenir compte de l'ge ou non, selon que la variable globale *age* est
;; t ou nil.
(defun age? (n) (if *age* n 1))
(when *age* (format t "~&;Au chargement : prise en compte de l'age...~%"))

;;   Choix d'un angle alatoire, avec contrle sigmode de la
;; probabilit du sens (signe), de manire  favoriser les ramifications
;; vers le "ciel" plutt que vers la "terre".
;;   L'argument d'appel <deja> est l'angle de la branche porteuse. Si celle-ci
;; est dj descendante, il est trs peu probable que la ramification aille
;; aussi vers le bas. Lorsque la branche porteuse est ascendante et verticale,
;; la ramification sera quiprobablement vers la gauche ou la droite (positive
;; ou ngative).
;;   La valeur rendue est l'angle absolu de la ramification : celui de la
;; branche porteuse +/- un angle alatoire tir dans ]*angle-min*,*angle-max*[.

(defun choix-angle (deja)
  (+ deja (* (alter2 -1 1 (/ 1.0 (+ 1.0 (exp (- (- deja pi))))))
             (corect *angle-min* *angle-max*))))

;; Initialisation du germe alatoire selon date et heure
(setq *random-state* (make-random-state t))
(format t "~&;Germe alatoire initialis selon date et heure...~%")

(defun alter2 (a b pdea)
"
;    alter2 <a> <b> <pdea> => choix entre <a>, de probabilite <pdea>, et <b>, de
;                             probabilite (1-<pdea>).
"
  (if (<= (random 1.0) pdea) a b))

(defun corect (r1 r2)
"
;    corect <r1> <r2>, ou <r1> < <r2>, et tous deux sont des reels
;        => un reel dans [<r1>,<r2>[.
"
  (+ r1 (* (- r2 r1) (random 1.0))))

(defun gauss (mu sigma)
"
;    gauss <mu> <sigma> => reel de distribution Gauss-Laplace de moyenne <mu>
;                          et d'ecart type <sigma>.
"
  (let ((s 0.0))
    (dotimes (cte 12 (+ (* (- s 6.0) sigma) mu))
      (setq s (+ s (random 1.0))))))

;; Cf. ped.13, modifie pour primitive max au lieu de fonction max1 du cours.
;;    profond <arb> => nombre de niveaux de profondeur dans l'arbre <arb>.
;;                     Variante de l'algorithme de nbratoms2 : cf. ci-dessous.

(defun profond (arb)
  (cond ((null arb) 1)
        ((atom arb) 0)
        (t (1+ (apply #'max (mapcar #'profond arb))))))
;; Cf. ped.13 itou.
;;    nbratoms <li> => nombre d'atomes prsents,  tous les niveaux, dans <li>.
;;                     Plus efficace que la programmation du Chapitre 11.2a.
;;                     Variante de l'algorithme de profond : cf. ci-dessus.

(defun nbratoms2 (li)
  (cond ((null li) 0)
        ((atom li) 1)
        (t (apply #'+ (mapcar #'nbratoms2 li)))))

;;; Auxiliaires proprement graphiques

;;   Conversion de vecteurs en coordonnes rectangulaires de l'extrmit 
;; atteindre. N'EST PLUS UTILE EN LispWorks : la conversion de vecteurs en
;; coordonnes cartsiennes est intgre  la fonction tracer ci-dessous.
;(defun next-pt (lon ang)
;  (let ((h (* lon (sin ang)))   
;        (v (* lon (cos ang))))
;    (incf pos-h h)
;    (incf pos-v v)
;    (values (round pos-h) (round pos-v))))

;;   Trac rel d'un segment de droite dans <fenetre> := un vecteur de longueur
;; <lon> et d'angle <ang>, commenant au point courant (<pos-h> . <pos-v>).
;; L'extrmit du vecteur trac devient ensuite le nouveau point courant.
;; Possible de modifier l'paisseur [<epais> = 1 par dfaut].

(defun tracer (lon ang fenetre &optional (epais 1))
  (capi::apply-in-pane-process fenetre 'gp::draw-line fenetre
                               pos-h
                               pos-v
                               (incf pos-h (* lon (sin ang)))
                               (incf pos-v (* lon (cos ang)))
                               :thickness epais))

;; Dessin d'un cercle 'blanc ou 'noir de centre <pt> et rayon *rayon-meristeme*.

(defun meristeme (fenetre pt couleur)
  (capi::apply-in-pane-process fenetre 'gp::draw-circle fenetre
                               (car pt) (cdr pt) *rayon-meristeme*
                               :thickness 1
                               :filled (when (equal couleur 'noir) t)))

;;   Petit utilitaire : cre un point aux coordonnes courantes
;; (<pos-h> . <pos-v>).

(defun pos () (cons pos-h pos-v))

;; Cre un point de forme (pos-h . pos-v) aux coordonnes courantes ARRONDIES.

(defun pospt () (cons (round pos-h) (round pos-v)))

;; Pour tests : fonction produisant le trac d'un carr. Exemple, carr de 333
;; pixels de ct,  partir du point d'origine (650 . 797) dans une fentre de
;; 801 x 821 :
;;    (carre 333 pi '(801 . 821))
;; Il faut 24 pixels de plus verticalement pour une fentre CAPI (sans doute 
;; cause de la barre de titre).

(defun carre (lon ang taille &aux *fen*)
  (setq *fen* (capi::contain
               (make-instance 'capi::output-pane)
               :best-width (car taille)
               :best-height (cdr taille)
               :title (string (gensym "Dessin-"))))
  (setq pos-h pos-h-orig)
  (setq pos-v pos-v-orig)
  (format t "Dpart : ~5D ~5D~%" (car (pos)) (cdr (pos)))
  (tracer lon ang *fen* 25)
  (format t "Fin 1 : ~5D ~5D~%" (car (pos)) (cdr (pos)))
  (tracer lon (incf ang (/ pi 2.0)) *fen*)
  (format t "Fin 2 : ~5D ~5D~%" (car (pos)) (cdr (pos)))
  (tracer lon (incf ang (/ pi 2.0)) *fen* 6)
  (format t "Fin 3 : ~5D ~5D~%" (car (pos)) (cdr (pos)))
  (tracer lon (incf ang (/ pi 2.0)) *fen*)
  (format t "Fin 4 : ~5D ~5D~%" (car (pos)) (cdr (pos))))