;===============================================================================
;;; 12.suppl --> Expansion / contraction des intervalles entre termes d'une
;;;              liste de nombres -- par exemple de hauteurs
;$$ 12.suppl --> Expansion / Zusammenziehung der Abstnde zwischen den Gliedern
;$$              einer Liste von Zahlen -- zum Beispiel Tonhhen      [d..H.B.]
;++ 12.suppl --> Expansion / Contraction of the Intervals Between Terms of a
;++              List of Numbers--Pitches for Instance
;===============================================================================

;;; -1- Ralisation sans labels, avec plusieurs defun : trois fonctions
;;;     auxiliaires et une interface
;$$ -1- Umsetzung ohne labels, mit einigen defun: drei Hilfsfunktionen und eine
;$$     Benutzerschnittstelle
;++ -1- Realization without labels, with several defun: three auxiliairy
;++     functions and one interface

;;   Calcule la "drive" d'une liste de nombres <li>, c'est--dire une liste
;; <rez> dont les termes sont les diffrences entre les couples de termes
;; successifs de <li> :
;;    deriv <li> => liste <rez>
;; o <rez<n>> est gal  (- <li<n+1>> <li<n>>). <rez> compte donc videmment un
;; terme de moins que <li>.
;$   Das "Derivat" einer <li> Nummernliste ausrechnet, das heit eine <rez>
;$ Liste von Glieder die die Differenzen zwischen die aufeinanderfolgende
;$ Gliederpaare von <li> sind:
;$    deriv <li> => <rez> Liste
;$ wo <rez<n>> gleich ist an (- <li<n+1>> <li<n>>). Also hat <rez> ein Glied
;$ weniger als <li>.
;+   Computes the "derivative" of a list <li> of numbers, which results in a
;+ <rez> list of terms equal to the differences between successive term couples
;+ of <li>:
;+    deriv <li> => <rez> list
;+ where <rez<n>> is equal to (- <li<n+1>> <li<n>>). The <rez> list thus
;+ contains one term less than <li>.

(defun deriv (li)
  (if (endp (cddr li)) (cons (- (cadr li) (car li)) nil)
  (cons (- (cadr li) (car li)) (deriv (cdr li)))))

;;   Applique,  tous les termes d'une liste <li>, une multiplication par un
;; facteur. Ce facteur est <fact> pour le premier des termes de <li>. Pour les
;; termes suivants, <fact> est chaque fois rvalu  (expt <fact> <exp>). Si
;;    <exp> = 1 => tous les termes de <li> sont multiplis par <fact> : on
;;                 obtient donc un effet d'expansion/compression constante des
;;                 termes de <li>
;;    <exp> > 1 => les multiplicateurs des termes successifs de <li> croissent
;;                 exponentiellement si <fact> > 1 ou
;;                 logarithmiquement si <fact> < 1
;;    <exp> < 1 => les multiplicateurs des termes successifs de <li> croissent
;;                 logarithmiquement si <fact> > 1 ou
;;                 exponentiellement si <fact> < 1
;; La fonction est sans effet sur <li> si <fact> = 1.
;;   Exemples, coups  trois dcimales :
;$   Multipliziert alle Glieder einer Liste <li> mit einem Faktor. Dieser Faktor
;$ <fact> ist fr das erste Glied von <li>. Fr die folgenden Glieder wird
;$ <fact> jedes Mal neu mit (expt <fact> <exp>) berechnet. Wenn
;$    <exp> = 1 => alle <li> Glieder werden mit <fact> multipliziert: eine
;$                 gleichmige Expansion/Kontraktion wird daher auf
;$                 die <li> Glieder angewendet
;$    <exp> > 1 => die Multiplikatoren der aufeinanderfolgenden <li> Glieder
;$                 erhhen sich
;$                 exponentiell wenn <fact> > 1 oder
;$                 logarithmisch wenn <fact> < 1
;$    <exp> < 1 => die Multiplikatoren der aufeinanderfolgenden <li> Glieder
;$                 erhhen sich
;$                 logarithmisch wenn <fact> > 1 oder
;$                 exponentiell wenn <fact> < 1
;$ Die Funktion hat keine Wirkung auf <li> wenn <fact> = 1.
;$   Beispiele, nach drei Dezimalen abgeschnitten:
;+   Applies to all terms of a <li> list a multiplication by a factor. This
;+ factor is <fact> for the first term of <li>. For the following terms, <fact>
;+ is each time reevaluated to (expt <fact> <exp>). If
;+    <exp> = 1 => all terms of <li> are multiplied by <fact>: a constant
;+                 expansion/contraction effect is thus applied to the <li>
;+                 terms
;+    <exp> > 1 => the multiplying factors for the successive terms of <li>
;+                 increase
;+                 exponentially if <fact> > 1 or
;+                 logarithmically if <fact> < 1
;+    <exp> < 1 => the multiplying factors for the successive terms of <li>
;+                 increase
;+                 logarithmically if <fact> > 1 or
;+                 exponentially if <fact> < 1
;+ The function has no effect on <li> if <fact> = 1.
;+   Examples, cut after three decimal places:
;-    (graf (expans '(100 100 100) 1   1  ) :ech 0.2) => (100 100     100    )
;-    (graf (expans '(100 100 100) 2   1  ) :ech 0.2) => (200 200     200    )
;-    (graf (expans '(100 100 100) 2   1.1) :ech 0.2) => (200 214.354 231.337)
;-    (graf (expans '(100 100 100) 2   0.9) :ech 0.2) => (200 186.606 175.321)
;-    (graf (expans '(100 100 100) 1/2 1  ) :ech 0.2) => ( 50  50      50    )
;-    (graf (expans '(100 100 100) 1/2 1.1) :ech 0.2) => ( 50  46.651  43.226)
;-    (graf (expans '(100 100 100) 1/2 0.9) :ech 0.2) => ( 50  53.588  57.038)

(defun expans (li fact exp)          ;<exp> constant inutile dans les rcursions
  (if (endp (cdr li)) (cons (* (car li) fact) nil)
  (cons (* (car li) fact) (expans (cdr li) (expt fact exp) exp))))

;;   Construit une liste dont les termes successifs sont obtenus par cumul de
;; <prem> et des termes de <lidiff> :
;$   Erstellt eine Liste, deren aufeinanderfolgende Glieder durch Anhufung
;$ von <prem> und den Gliedern der <lidiff> erhalten werden:
;+   Builds a list of which the successive terms are obtained by cumulation of
;+ <prem> and the terms of <lidiff>:
;-    constr <prem> <lidiff> => <rez>
;- o / wo / where
;-    <rez<0>> = (+     <prem> <ldiff<0>>  )
;-    <rez-1-> = (+   <rez<0>> <ldiff-1->  )
;-    ...
;-    <rez<n>> = (+ <rez<n-1>>> <ldiff<n>>>)
;; <rez> compte donc autant de termes que <lidiff>.
;$ also hat <rez> dieselbe Elementenanzahl wie <lidiff>.
;+ <rez> thus contains as many elements as <lidiff>.

(defun constr (prem lidiff &aux tmp)
  (if (endp (cdr lidiff)) (cons (+ prem (car lidiff)) nil)
  (cons (setq tmp (+ prem (car lidiff)))
        (constr tmp (cdr lidiff)))))

;;   Applique une expansion/contraction aux intervalles sparant les termes
;; successifs d'une liste <li>. Cette expansion/contraction est
;;    - nulle si <fact> = 1
;;    - constante  <fact> si <exp> = 1
;;    - etc. (cf. fonction expans ci-dessus)
;; Il s'agit en fait d'une interface d'appel des trois fonctions auxiliaires
;; prcdentes, qui ont t dfinies de manire rmanente avec la primitive
;; defun.
;$   Wendet eine Expansion/Kontraktion auf die Intervalle zwischen
;$ aufeinanderfolgenden Gliedern einer Liste <li> an. Die Expansion/Kontraktion
;$ ist:
;$    - null wenn <fact> = 1
;$    - konstant bei <fact> wenn <exp> = 1
;$    - usw. (Vgl. expans-Funktion oben)
;$ Sie besteht in Wirklichkeit aus einer Aufrufschnittstelle fr drei vorher
;$ genannte Funktionen, welche remanent mit dem Primitive defun defininiert
;$ wurden.
;+   Applies an expansion/contraction to the intervals between successive terms
;+ of a <li> list. This expansion/contraction is
;+    - nul if <fact> = 1
;+    - constant at <fact> if <exp> = 1
;+    - etc. (cf. expans function above)
;+ It consists in fact in a calling interface for the three preceding functions
;+ which have been remanently defined with the defun primitive.

(defun list-expans1 (li fact exp)
  (cons (car li) (constr (car li) (expans (deriv li) fact exp))))

;;; -2- Ralisation avec un seul defun incluant un labels
;$$ -2- Umsetzung mit einem einzigen defun inklusive einem labels
;++ -2- Relization with a single defun including a labels

;;   Les fonctions auxiliaires _deriv _expans et _constr jouent le mme rle que
;; leurs homonymes ci-dessus. Cependant, au lieu d'tre cres de manire
;; rmanente avec defun, elles sont dfinies provisoirement de manire interne 
;; list-expans, par la primitive labels. La fonction interne _expans peut tre
;; dispense d'un troisime argument <exp>, car il reste constant.
;;   Exemples, coups  trois dcimales :
;$   Die Hilfsfunktionen _deriv, _expans und _constr haben dieselbe Aufgabe wie
;$ ihre Homonyme oben. Aber statt remanent mit defun definiert zu sein, werden
;$ sie temporr intern in list-expans mittels des labels-Primitive definiert.
;$ Die interne Funktion _expans kann sich ein drittes Aufrufargument <exp>
;$ sparen, da dieses konstant bleibt.
;$   Beispiele, nach drei Dezimalen abgeschnitten:
;+   The auxiliairy functions _deriv _expans and _constr play the same role as
;+ their homonyms above. However, instead of being remanently defined with
;+ defun, they are temporarily defined internally in list-expans with the labels
;+ primitive. The _expans internal function can save a third calling argument
;+ <exp> because it remains constant.
;+   Examples, cut after three decimal places:
;-    (graf (list-expans '(100 200 300 400) 1   1  ) :ech 0.15)
;-         => (100 200 300     400    )
;-    (graf (list-expans '(100 200 300 400) 2   1  ) :ech 0.15)
;-          => (100 300 500     700    )
;-    (graf (list-expans '(100 200 300 400) 2   1.1) :ech 0.15)
;-          => (100 300 514.354 745.692)
;-    (graf (list-expans '(100 200 300 400) 2   0.9) :ech 0.15)
;-          => (100 300 486.606 661.927)
;-    (graf (list-expans '(100 200 300 400) 1/2 1  ) :ech 0.15)
;-          => (100 150 200     250    )
;-    (graf (list-expans '(100 200 300 400) 1/2 1.1) :ech 0.15)
;-          => (100 150 196.651 239.878)
;-    (graf (list-expans '(100 200 300 400) 1/2 0.9) :ech 0.15)
;-          => (100 150 203.588 260.626)
;;   Les termes de <li> peuvent videmment tre des hauteurs en MIDI-cents.
;; Exemples  partir d'un accord de Do majeur, arrondis au cent :
;$   Die Glieder der <li> knnen natrlich Tonhhen in MIDI-Cents sein.
;$ Beispiele, auf einen C-Dur-Akkord angewendet:
;+   The terms of <li> can of course be MIDI-cents pitches. Examples applied to
;+ a C Major chord, rounded to the cent:
;-    (graf (list-expans '(6000 6400 6700 7200) 1   1  ) :ech 0.01)
;-          => (6000 6400 6700 7200)
;-    (graf (list-expans '(6000 6400 6700 7200) 3/2 1  ) :ech 0.01)
;-          => (6000 6600 7050 7800)
;-    (graf (list-expans '(6000 6400 6700 7200) 3/2 1.3) :ech 0.01)
;-          => (6000 6600 7108 8100)
;-    (graf (list-expans '(6000 6400 6700 7200) 3/2 0.7) :ech 0.01)
;-          => (6000 6600 6998 7608)
;-    (graf (list-expans '(6000 6400 6700 7200) 3/4 1  ) :ech 0.01)
;-          => (6000 6300 6525 6900)
;-    (graf (list-expans '(6000 6400 6700 7200) 3/4 1.3) :ech 0.01)
;-          => (6000 6300 6506 6814)
;-    (graf (list-expans '(6000 6400 6700 7200) 3/4 0.7) :ech 0.01)
;-          => (6000 6300 6545 6980)

(defun list-expans (li fact exp)
  (labels ((_deriv (li)
             (if (endp (cddr li)) (cons (- (cadr li) (car li)) nil)
             (cons (- (cadr li) (car li)) (_deriv (cdr li)))))
           (_expans (li fact)
             (if (endp (cdr li)) (cons (* (car li) fact) nil)
             (cons (* (car li) fact) (_expans (cdr li) (expt fact exp)))))
           (_constr (prem lidiff &aux tmp)
             (if (endp (cdr lidiff)) (cons (+ prem (car lidiff)) nil)
             (cons (setq tmp (+ prem (car lidiff)))
                   (_constr tmp (cdr lidiff))))))
    (cons (car li) (_constr (car li) (_expans (_deriv li) fact)))))

;-   Fonction "graphique" supplmentaire. Imprimer dans une fentre  trs
;- petite police de caractres...
(defun graf (li &key (ech 0.1))
  (format t "~%~A =>~%X" li)
  (labels ((interne (l)
             (if (endp l) t
               (progn
                 (dotimes (n (1- (round (* ech (car l)))))
                   (format t "_"))
                 (format t "X")
                 (interne (cdr l))))))
    (interne li)))
;- Exemple :
;-    (graf '(57 211 333 439 73) :ech 0.2)
