;===============================================================================
;;; 13.suppl --> EXEMPLE SUPPLMENTAIRE (pour CRIM 03/99)
;$$ 13.suppl --> ZUSTZLICHES BEISPIEL (fr CRIM 03/99)               [d..H.B.]
;++ 13.suppl --> EXTRA EXAMPLE (for CRIM 03/99)
;===============================================================================

;;   Mlodie fractale entre un point de dpart et un point d'arrive donns. La
;; mlodie applique est dfinie par une squence d'intervalles temprs.
;;   Les calculs sont faits de manire purement numrique, abstraite.
;; Un *point* numrique peut correspondre  un degr dans une chelle
;; quelconque de hauteurs. Par exemple : les trois *points* (2.0 2.5 3.0)
;; pourraient correspondre, dans une chelle tempre  12 demi-tons, 
;; (Do1 Fa#1 Do2), ou (Fa3 Sol#3 Si3), ou (Re#0 Mi0 Fa0), etc. Dans une chelle
;; de frquences, si l'on dfinissait 2.0 := 440.0 Hz et 3.0 := 779.657 Hz, ils
;; pourraient aussi correspondre aux frquences (440.0 585.7039 779.657).
;; Tout dpendrait entirement de la dfinition de 2.0 et de 3.0 dans la
;; *reprsentation externe* choisie, qui serait dtermine ultrieurement.
;;   Une telle laboration purement numrique permet de travailler librement
;; dans une dimension abstraite -- par exemple entre 0.0 et 1.0, ou entre
;; MIDI-pitches quelconques, etc. : cf. -1-, -2- et -3- ci-dessous.
;;   La description des intervalles dfinissant le motif  fractaliser est
;; arbitraire. Les exemples ci-dessous font penser  des intervalles en
;; demi-tons temprs. En ralit, seules leurs proportions relatives sont
;; utilises. 
;;   La traduction en *reprsentation externe* -- frquences, MIDI-pitches,
;; paires pointes (note . octave), notation musicale, etc. -- sera ralise au
;; dernier moment : cf. -4- ci-dessous.
;;   Les premiers exemples sont intentionnellement faits avec des valeurs assez
;; "simples", qui tombent assez "juste", pour s'assurer de la validit des
;; fonctions.
;$   Fraktale Melodie zwischen zwei gegebenen Anfang- und Endpunkt. Die
;$   Fraktale Melodie zwischen gegebenem Anfangs- und Endpunkt. Die angewandte
;$ Melodie wird durch eine Folge temperierter Intervalle bestimmt.
;$   Die Kalkulationen werden nur numerisch, "abstrakt" durchgefhrt. Ein
;$ numerischer *Punkt* kann einem Ton einer beliebigen Tonleiter entsprechen.
;$ Beispielsweise knnen die drei *Punkte* (2.0 2.5 3.0) in einer temperierten
;$ Skala mit zwlf Halbtnen (C1 F#1 C2), (F3 G#3 B3) oder (D#0 E0 F0) usw. 
;$ entsprechen. In einer Frequenzskala, wobei 2.0 := 440 Hz und 3.0 := 779.657
;$ Hz definiert ist, wrden sie den Frequenzen (440.0 585.7039 779.657)
;$ entsprechen. Alles hinge von der Definition von 2.0 und 3.0 in der
;$ ausgewhlten *ueren Reprsentation* ab, welche spter bestimmt wrde.
;$   Diese nur numerische Arbeitsweise macht es mglich, in einer abstrakten
;$ Dimension frei zu arbeiten -- z. B. zwischen 0.0 und 1.0, oder zwischen
;$ beliebigen MIDI-Pitches, usw.: vgl. -1-, -2- und -3- unten.
;$   Die Beschreibung der Abstnde des zu fraktalisierenden Motivs ist frei
;$ whlbar. Die folgenden Beispiele suggerieren temperierte Halbtne. Aber nur
;$ ihre relative Beziehung zueinander wird wirklich beachtet.
;$   Die bersetzung in eine *uere Reprsentation* -- Frequenzen,
;$ MIDI-Pitches, (Note . Oktave) dotted pairs, musikalische Notenschrift, usw.
;$ -- wird erst im letzten Augenblick durchgefhrt: vgl. -4- unten.
;$   Die ersten Beispiele verwenden absichtlich ziemlich "einfache" Werte, die
;$ sich "sauber" zusammenfgen, um die Richtigkeit der Funktionen zu berprfen.
;+   Fractal melody between given start and end points. The applied melody is
;+ defined as a sequence of tempered intervals.
;+   The computations are performed purely numerically, abstractly. A numerical
;+ *point* can correspond to a degree in any pitch scale. For instance: the
;+ three *points* (2.0 2.5 3.0) could correspond, in a 12 semi-tones tempered
;+ scale, to (C1 F#1 C2) or (F3 G#3 B3) or (D#0 E0 F0), etc. In a frequency
;+ scale, if 2.0 were defined as 440 Hz and 3.0 as 779.657, they would
;+ correspond to the frequencies (440.0 585.7039 779.657). All would depend on
;+ the definition of 2.0 and 3.0 in the chosen *external representation*, which
;+ would be later specified.
;+   Such a purely numerical elaboration allows to work freely in an abstract
;+ dimension--for instance between 0.0 and 1.0, or between any MIDI-pitches,
;+ etc.: cf. -1-, -2- and -3- below.
;+   The description of the intervals defining the motif to be fractalized is
;+ arbitrary. The examples below suggest tempered semi-tone intervals. But only
;+ their relative proportions are really taken into account.
;+   The translation into an *external representation*--frequencies, MIDI-
;+ pitches, (note . octave) dotted pairs, musical notation, etc.-- will be
;+ realized later in the ultimate stage: c. -4- below.
;+   The first examples are intentionally made with "simple" values that fall
;+ quite "neatly" into place, in order to test the validity of the function.

;;; -1- Cas simple, algorithme de base : mlodie entre deux points
;$$ -1- Einfacher Fall, Grundalgorithmus: Melodie zwischen zwei Punkten
;++ -1- Simple case, basic algorithm: melody between two points

;;   Cre une ligne mlodique allant de <haut1>  <haut2>, en enchanant le
;; profil du motif d'intervalles de la liste <lintrvs>. Exemple :
;$   Eine melodische Linie von <haut1> bis <haut2> gehend erstellt, das Profil
;$ des Abstndemotives der Liste <lintrvs> fortfahrend. Beispiel:
;+   Creates a melodic line going from <haut1> to <haut2>, following the profile
;+ of the interval motif of the <lintrvs> list. Example:
;-    (melo '(2 -1 3 1) 0.0 1.0) => (0.0 0.4 0.2 0.8 1.0)

(defun melo (lintrvs haut1 haut2 &aux ambitus)
  (when (zerop (setq ambitus (apply #'+ lintrvs))) (error "Ambitus motif = 0."))
  (let ((pas   (/ (- haut2 haut1) ambitus))
        (cumul haut1)
        (rez   ()   ))
    (dolist (lintrv lintrvs (cons haut1 (reverse rez)))
      (push (incf cumul (* lintrv pas)) rez))))

;;; -2- Ritration de -1- : <n>-1 mlodies entre <n> points
;$$ -2- Wiederholung von -1-: <n>-1 Melodien zwischen <n> Punkten
;++ -2- Reiteration of -1-: <n>-1 melodies between <n> points

;;   Applique un niveau supplmentaire (fractalisation...)  la mlodie
;; <lihauts>, en insrant, entre tous ses points, une micro-ligne mlodique
;; dfinie par le profil des intervalles de <lintrvs>. Exemples :
;$   Fgt der Melodie <lihauts> ein zustzliches Level (Fraktalisierung...)
;$ hinzu, indem eine micro-melodische Linie, bestimmt durch das Intervalprofil
;$ von <lintrvs>, zwischen allen ihren Punkten eingefgt wird. Beispiele:
;+   Applies an extra level (fractalization...) to the <lihauts> melody, by
;+ inserting, between all its points, a micro-melodic line defined by the
;+ interval profile <lintrvs>. Examples:
;-    (melfrac '(2 -1 3 1) '(0.0 0.4 0.2 0.8 1.0)) ou
;-    (melfrac '(2 -1 3 1) (melo '(2 -1 3 1) 0.0 1.0))
;-        => (0.0 0.16 0.08 0.32 0.4 0.32 0.35999998 0.23999998 0.2 0.44 0.32
;-            0.68 0.8 0.88 0.84 0.96 1.0)

(defun melfrac (lintrvs lihauts)
  (labels ((interne (lhts)
             (cond ((endp (cddr lhts)) (melo lintrvs (car lhts) (cadr lhts)))
                   (t (append (butlast (melo lintrvs (car lhts) (cadr lhts)))
                              (interne (cdr lhts)))))))
    (interne lihauts)))

;;; -3- Interface : ritration de -2- sur plusieurs niveaux successifs
;$$ -3- Schnittstelle: Wiederholung von -2- auf vielen aufeinanderfolgenden
;$$     Ebenen
;++ -3- Interface: reiteration of -2- on a number of successsive levels

;;   Interface pour raliser en une seule fois des appels aux deux fonctions
;; -1- et -2- prcdentes, afin d'obtenir <nbr> niveaux mlodiques fractals
;; imbriqus. Exemple :
;$   Schnittstelle zum gemeinsamen Aufruf der obigen zwei Funktionen -1- und -2-
;$ um <nbr> verschachtelte fraktalen melodische Ebenen zu erhalten. Beispiel:
;+   Interface realizing at once the calls to the preceding functions -1- and
;+ -2-, in order to obtain <nbr> fractally nested melodic levels. For instance:
;-    (makemel 2 '(2 -1 3 1) 0.0 1.0)
;-        => (0.0 0.16 0.08 0.32 0.4 0.32 0.35999998 0.23999998 0.2 0.44 0.32
;-            0.68 0.8 0.88 0.84 0.96 1.0)
;;   Le nombre de notes de la mlodie fractale rsultante est
;$   Die Notenzahl der resultierenden fraktalen Melodie ist
;+   The number of notes of the resulting fractal melody is
;-    (1+ (expt (list-length <lintrvs>) <nbr>))

(defun makemel (nbr lintrvs haut1 haut2 &aux rez)
  (setq rez (melo lintrvs haut1 haut2))
  (dotimes (x (decf nbr) rez)
    (setq rez (melfrac lintrvs rez))))

;;; -4.a- Traduction en reprsentation externe en frquences
;$$ -4.a- bertragung auf eine uere Reprsentation in Frequenzen
;++ -4.a- Translation into an external representation in frequencies

;;   Conversion des points de la liste <lihauts> en frquences : le point
;; <ht1> correspondra  la frquence <frq1>, et ainsi <ht2>  <frq2>. Tous les
;; points de <lihauts> sont convertis en frquences en fonction de ces deux
;; points d'ancrage. Exemple :
;$   Konvertierung der Punkte der Liste <lihauts> in Frequenzen: der Punkt <ht1>
;$ entspricht der Frequenz <frq1>, und <ht2> ebenso <frq2>. Alle Punkte der
;$ <lihauts> werden in Frequenzen relativ zu diesen beiden Bezugspunkten
;$ konvertiert. Beispiel:
;+   Conversion of the points of the <lihauts> list into frequencies: the <ht1>
;+ point corresponds to the <frq1> frequency, and <ht2> to <frq2>. All points of
;+ <lihauts> are converted into frequencies relatively to these two reference
;+ points. Example:
;-    (makefrqs 0 1 440.0 880.0 (makemel 2 '(2 -1 3 1) 0.0 1.0))
;-        => (440.0 491.60635 465.08795 549.26544 580.5835 549.26544 564.7074
;-            519.6367 505.42728 596.9059 549.26544 704.94147 766.08453
;-            809.76514 787.6221 855.93634 880.0)

(defun makefrqs (ht1 ht2 frq1 frq2 lihauts)
  (let ((tottemp (- ht2  ht1 ))
        (rapfrq  (/ frq2 frq1)))
    (labels ((interne (lhts)
               (cond ((endp lhts) ())
                     (t (cons (* frq1 (expt rapfrq
                                            (/ (- (car lhts) ht1)
                                               tottemp)))
                              (interne (cdr lhts)))))))
      (interne lihauts))))

;;   Autres exemples, avec valeurs un peu plus "bizarres" -- N.B. : les points
;; ngatifs sont, bien entendu, possibles :
;$   Andere Beispiele, mit ein wenig "bizarreren" Werten -- N.B.: negative
;$ Punkte sind selbstverstndlich mglich:
;+   Other examples, with a little more "bizarre" values--N.B.: negative points
;+ are of course possible:
;-    (makemel 1 '(3 -7 13) 33.0 97.0)
;-        => (33.0 54.333336 4.5555573 97.0)
;-    (makemel 2 '(3 -7 13) 33.0 97.0)
;-        => (33.0 40.11111 23.518517 54.333336 37.740746 76.456795 4.5555573
;-            35.37037 -36.53086 96.99999)
;-    (makemel 3 '(3 -7 13) 33.0 97.0)
;-        => (33.0 35.37037 29.839504 40.11111 34.580246 47.485596 23.518517
;-           33.790123 9.82304 54.333336 48.80247 61.707817 37.740746 50.646095
;-           20.533613 76.456795 52.489716 108.4129 4.5555573 14.827161 -9.13991
;-           35.37037 11.403292 67.32647 -36.53086 7.9794235 -95.87791 96.99999)
;-    (makefrqs 33.0 97.0 880.0 1999.0 (makemel 3 '(3 -7 13) 33.0 97.0))
;-        => (880.0 907.15204 845.0572 963.9953 898.0095 1059.5795 779.2791
;-            888.9591 653.7945 1156.803 1077.6194 1271.5048 935.1419 1103.3927
;-            750.02216 1536.1558 1129.7823 2313.9604 611.10205 697.1119 512.698
;-            907.15204 667.1747 1366.4719 360.8765 638.5232 168.6299 1998.9999)

;;; -4.b- Traduction en reprsentation externe en MIDI-pitches
;$$ -4.b- bertragung auf MIDI-Pitches in der ueren Reprsentation
;++ -4.b- Translation into a MIDI-pitches external representation

;;   Conversion des points de la liste <lihauts> en MIDI-pitches : le point
;; <ht1> correspondra au MIDI-pitch <pit1>, et ainsi <ht2>  <pit2>. Tous les
;; points de <lihauts> sont convertis et arrondis en MIDI-pitches en fonction
;; de ces deux points d'ancrage.
;;   En donnant la valeur t au mot-clef :midi-cents [nil par dfaut], les
;; arrondis se feront en MIDI-cents (Do central := 6000).
;;   Noter la parfaite similitude avec la fonction makefrqs ci-dessus. Cette
;; dernire agit dans le domaine multiplicatif des rapports de frquences,
;; tandis que la prsente fonction, elle, agit dans le domaine additif d'une
;; chelle de degrs temprs : une multiplication de makefrqs est donc ici
;; remplace par une addition, et une exponentiation par une multiplication.
;;   Exemples simples, correspondant  ceux du tout dbut du prsent fichier :
;$   Konvertierung der Punkte der Liste <lihauts> in MIDI-Pitches: der Punkt
;$ <ht1> entspricht dem MIDI-Pitch <pit1>, und <ht2> ebenso <pit2>. Alle Punkte
;$ der <lihauts> werden in MIDI-Pitches relativ zu den beiden Bezugpunkten
;$ konvertiert und gerundet.
;$   Mit dem t Wert des :midi-cents Schlsselwortes  [standardmig nil], wird 
;$ auf MIDI-cents gerundet (mittleres C := 6000).
;$   Man beachte die perfekte Gleichartigkeit zur obigen makefrqs Funktion.
;$ Letztere arbeitet im multiplikativen Bereich der Frequenzbeziehungen, whrend
;$ diese Funktion im additiven Bereich einer temperierten chromatischen
;$ Tonleiter arbeitet: eine Multiplikation in makefrqs wird daher hier durch 
;$ eine Addition ersetzt, eine Potenzierung durch eine Multiplikation.
;$   Einfache Beispiele, hnlich denen am Anfang dieser Datei:
;+   Conversion of the points of the <lihauts> list into MIDI-pitches: the <ht1>
;+ point corresponds to the <pit1> MIDI-pitch, and <ht2> to <pit2>. All the
;+ points of <lihaut> are converted and rounded into MIDI-pitches relatively to
;+ these two reference points.
;+   By specifyinf t as value of the :midi-cents key-word [nil by default], the
;+ roundings will be performed in MIDI-cents (Middle C := 6000).
;+   Note the perfect similarity with the above makefrqs function. The latter
;+ acts in the multiplicative domain of frequencies, while this function acts in
;+ the additive domain of a tempered pitch scale: a multiplication in makefrqs
;+ is thus here replaced by an addition, and an exponentiation by a
;+ multiplication.
;+   Simple examples, corresponding to the very first of the present file:
;-    (makepits 2 3 36 48 '(2.0 2.5 3.0)) => (36 42 48) := ( Do1  Fa#1 Do2) 
;-    (makepits 2 3 65 71 '(2.0 2.5 3.0)) => (65 68 71) := ( Fa3 Sol#3 Si3)
;-    (makepits 2 3 27 29 '(2.0 2.5 3.0)) => (27 28 29) := (Re#0   Mi0 Fa0)
;;   Autres exemples -- on remarque l'effet des arrondis (cf. primitive round)
;; et du mot-clef :midi-cents :
;$   Andere Beispiele -- man beachte den Effekt der Rundungen (Vgl. Primitive
;$ round) und des Schlsselworts :midi-cents:
;+   Other examples--note the effects of the roundings (cf. round primitive) and
;+ of the :midi-cents keyword:
;-    (makepits 0 1 60 70 '(0.0 0.4 0.2 0.8 1.0 -1.0 0.5 -0.2))
;-        => (60   64   62   68   70   50   65   58  )
;-    (makepits 0 1 60 70 '(0.0 0.4 0.2 0.8 1.0 -1.0 0.5 -0.2) :midi-cents t)
;-        => (6000 6400 6200 6800 7000 5000 6500 5800)
;-    (makepits 0 1 60 72 '(0.0 0.4 0.2 0.8 1.0 -1.0 0.5 -0.2))
;-        => (60   65   62   70   72   48   66   58  )
;-    (makepits 0 1 60 72 '(0.0 0.4 0.2 0.8 1.0 -1.0 0.5 -0.2) :midi-cents t)
;-        => (6000 6480 6240 6960 7200 4800 6600 5760)
;-    (makepits 0 1 60 84 '(0.0 0.4 0.2 0.8 1.0 -1.0 0.5 -0.2))
;-        => (60   70   65   79   84   36   72   55  )
;-    (makepits 0 1 60 84 '(0.0 0.4 0.2 0.8 1.0 -1.0 0.5 -0.2) :midi-cents t)
;-        => (6000 6960 6480 7920 8400 3600 7200 5520)
;-    (makepits 33.0 97.0 69 85 (makemel 3 '(3 -7 13) 33.0 97.0))
;-        => (69   70   68   71   69   73   67   69   63   74   73   76   70
;-            73   66   80   74   88   62   64   58   70   64   78   52   63
;-            37   85  )                 ;cf. dernier exemple de -4.a- ci-dessus
;-    (makepits 33.0 97.0 69 85 (makemel 3 '(3 -7 13) 33.0 97.0) :midi-cents t)
;-        => (6900 6959 6821 7078 6940 7262 6663 6920 6321 7433 7295 7618 7019
;-            7341 6588 7986 7387 8785 6189 6446 5847 6959 6360 7758 5162 6274
;-            3678 8500)                                                   ;idem
 
(defun makepits (ht1 ht2 pit1 pit2 lihauts &key midi-cents)
  (let ((tottemp (- ht2  ht1 ))
        (totpit  (- pit2 pit1))
        (arrondi (if midi-cents 0.01 1.0)))
    (labels ((interne (lhts)
               (cond ((endp lhts) ())
                     (t (cons (round (+ pit1 (* totpit
                                                (/ (- (car lhts) ht1)
                                                   tottemp))) arrondi)
                              (interne (cdr lhts)))))))
      (interne lihauts))))

;;; -4.c- Traduction en reprsentation externe en paires pointes
;;;       (note . octave)
;$$ -4.c- bertragung auf eine uere Reprsentation in dotted pairs
;$$       (Note . Oktave)
;++ -4.c- Translation into a dotted pair external representation (note . octave)

;;   On peut procder  partir des MIDI-pitches... en appliquant facilement une
;; fonction dj vue au Chapitre 12 (cf. fichier exo.12) : midin>haut -- ou
;; midi>NumNot de l'environnement dlo CLCE-IV. Il suffit de faire
;$   Man kann von den MIDI-Pitches ausgehen... man wendet einfach eine Funktion
;$ aus dem 12. Kapitel an (Vgl. Datei exo.12): midin>haut -- oder midi>NumNot
;$ der dlo CLCE-IV-Umgebung. Es ist so einfach wie
;+   One can proceed from the MIDI-pitches... simply by applying a function seen
;+ in Chapter 12 (cf. exo.12 file): midin>haut -- or midi>NumNot of the dlo
;+ CLCE-IV environment. It is as simple as
;-    (mapcar #'midin>haut (makepits ... ))


