(defun put (symb propr val)                ;N.B. : au lieu de charger tout 16.1a
  (setf (get symb propr) val))
(format t "~&; ... put de ped.16.1a~%")
;- <1> Definition des Wurzel-Paths
(setq common-lisp::*wurzel-path*
  (merge-pathnames
    (make-pathname :directory (pathname-directory *load-truename*)) "*"))
;- <2> Zustzliche Aufladung(en)
(dolist (lefic '(
                 "ped.11.3.lisp"                 ;N.B. : 11.3 charge lui-mme 08
                 "ped.12.lisp"
                 "ped.13.lisp"
                 "ped.15.lisp"
                 ))
  (load (setq lefic (merge-pathnames lefic common-lisp::*wurzel-path*)))
  (format t "~&;; Geladen ~S~%" lefic))
;===============================================================================
;;; 16.1b --> P-LISTES : exemple d'application  la gestion de familles d'objets
;;;           musicaux engendrs les uns des autres
;$$ 16.1b --> P-LISTEN: Beispielanwendung fr die Verwaltung von Familien
;$$           musikalischer Objekte, die auseinander generiert werden [d..H.B.]
;++ 16.1b --> P-LISTS: Example Application to the Management of Families of
;++           Musical Objects Generated One from Another
;===============================================================================

;;   Des objets peuvent tre donns arbitrairement, ou engendrs les uns des
;; autres par transformation(s) d'objet(s) pr-existant(s).
;$   Objekte knnen beliebig eingegeben oder aus einander durch
;$ Transformation(en) vorhandenes(er) Objekts(e) erzeugt werden.
;+   Objects can be given arbitrarily, or generated one from another through
;+ transformation(s) of pre-existing object(s).

;;   La C-val (Cell-Value) d'un objet est une liste de notes, de forme
;$   Der C-val (Zellen-Wert) eines Objekts ist eine Liste von Noten der Form
;+   The C-val (Cell-Value) of an object is a list of notes of form
;-    ( <note1> ... <note<n>> )
;; o <note> est une paire pointe de forme
;$ wobei <note> ein dotted pair der Form
;+ where <note> is a dotted pair of form
;-    (<num> . <oct>) avec / mit / with
;;        <num> entier dans [0,11] := numro de note, et
;;        <oct> entier := numro d'octave
;$        <num> Ganzzahl in [0,11] := Notennummer, und
;$        <oct> Ganzzahl := Nummer der Oktave ist
;+        <num> integer in [0,11] := note number, and
;+        <oct> integer := octave number

;;   Gestion d'objets engendrs les uns des autres : conservation de l'origine
;; de chacun dans sa P-liste, sous forme d'une proprit <pere> code par le
;; couple
;$   Verwaltung der aus einander erzeugten Objekte: der Ursprung eines jeden
;$ Objekts wird in Form einer <pere>-Eigenschaft in seiner P-Liste gespeichert,
;$ die durch folgendes Wertpaar codiert wird
;+   Management of objects generated one from another: the origin of each object
;+ will be saved in its P-list, in the form of a <pere> property coded by the
;+ couple
;-    pere (<tr> <orig> [<argums>])
;; lorsque l'objet a t obtenu par application de la fonction <tr>  l'objet
;; pr-existant <orig>, avec l(es) argument(s) optionnel(s) <argums>.
;;   Si un objet est donn arbitrairement par l'utilisateur, c'est--dire qu'il
;; n'est pas issu de la transformation d'un autre, sa proprit de P-liste
;; <pere> sera NIL.
;;   On a la possibilit d'utiliser une proprit de P-liste <commentaire> pour
;; associer un commentaire descriptif  un objet donn.
;$ falls das Objekt durch die Anwendung der Funktion <tr> auf das vorhandene
;$ Objekt <orig> mit (dem/den) eventuellen Argument(en) <argums> entstand.
;$   Falls ein Objekt beliebigerweise vom Nutzer eingegeben wird, also wenn es
;$ nicht durch eine Transformation eines anderen entsteht, ist seine
;$ <pere>-P-Listen-Eigenschaft NIL.
;$   Die P-Listen-Eigenschaft <commentaire> ist vorhanden um einen erklrenden
;$ Kommentar mit einem beliebig Objekt zu verknpfen.
;+ when the object results from the application of the <tr> function to the
;+ pre-existing object <orig>, with the eventual argument(s) <argums>.
;+   If an object is given arbitrarily by the user, that is if it does not come
;+ out of the transformation of another one, its <pere> P-list property will be
;+ NIL.
;+   The P-list property <commentaire> ist available to associate a descriptive
;+ statement to a given object.

;;   La variable *les_objets* contiendra une liste de tous les objets connus.
;$   Die *les_objets*-Variable enthlt eine Liste aller bekannten Objekte.
;+   The *les_objets* variable will contain a list of all known objects.

(if (boundp '*les_objets*) (setq *les_objets* ())
(defparameter *les_objets* ()))

;;   UNIX La commande top-level :o permet de consulter facilement *les_objets*.
;$   UNIX Der top-level Befehl :o erlaubt einfaches Auslesen von *les_objets*.
;+   UNIX The top-level command :o allows to read easily *les_objets*.

;(top-level:alias "o" (&rest bof)
;    (format t "~&*les_objets* :~%~4T~A" *les_objets*))

;;   Qu'un objet soit donn arbitrairement, ou cr par transformation d'un
;; autre, il est toujours ncessaire de passer par la fonction suivante pour sa
;; cration :
;$   Ein Objekt muss immer durch folgende Funktion erzeugt werden, egal, ob es
;$ beliebigerweise eingegeben oder durch Transformation eines anderen erzeugt
;$ wurde: 
;+   Whether an object is given arbitrarily, or created by the transformation of
;+ another, it is always compulsory to create it through the following function:
;-    crobjet <valeur> [:nom (<nom> NIL) :commentaire (<commentaire> NIL)]
;; Cette fonction inscrit le nouvel objet <nom> dans *les_objets*, lui attribue
;; sa valeur, et gre la proprit commentaire de sa P-liste. La proprit
;; <pere> n'est pas affecte ici : soit elle sera initialise par la fonction
;; transfo (cf. ci-dessous), soit elle restera indfinie (quivalant  NIL) dans
;; le cas d'un objet donn arbitrairement par l'utilisateur.
;;   Lorsque <nom> = NIL par dfaut, on cre un nom avec la primitive gentemp, 
;; partir du radical "D_OBJ-".
;;   La fonction utilitaire put est dfinie dans le fichier ped.16.1a.
;$ Diese Funktion trgt das neue Objekt <nom> in *les_objets* ein, setzt seinen
;$ Wert und verwaltet die Eigenschaft commentaire seiner P-Liste. Die <pere>-
;$ Eigenschaft wird hier nicht tangiert: entweder wird sie durch die Funktion
;$ transfo initialisiert (vgl. unten), oder sie bleibt undefiniert (gleich NIL)
;$ im Fall eines beliebigerweise vom Nutzer eingegebenen Objekts.
;$   Falls <nom> = NIL defaultmig wird ein Name durch das Primitive gentemp
;$ aus der Wurzel "D_OBJ-" erzeugt.
;$   Die Hilfsfunktion put ist in Datei ped.16.1a definiert.
;+ This function enrolls the new object <nom> in *les_objets*, sets its value,
;+ and manages the commentaire property of its P-list. The <pere> property is
;+ not affected here: either it will be initialized by the transfo function (cf.
;+ below), or it will remain undefined (equivalent to NIL) in the case of an
;+ object arbitrarily given by the user.
;+   When <nom> = NIL by default, a name is created through the gentemp
;+ primitive with root "D_OBJ-".
;+   The put utility function is defined in file ped.16.1a.

(defun crobjet (val &key nom commentaire)
  (when (null nom) (setq nom (gentemp "D_OBJ-")))
  (when (atom val)
    (error "crobjet : valeur non utilisable pour ~A : ~S." nom val))
  (terpri) (princ ";") (princ nom) (princ " => ") (princ (set nom val))
  (put nom 'commentaire commentaire)
  (push nom *les_objets*)
  'ok)

;;   Fonction de transformation, destine  engendrer un nouvel objet par
;; application d'une fonction  un objet pr-existant, avec les arguments
;; d'appel appropris :
;;    transfo <fonction--appliquer> <objet-pr-existant> [<argument(s)>]
;; Les fonctions suivantes peuvent tre appliques dans la prsente
;; implmentation simple, par exemple :
;$   Transformationsfunktionen um ein neues Objekt durch Anwendung einer
;$ Funktion auf ein vorhandenes Objekt zu erzeugen, mit geeigneten
;$ Aufrufparametern:
;$    transfo <anzuwendende-Funktion> <vorhandenes-Objekt> [<Argument(e)>]
;$ Die folgenden Funktionen knnen in dieser einfachen Implementierung
;$ z. B. angewendet werden:
;+   Transformation function used to generate a new object by applying a
;+ function to a pre-existing object, with the appropriate calling arguments:
;+    transfo <function-to-be-applied> <pre-existing-object> [<argument(s)>]
;+ The following functions can be applied in this simple implementation, for
;+ instance:
;-    cdr, etc. <objet>       -- primitive
;-    elt       <objet> [<n>] --     "
;-    append   {<objet>}+     --     "
;-    reverse   <objet>       --     "
;-    last      <objet> [<n>] --     "
;-    butlast   <objet> [<n>] --     "
;-    firstn2   <objet> [<n>] -- cf. ped.08
;-    palindroA <objet>       --     "
;-    palindroB <objet>       --     "
;-    rotag2    <objet> [<n>] -- cf. ped.11.3
;-    rotad2    <objet> [<n>] --     "
;-    croitre1  <objet>  <n>  -- cf. ped.12
;-    transpoz2 <objet>  <n>  -- cf. ped.13
;-    rplaca    <objet>  <v>  -- cf. ped.15
;-    rplacd    <objet>  <v>  --     "
;-    permuter1 <objet>       --     "
;-    etc. / u.s.w. / etc.
;;   Un nom est cr par la primitive gentemp pour le nouvel objet,  partir du
;; radical "T_OBJ-".
;$   Ein neuer Objektname wird durch das Primitive gentemp aus der Wurzel
;$ "T_OBJ-" erzeugt.
;+   A name is created for the new object through primitive gentemp, with root
;+ "T_OBJ-".

(defun transfo (fonction objet &rest arguments)
  (cond ((not (member objet *les_objets*))
         (error "~A : objet inconnu." objet))
        (t (let ((nom (gentemp "T_OBJ-")))
             (crobjet
              (apply fonction
                     (cons (symbol-value objet) arguments))
              :nom nom
              :commentaire "transfo")
             (put nom 'pere (cons fonction (cons objet arguments)))))))

;;   Fonction de recherche de l'historique d'un objet : remonte de pre en pre,
;; jusqu' un objet sans pre -- c'est--dire cr directement par
;; l'utilisateur, et non pas obtenu par transformation d'un autre objet
;; pr-existant. La fonction rend, de droite  gauche, une liste des
;; transformations successives ayant engendr l'objet final. Par convention, le
;; premier lment de cette liste -- le plus  droite -- est le commentaire
;; descriptif de l'objet qui est donc  la racine de la ligne.
;$   Funktion zur Erforschung der Geschichte eines Objekts: sucht rckwrts von
;$ Vater zu Vater bis es ein vaterloses Objekt findet -- dies ist das Objekt,
;$ das direkt vom Nutzer erzeugt wurde, nicht durch Transformation eines anderen
;$ vorhandenen Objekts. Die Funktion gibt von rechts nach links eine Liste der
;$ aufeinanderfolgenden Transformationen zurck, die das Endobjekt erzeugt
;$ haben. Das erste Glied -- ganz rechts -- dieser Liste ist laut Vereinbarung
;$ der erklrende Kommentar des Objekts, welches am Anfang der Abstammungslinie
;$ steht.
;+   Function tracing the history of an object: traces back from father to
;+ father until it reaches a fatherless object--that is an object directely
;+ created by the user, not generated through the transformation of another pre-
;+ existing object. The function returns, from right to left, a list of the
;+ successive transformations that have generated the final object. By
;+ convention, the first--rightmost--element of this list is the descriptive
;+ comment of the object thus situated at the origin of the lineage.

(defun histo (obj)
  (let ((papa (get obj 'pere)))
    (if (null papa)
      (list (get obj 'commentaire))
    (cons papa (histo (cadr papa))))))

;;   UNIX La commande top-level :h facilite la consultation de l'historique.
;$   UNIX Der top-level Befehl :h erleichtert den Zugriff auf die Geschichte.
;+   UNIX The top-level command :h facilitates access to the historical trace.

;(top-level:alias "h" (objet)
;    (princ (histo objet)))

;;   UNIX La commande top-level :pl permet d'accder facilement  la P-liste
;; d'un objet.
;$   UNIX Der top-level Befehl :pl erlaubt einfachen Zugriff auf die P-Liste
;$ eines Objekts.
;+   UNIX The top-level command :pl allows easy access to an object's P-list.

;(top-level:alias "pl" (objet)
;    (princ (symbol-plist objet)))

;;   La fonction suivante permet de  sauvegarder dans un fichier l'environnement
;; constitu au cours d'une sance de travail :
;$   Die folgende Funktion erlaubt es, eine Umgebung, die ber eine
;$ Arbeitssitzung erstellt wurde, in einer Datei zu speichern:
;+   The following function allows to save in a file an environment elaborated
;+ during a work session:
;-    sauvegarde "<fichier>"

(defun sauvegarde (fichier)
 (let ((orig-p1 *print-length*)
       (orig-p2 *print-level*))
  (setq *print-length* nil)       ;pour s'assurer que l'impression ne soit pas
  (setq *print-level* nil)        ;limite par l'utilisateur, mme  son insu...
  (with-open-file (flux fichier :direction         :output
                                :if-exists         :rename-and-delete
                                :if-does-not-exist :create)
    (format flux "~&;;;-------------------------------------------~%~
                    ;;; Sauvegarde, environnement type ped.16.1b :~%~
                    ;;; ~A~%~
                    ;;;-------------------------------------------~%"
                 (jour-date))
    (format flux "
((lambda ()                                ;ceci equivaudra a un appel (FlyTox),
   (when (boundp '*les_objets*)                              ;au  cas  ou  cette
     (labels ((intrn (lis)                                   ;fonction ne serait
                (if (endp lis) t                             ;pas encore dfinie
                  (progn (makunbound (car lis))              ;au  moment  de  la
                    (intrn (cdr lis))))))                    ;restauration...
       (intrn (symbol-value '*les_objets*))))))~%")
    (format flux "(if (boundp '*les_objets*) (setq *les_objets* ())~
                  (defparameter *les_objets* ()))~2%")
    (format flux ";re-creation des objets~%")
    (labels ((sauve (listobj) (cond
                ((null listobj) nil)
                (t (let ((objet (car listobj)))
                     (format flux "(push '~S *les_objets*)~%" objet)
                     (format flux "(setq ~S '~S)~%"
                                  objet (symbol-value objet))
                     (format flux "(setf (symbol-plist '~S) '~S)~2%"
                                  objet (symbol-plist objet))
                     (sauve (cdr listobj)))))))
      (sauve *les_objets*))
    (format flux ";impression des objets : <nom> => <valeur>
(apply
 (lambda (liste)
   (labels ((interne (li &aux obj)
            (when li
              (terpri) (princ \";\") (princ (setq obj (car li)))
              (princ \" => \") (princ (symbol-value obj))
              (interne (cdr li)))))
     (interne liste))) (list *les_objets*))~%")
    (format flux "~%;remise en ordre normal de *les_objets*~%")
    (format flux "(setq *les_objets* (reverse *les_objets*))~%")
    (format flux "(format t \"~~%;*les_objets* => ~~A~~%\" *les_objets*)~%")
    (format flux "
;impression des historiques
(apply
  (lambda (liste)
    (labels
        ((interne (li)
           (if (endp li) 'ok
             (progn (format t \";(symbol-plist '~~A) => ~~S~~%\"
                               (car li) (symbol-plist (car li)))
                    (interne (cdr li))))))
         (interne liste))) (list (reverse *les_objets*)))~%"))
  (setq *print-length* orig-p1)          ;restaurer les valeurs orig. utilsateur
  (setq *print-level* orig-p2))
  (format nil "Sauvegarde ok, fichier ~A." fichier))

;;   Fonction liminant tous les symboles numrs dans *les_objets*. Une forme
;; lambda, quivalente  un appel  FlyTox, est place au dbut d'une sauvegarde
;; (cf. ci-dessus), afin, lors de son (re)chargement, de dtruire toute trace
;; d'un ventuel tat antrieur de l'environnement.
;$   Funktion, die alle in *les_objets* aufgezhlten Symbole zerstrt. Ein
;$ Lambda-Ausdruck, equivalent einem Aufruf von FlyTox, wird am Anfang einer
;$ gespeicherten Datei (Vgl. oben) eingefgt, um Spuren eines vorhergehenden
;$ Umgebungszustands zu vernichten, wenn es (neu)geladen wird.
;+   Function eliminating all symbols enumerated *les_objets*. A lambda form,
;+ equivalent to a call to FlyTox, is inserted at the beginning of a file save
;+ (cf. above), in order, when it will be (re)loaded, to destroy traces of any
;+ preceding state of the environment.

(defun FlyTox ()
  (when (boundp '*les_objets*)              ;mme si *les_objets* n'existait pas
    (labels ((intrn (lis)
               (if (endp lis) t
                 (progn (makunbound (car lis))
                   (intrn (cdr lis))))))
      (intrn *les_objets*))))

;;   Chargement des exemples de base.
;$   Ladung der Basis-Beispiele.
;+   Load the basic examples.

(when (??defaut "Charger exemples de base : O(Oui)/NIL" nil)
  (FlyTox)
  (setq *les_objets* nil)
  (load (merge-pathnames "ped.16.1b.ex" common-lisp::*wurzel-path*)))
