;;; Cf. [DELAHAYE, J.-P., 2008 : "Dconcertantes conjectures", Pour la science :
;;; 367 : 96]

;;   Suite rcurrente g(n) de Benoit Cloitre :
;;    g(1) = 1
;;    g(n) = g(n-1) + lcm(n,g(n-1))
;; o lcm(a,b) est le plus petit commun multiple (ppcm) de a et b. On obtient
;; une suite croissant assez rapidement. Ex. :
;;    (BenoitCloitre-gn <nbr>) => <nbr>+1 premiers termes de la suite g(n) de
;;                                Benoit Cloitre

(defun BenoitCloitre-gn (nbr &aux (gn 1) (cpt 1))
  (format t "~12D : ~D~%" cpt gn)
  (dotimes (etp nbr 'ok)
    (format t "~12D : ~D~%" (incf cpt) (setq gn (+ gn (lcm cpt gn))))))

;;   En posant
;;    a(n) = (g(n) / g(n-1)) - 1
;; on obtient des 1 et des nombres premiers -- 2 inclus, mais 3 exclu.  Ex. :
;;    (BenoitCloitre-an-avec-1 <nbr>) => <nbr> premiers termes de la suite a(n)
;;                                       de Benoit Cloitre

(defun BenoitCloitre-an-avec-1 (nbr &aux (gn-1 1) (gn 3) (cpt 2))
  (dotimes (etp nbr 'ok)
    (format t "~12D : ~D~%" (1- cpt) (- (/ gn gn-1) 1))
    (setq gn-1 gn)
    (setq gn (+ gn-1 (lcm (incf cpt) gn-1)))))

;;   En supprimant les 1 de la suite a(n), par ex. :
;;    (BenoitCloitre-an-sans-1 <nbr>)
;;        => <nbr> premiers termes de la suite a(n) de Benoit Cloitre sans
;;           impression des 1

(defun BenoitCloitre-an-sans-1 (nbr &aux (gn-1 1) (gn 3) (cpt 2) an)
  (dotimes (etp nbr 'ok)
    (when (/= 1 (setq an (- (/ gn gn-1) 1)))
      (format t "~12D : ~D~%" (1- cpt) an))
    (setq gn-1 gn)
    (setq gn (+ gn-1 (lcm (incf cpt) gn-1)))))

;;   Autre suite rcurrente de Benoit Cloitre :
;;    h(1) = 1
;;    h(n) = 2(h(n-1)) + lcm(n,h(n-1))
;; On obtient une suite de nombres pairs croissant assez rapidement. Ex. :
;;    (BenoitCloitre-hn <nbr>)
;;        => <nbr>+1 premiers termes de la suite h(n) de Benoit Cloitre

(defun BenoitCloitre-hn (nbr &aux (hn 1) (cpt 1))
  (format t "~12D : ~D~%" cpt hn)
  (dotimes (etp nbr 'ok)
    (format t "~12D : ~D~%" (incf cpt) (setq hn (+ (* 2 hn) (lcm cpt hn))))))

;;   En posant
;;    b(n) = (h(n+1) / h(n)) - 2
;; on obtient des 1 et des nombres premiers. Ex. :
;;    (BenoitCloitre-bn-avec-1 <nbr>)
;;        => <nbr> premiers termes de la suite b(n) de Benoit Cloitre

(defun BenoitCloitre-bn-avec-1 (nbr &aux (hn 1) (hn+1 4) (cpt 2))
  (dotimes (etp nbr 'ok)
    (format t "~12D : ~D~%" (1- cpt) (- (/ hn+1 hn) 2))
    (setq hn hn+1)
    (setq hn+1 (+ (* 2 hn) (lcm (incf cpt) hn)))))

;;   En enlevant les 1, par ex. :
;;    (BenoitCloitre-bn-sans-1 <nbr>)
;;        => <nbr> premiers termes de la suite b(n) de Benoit Cloitre sans
;;           impression des 1
;; N.B. : les nombre premiers suprieurs  7 qui sont les seconds d'un couple de
;; nombres premiers jumeaux n'apparaissent pas ! Par ex. : 31, second du couple
;; (29,31).

(defun BenoitCloitre-bn-sans-1 (nbr &aux (hn 1) (hn+1 4) (cpt 2) bn)
  (dotimes (etp nbr 'ok)
    (when (/= 1 (setq bn (- (/ hn+1 hn) 2)))
      (format t "~12D : ~D~%" (1- cpt) bn))
    (setq hn hn+1)
    (setq hn+1 (+ (* 2 hn) (lcm (incf cpt) hn)))))
