;;; Cf. [DELAHAYE, J.-P., 2008 : "Dconcertantes conjectures", Pour la science :
;;; 367 : 96-97]

;;   Suite rcurrente de Matt Frank :
;;    f(1) = 7
;;    f(n) = f(n-1) + gcd(n,f(n-1))
;; On obtient une suite graduellement croissante. Ex. :
;;    (MattFrank-fn <nbr>) => <nbr>+1 premiers termes de la suite

(defun MattFrank-fn (nbr &aux (fn 7) (cpt 1))
  (format t "~12D : ~D~%" 1 fn)
  (dotimes (etp nbr 'ok)
    (format t "~12D : ~D~%" (incf cpt) (setq fn (+ fn (gcd cpt fn))))))

;;   Si l'on fait la diffrence entre les f(n) successifs
;;    f(n) - f(n-1) = gcd(n,f(n-1))
;; on obtient des 1 et des nombres premiers globalement croissants. Ex. :
;;    (MattFrank-diff-avec-1 480) => suite de Sloane A132199, cf. pp. 96

(defun MattFrank-diff-avec-1 (nbr &aux (fn 7) (cpt 1))
  (dotimes (etp nbr 'ok)
    (format t "~12D : ~D~%" (incf cpt) (gcd cpt fn))
    (setq fn (+ fn (gcd cpt fn)))))

;;   En supprimant les 1 de la suite prcdente, on obtient des nombres premiers
;; graduellement croissants par saccades, ces dernires tant ensuite
;; individuellement croissantes puis dcroissantes. Ex. :
;;    (MattFrank-diff-sans-1 7777777) => suite de Sloane A13761, cf. pp. 96-97
;; N.B. : rsultats saccads. Chaque long temps d'attente aboutit  un plus
;; grand nombre premier, suivi de quelques autres grosso modo dcroissants.
;;   Permet de voir que chaque nouveau grand <N> est justement le terme numro
;; <N> de la suite !

(defun MattFrank-diff-sans-1 (nbr &aux (fn 7) (cpt 1) tmp) tmp
  (dotimes (etp nbr 'ok)
    (when (/= 1 (setq tmp (gcd (incf cpt) fn)))
      (format t "~12D : ~D~%" cpt (gcd cpt fn)))
    (setq fn (+ fn (gcd cpt fn)))))
