(defun put (symb propr val)                ;N.B. : au lieu de charger tout 16.1a
  (setf (get symb propr) val))
(format t "~&; ... put de ped.16.1a~%")
;; <1> Definition des Wurzel-Paths
(setq common-lisp::*wurzel-path*
  (merge-pathnames
    (make-pathname :directory (pathname-directory *load-truename*)) "*"))
;; <2> Zustzliche Aufladung(en)
(dolist (lefic '(
                 "ped.11.1.lisp"
                 ))
  (load (setq lefic (merge-pathnames lefic common-lisp::*wurzel-path*)))
  (format t "~&;; Geladen ~S~%" lefic))
(format t "~&; ... exo.11.lisp~%")
(load (merge-pathnames  "exo.11.lisp" common-lisp::*PathLocPedExo*))
(trace fibo1)
;===============================================================================
;;; 16.1c --> P-LISTES : exemple d'application  des mmo-fonctions capables
;;;           d'apprentissage
;$$ 16.1c --> P-Listen: Beispielanwendung auf lernfhige Memo-Funktionen
;$$                                                                   [d..H.B.]
;++ 16.1c --> P-LISTS: Example Application to Memo-Functions Capable of Learning
;===============================================================================

;;   Mmo-fonction pour la srie de Fibonacci : la P-liste du symbole fibo3 est
;; utilise comme mmoire d'apprentissage. Cf. [Wertz:59].
;;    fibo3 <n> => <n>-ime terme de la srie de Fibonacci
;;   La fonction utilitaire put est dfinie dans le fichier ped.16.1a. Pour les
;; besoins de la dmonstration, la fonction fibo3 est rendue traable  tous
;; niveaux en Macintosh Common LISP (MCL) -- forme declare : cf. ped.11.1.
;$   Memo-Funktion fr die Fibonacci-Folge: die P-Liste des fibo3-Symbols wird
;$ als lernfhiger Speicher benutzt. Vgl. [Wertz:59].
;$    fibo3 <n> => <n>tes Glied der Fibonacci-Folge
;$   Die Hilfsfunktion put wird in Datei ped.16.1a definiert. Aus
;$ Veranschaulichungsgrnden wurde die fibo3-Funktion in Macintosh Common LISP
;$ (MCL) auf allen Ebenen rckverfolgbar gemacht -- declare-Anweisung: Vgl.
;$ ped.11.1.
;+   Memo-function for the Fibonacci series: the P-list of the fibo3 symbol is
;+ used as learning memory. Cf [Wertz:59]. 
;+    fibo3 <n> => <n>-th term of the Fibonacci series
;+   The put utility function is defined in file ped.16.1a. For the sake of
;+ demonstration, the fibo3 function is made tracable at all levels in Macintosh
;+ Common LISP (MCL)--declare form: cf. ped.11.1.

(defun fibo3 (n) (declare (notinline fibo3))
  (cond ((zerop n) 1)                                      ;cas terminal
        ((= n 1) 1)                                        ;cas terminal
        ((get 'fibo3 n))                                   ;retour si dj connu
        (t (put 'fibo3 n                                   ;sinon : calcul, ...
                (+ (fibo3 (- n 1))                         ;... recursion, ...
                   (fibo3 (- n 2)))))))                    ;... et retour

(setf (symbol-value 'fibo3) 'F-apprenti)
(trace fibo3)

;;   Version amliore et blinde de la prcdente -- cependant non traable.
;; Cf. [Wertz:60].
;$   Verbesserte und verstrkte Version des vorhergehenden -- allerdings nicht
;$ rckverfolgbar. Vgl. [Wertz:60].
;+   Improved and reinforced version of the preceding--although not tracable.
;+ Cf. [Wertz:60].

(defun fibo4 (n)
  (if (or (not (integerp n)) (minusp n))                   ;par scurit
    (error "fibo4 : ~A : argument non valable." n)
  (progn
    (when (null (symbol-plist 'fibo4))                     ;si "vierge"
      (setf (symbol-plist 'fibo4) '(1 1 0 1)))
    (labels ((fibaux (num)
               (cond ((get 'fibo4 num))                    ;retour si dj connu
                     (t (put 'fibo4 num                    ;sinon : calcul, ...
                             (+ (fibaux (- num 1))         ;... recursion, ...
                                (fibaux (- num 2))))))))   ;... et retour
      (fibaux n)))))

(setf (symbol-value 'fibo4) 'F-apprenti)
(trace fibo4)                                              ;trace sans utilit

;;   Eratosthne a dfini un algorithme pour dterminer la primalit d'un
;; nombre : le crible d'Eratosthne. Cette procdure ncessite la connaissance
;; de tous les nombres premiers infrieurs au nombre sur lequel on doit se
;; prononcer. Il s'agit donc d'une procdure qui peut s'avrer assez laborieuse.
;; Mais on pourra considrablement faciliter la tche de l'algorithme si on le
;; dote de la capacit d'acqurir, en rptant l'exprience, la mmoire
;; pralable d'une liste de plus en plus longue de nombres premiers dj
;; reprs.
;;   La mmo-fonction suivante possde cette capacit d'apprentissage : tout en
;; appliquant le crible d'Eratosthne pour se prononcer sur les nombres qui sont
;;  la porte de ses connaissances, elle tend sans cesse l'tat de ces
;; dernires.
;;   Dfinition de l'algorithme du crible d'Eratosthne :
;; pour savoir si un nombre N est premier, il faut effectuer les divisions
;; euclidiennes de N par les nombres premiers successifs, jusqu' trouver
;;    - soit un reste nul => dans ce cas N n'est pas premier
;;    - ou un reste non nul et un quotient infrieur au diviseur => dans ce cas
;;      N est premier
;$   Eratosthenes definiert einen Algorithmus um zu prfen, ob eine Zahl eine
;$ Primzahl ist: das Sieb des Eratothenes. Dieser Prozess setzt die Kenntnis
;$ aller Primzahlen kleiner als die zu prfende Zahl voraus. Der Prozess knnte
;$ daher sehr mhsam werden. Aber die Aufgabe des Algorithmus kann sehr
;$ vereinfacht werden, wenn man ihn mit der Mglichkeit ausstattet, durch
;$ wiederholte Erfahrung einen vorherigen Wissensstand bestehend aus einer sich
;$ vergrernden Liste schon geprfter Primzahlen zu sammeln.
;$   Die folgende Memo-Funktion besitzt diese Lernfhigkeit: whrend das Sieb
;$ des Eratosthenes angewendet wird um zu entscheiden, ob Zahlen Primzahlen
;$ sind, eignet sie sich unaufhrlich neues Wissen an.
;$   Definition des Sieb des Eratosthenes-Algorithmus:
;$ um zu wissen, ob eine Zahl N eine Primzahl ist, fhre die euklidische
;$ Division von N durch die aufeinanderfolgenden Primzahlen durch bis das
;$ Ergebnis
;$     - entweder einen Rest gleich 0 => in diesem Fall ist N keine Primzahl
;$     - oder einen Rest ungleich 0 enthlt und einen Quotienten kleiner als
;$       der Teiler => in diesem Fall ist N eine Primzahl
;+   Eratosthenes defined an algorithm to determine the primality of a number:
;+ the Sieve of Eratosthenes. This procedure requires the knowledge of all prime
;+ numbers smaller than the tested number. The procedure may thus become quite
;+ laborious. But the task of the algorithm can be greatly facilitated if it is
;+ endowed with the capacity, through repeated experiences, of acquiring a
;+ previous knowledge of an increasing list of pre-tested prime numbers.
;+   The following memo-function possesses this learning capability: while
;+ applying the Sieve of Eratosthenes to decide the primality of numbers within
;+ its reach, it unceasingly extends its knowledge state.
;+   Definition of the Sieve of Eratosthenes algorithm:
;+ to know if a number N is prime, perform the euclidean divisions of N by the
;+ successive prime numbers until the result includes
;+    - either a nul rest => in that case N is not prime
;+    - or a non nul rest and a quotient smaller than the divisor => in that
;+      case N is prime

(defun Eratosthene (N &aux orig tmp fin (rez "Non")) (cond
  ((not (and (integerp N) (plusp N))) "Entiers positifs seulement, s.v.p...")
  ((= N 2) "Oui")                                    ;Cas particulier
  ((or (evenp N) (= N 1)) "Non")                     ;Pair => Non
  ((> N (setq tmp (get 'Eratosthene 'limite)))       ;On doit tous les connatre
     (format nil "Je ne sais meme pas pour ~@R (~D), alors..." tmp tmp))
  ((member N (setq orig (get 'Eratosthene 'premiers))) "Oui");Dj Oui
  ((< N tmp) "Non")                                          ;Dj Non
  (t (do ((premiers (copy-list orig)))               ;Sinon crible d'Eratosthene
         (fin rez)
       (multiple-value-bind (quo rst) (floor (/ N (setq tmp (pop premiers))))
         (cond ((zerop rst) (setq fin t)                 ;Reste = 0 => Non et...
                            (put 'Eratosthene 'limite (+ N 2))) ;... nlle limite
               ((< quo tmp) (setq fin t)          ;Quotient < diviseur => Oui...
                            (setq rez "Oui")              ;...
                            (put 'Eratosthene 'premiers   ;... et le noter et...
                                 (append orig (list N)))  ;... nlle limite
                            (put 'Eratosthene 'limite (+ N 2)))))))))

(setf (symbol-plist 'Eratosthene) '(premiers (3) limite 5)) ;Apprentissage mini.
(setf (symbol-value 'Eratosthene) 'F-apprenti)

;;   Pour tester ventuellement la fonction Eratosthene : consulter d'abord son
;; tat de connaissance avec la forme
;$   Um die Eratosthenes-Funktion schlielich zu prfen: prfe zuerst ihren
;$ Wissensstand mit dem Ausdruck
;+   To eventually test the Eratosthene function: first check its knowledge
;+ state with the form
;-    (symbol-plist 'Eratosthene)
;; puis valuer la forme
;$ dann evaluiere folgenden Ausdruck
;+ then evaluate the form
;-    (let ((n (- (get 'Eratosthene 'limite) 2)))
;-        (pasapas (progn (format t "~4D : ~A.~%" n (Eratosthene n))
;-                        (incf n 2))))
;; Cf. ped.17.2a.clinit pour la dfinition de la macro-fonction pasapas.
;$ Vgl. ped.17.2a.clinit fr eine Definition der Makrofunktion pasapas.
;+ Cf. ped.17.2a.clinit for a definition of the pasapas macro-function.

;;   Sauvegarde de l'tat de connaissance de mmo-fonctions, automatiquement
;; insr en queue de leur propre fichier de dfinition. On donne la valeur
;; F-apprenti aux symboles nommant ces fonctions, afin de permettre une
;; identification ventuelle de leur qualit de mmo-fonction.
;;    memoriser [<fonction(s)>]
;;        => inscription des connaissance acquises par la(les) <fonction(s)> 
;;           la fin de son(leur) propre fichier de dfinition
;$   Backup des Wissensstands der Memo-Funktionen, welcher automatisch an das
;$ Ende der Datei angehngt wird, in welcher diese definiert wurden. Um eine
;$ etwaige Erkennung als Memo-Funktion zu ermglichen, bergibt man den Wert
;$ F-apprenti den Symbolen, die diese Funktionen benennen.
;$    memoriser [<Funktion(en)>]
;$        => Der Wissenstand, der durch die <Funktion(en)> erreicht wurde, wird
;$           zu der Datei hinzugefgt, in der diese selbst definiert wurden
;+   Backup of the knowledge state of memo-functions, automatically inserted at
;+ the end of their own defining file. The value F-apprenti is given to the
;+ symbols naming these functions, to allow their eventual identification as
;+ memo-functions.
;+    memoriser  [<function(s)>]
;+        => the knowledge state acquired by the <function(s)> is appended to
;+           its(their) own defining file

(defun memoriser (&rest args &aux fichier memo-fonc p-liste)
 (let ((orig-pl *print-length*))
  (setq *print-length* nil)   ;assurer impression non limite par utilisateur...
  (catch 'fini       ;sortie spciale, pour viter de poursuivre des rcursions
                     ;restes en suspens, en cas de continuation(s) aprs les
                     ;formes cerror. Cette rdaction simplifie celle des clauses
                     ;    (when ... (cerror ... (boucle (cdr listfoncs)))),
                     ;et de leur agencement avec ce qui leur fait suite
    (labels ((boucle (listfoncs) (cond
               ((endp listfoncs)
                  (setq *print-length* orig-pl)
                  (throw 'fini 'ok))
               (t (setq memo-fonc (car listfoncs))
                  (setq fichier
                    ;(excl:source-file memo-fonc))                        ;UNIX
                    (when (setq fichier
                            ;(cdr (assoc 'function                         ;MCL
                            ;           (edit-definition-p memo-fonc))))   ;MCL
                            (cadr (assoc `(defun ,(car listfoncs))
                                    (dspec:name-definition-locations  ;LispWorks
                                     '(function) (car listfoncs))     ;LispWorks
                                    :test #'equal)))                  ;LispWorks
                      (pathname fichier)))
                  (when (not (pathnamep fichier))
                    (cerror "continuer en eliminant cet argument"
                            "memoriser : ~A : ~
                             impossible de localiser un fichier de definition."
                            memo-fonc)
                    (boucle (cdr listfoncs)))
                  (setq p-liste (symbol-plist memo-fonc))
                  (when (null p-liste)
                    (cerror "continuer en eliminant cet argument"
                            "memoriser : ~A : ~
                             ne possede pas de memoire d'apprentissage."
                            memo-fonc)
                    (boucle (cdr listfoncs)))
                  (with-open-file (flux fichier :direction         :output
                                                :if-exists         :append
                                                :if-does-not-exist :error)
                    (format flux "~%;; ~A : memoire acquise le ~A~%"
                                 memo-fonc (jour-date))
                    (format flux "(setf (symbol-value '~S) 'F-apprenti)~%"
                                 memo-fonc)
                    (format flux "(setf (symbol-plist '~S) '~S)~%"
                                 memo-fonc p-liste))
                  (boucle (cdr listfoncs))))))
      (boucle args)))))

;...............................................................................
;; Ci-dessous : exemple(s) de sauvegarde(s)...
;$ Unten: Backup-Beispiel(e)...
;+ Below: save example(s)...

;; ERATOSTHENE : memoire acquise le Mardi, 04/11/2014, 16:23:49
(setf (symbol-value 'ERATOSTHENE) 'F-apprenti)
(setf (symbol-plist 'ERATOSTHENE) '(PREMIERS (3 5 7 11 13 17 19 23 29 31 37 41 43 47 53 59 61 67 71 73 79 83 89 97 101 103 107 109 113 127 131 137 139 149 151 157 163 167 173 179 181 191 193 197 199 211 223 227 229 233 239 241 251 257 263 269 271 277 281 283 293 307 311 313 317 331 337 347 349 353 359 367 373 379 383 389 397 401 409 419 421 431 433 439 443 449 457 461 463 467 479 487 491 499 503 509 521 523 541 547 557 563 569 571 577 587 593 599 601 607 613 617 619 631 641 643 647 653 659 661 673 677 683 691 701 709 719 727 733 739 743 751 757 761 769 773 787 797 809 811 821 823 827 829 839 853 857 859 863 877 881) LIMITE 883))

;; FIBO3 : memoire acquise le Mardi, 28/10/2014, 17:20:20
(setf (symbol-value 'FIBO3) 'F-apprenti)
(setf (symbol-plist 'FIBO3) '(100 573147844013817084101 99 354224848179261915075 98 218922995834555169026 97 135301852344706746049 96 83621143489848422977 95 51680708854858323072 94 31940434634990099905 93 19740274219868223167 92 12200160415121876738 91 7540113804746346429 90 4660046610375530309 89 2880067194370816120 88 1779979416004714189 87 1100087778366101931 86 679891637638612258 85 420196140727489673 84 259695496911122585 83 160500643816367088 82 99194853094755497 81 61305790721611591 80 37889062373143906 79 23416728348467685 78 14472334024676221 77 8944394323791464 76 5527939700884757 75 3416454622906707 74 2111485077978050 73 1304969544928657 72 806515533049393 71 498454011879264 70 308061521170129 69 190392490709135 68 117669030460994 67 72723460248141 66 44945570212853 65 27777890035288 64 17167680177565 63 10610209857723 62 6557470319842 61 4052739537881 60 2504730781961 59 1548008755920 58 956722026041 57 591286729879 56 365435296162 55 225851433717 54 139583862445 53 86267571272 52 53316291173 51 32951280099 50 20365011074 49 12586269025 48 7778742049 47 4807526976 46 2971215073 45 1836311903 44 1134903170 43 701408733 42 433494437 41 267914296 40 165580141 39 102334155 38 63245986 37 39088169 36 24157817 35 14930352 34 9227465 33 5702887 32 3524578 31 2178309 30 1346269 29 832040 28 514229 27 317811 26 196418 25 121393 24 75025 23 46368 22 28657 21 17711 20 10946 19 6765 18 4181 17 2584 16 1597 15 987 14 610 13 377 12 233 11 144 10 89 9 55 8 34 7 21 6 13 5 8 4 5 3 3 2 2))

;; FIBO4 : memoire acquise le Mardi, 28/10/2014, 17:20:20
(setf (symbol-value 'FIBO4) 'F-apprenti)
(setf (symbol-plist 'FIBO4) '(100 573147844013817084101 99 354224848179261915075 98 218922995834555169026 97 135301852344706746049 96 83621143489848422977 95 51680708854858323072 94 31940434634990099905 93 19740274219868223167 92 12200160415121876738 91 7540113804746346429 90 4660046610375530309 89 2880067194370816120 88 1779979416004714189 87 1100087778366101931 86 679891637638612258 85 420196140727489673 84 259695496911122585 83 160500643816367088 82 99194853094755497 81 61305790721611591 80 37889062373143906 79 23416728348467685 78 14472334024676221 77 8944394323791464 76 5527939700884757 75 3416454622906707 74 2111485077978050 73 1304969544928657 72 806515533049393 71 498454011879264 70 308061521170129 69 190392490709135 68 117669030460994 67 72723460248141 66 44945570212853 65 27777890035288 64 17167680177565 63 10610209857723 62 6557470319842 61 4052739537881 60 2504730781961 59 1548008755920 58 956722026041 57 591286729879 56 365435296162 55 225851433717 54 139583862445 53 86267571272 52 53316291173 51 32951280099 50 20365011074 49 12586269025 48 7778742049 47 4807526976 46 2971215073 45 1836311903 44 1134903170 43 701408733 42 433494437 41 267914296 40 165580141 39 102334155 38 63245986 37 39088169 36 24157817 35 14930352 34 9227465 33 5702887 32 3524578 31 2178309 30 1346269 29 832040 28 514229 27 317811 26 196418 25 121393 24 75025 23 46368 22 28657 21 17711 20 10946 19 6765 18 4181 17 2584 16 1597 15 987 14 610 13 377 12 233 11 144 10 89 9 55 8 34 7 21 6 13 5 8 4 5 3 3 2 2 1 1 0 1))
