;===============================================================================
;;; 16.4a --> STRUCTURES : premier exemple
;$$ 16.4a --> STRUKTUREN: erstes Beispiel
;++ 16.4a --> STRUCTURES: First Example
;===============================================================================

;;  Synthse sonore Csound dduite de fichiers d'analyse de pomes de
;; Vicente Huidobro (pour Cristian Morales-Ossio, 1998-99).
;;   Dans cet exemple simplifi, chaque caractre typographique d'un pome
;; est traduit par un nonc i pour Csound.
;$   Csound-Synthese hergeleitet aus Analysedateien von Gedichten von Vicente
;$ Huidobro (fr Cristian Morales-Ossio, 1998-99).
;$   In diesem vereinfachten Beispiel wird jedes typographische Zeichen des
;$ Gedichts in eine Csound i-Anweisung bersetzt.
;+   Csound synthesis deduced from analysis files of poems by Vicente Huidobro
;+ (for Cristian Morales-Ossio, 1998-99).
;+   In this simplified example, each typographical character of a poem is
;+ translated into a Csound i statement.

;;; Structures
;$$ Strukturen
;++ Structures

;;   Dfinition de structures diverses, correspondant chacune  un type
;; d'instrument Csound. Des groupes d'instruments partagent un certain nombre de
;; donnes communes, mais chacun possde en outre d'autres donnes qui lui sont
;; spcifiques. Il peut y avoir plusieurs ensembles de donnes communes.
;$   Definition verschiedener Strukturen, die jeweils einem Csound Instrumenten
;$ -Typ entsprechen. Instrumentengruppen teilen eine gewisse Anzahl an
;$ gemeinsamen Daten, aber jede besitzt auch spezifische Daten. Viele Gruppen
;$ gemeinsamer Daten sind mglich.
;+   Definition of various structures, each corresponding to a Csound instrument
;+ type. Groups of instruments share a certain number of common data, but each
;+ also possesses other specific data. Many sets of common data are possible.

(defstruct (PCOMs)                                         ;type spcial pour
  Ps                                                       ;les donnes communes
  )
(defstruct (voy1)                                          ;type "voyelle-1"
  instrument
  Pcommuns                                                 ;donnes communes
  Pspecifs                                                 ;donnes spcifiques
  )
(defstruct (cons5)                                         ;type "consonne-5"
  instrument
  Pcommuns                                                 ;donnes communes
  Pspecifs                                                 ;donnes spcifiques
  )
;etc. / usw. / etc.

;;; Base de donnes
;$$ Datenbasis
;++ Data Base

;- a.
;;   Dfinition des valeurs des paramtres communs  des groupes d'instruments.
;; Par exemple :
;;    plages de variations de dure (+/- 0.5 sec.) et d'amplitude (+/- 1.5 dB)
;;    pour COMMUNs1 ci-dessous. Les valeurs de COMMUNs2 sont de mme type, mais
;;    diffrentes
;; Un changement de ces donnes communes affectera automatiquement tous les
;; noncs i des instruments qui y font rfrence.
;$   Definition der gemeinsamen Parameterwerte einer Instrumentengruppe.
;$ Zum Beispiel:
;$    Variationsspielraum der Dauer (+/- 0.5 sec.) und Amplitude (+/- 1.5 dB)
;$    fr COMMUNs1 unten. Werte fr COMMUNs2 haben denselben Typ, aber andere
;$    Werte
;$ Eine nderung an diesen gemeinsamen Daten beeinflusst automatisch alle
;$ i-Anweisungen der Instrumente, die auf diese Bezug nehmen.  
;+   Definition of the values of parameters common to instrument groups. For
;+ instance:
;+    duration (+/- 0.5 sec.) and amplitude (+/- 1.5 dB) variation ranges for
;+    COMMUNs1 below. Values for COMMUNs2 are of the same type, but different
;+ A change in these common data will automatically have consequences on all
;+ i statements for the instruments refering to them.

(defparameter COMMUNs1                                       ;donnes communes 1
  (make-PCOMs
   :Ps '((- (random 1.0) 0.5) . (- (random 3.0) 1.5))
   ))
(defparameter COMMUNs2                                       ;donnes communes 2
  (make-PCOMs
   :Ps '((- (random 0.4) 0.2) . (- (random 6.0) 3.0))
   ))
;etc. / usw. / etc.

;- b.
;;   Chaque caractre typographique est traduit en un symbole pointant vers une
;; structure du type dsir, dont les champs comportent les donnes ncessaires
;;  la ralisation du caractre sous forme d'nonc i pour Csound.
;$   Jedes typographische Zeichen wird in ein Symbol bersetzt, welches zu einer
;$ Struktur des gewnschten Typs zeigt, deren Datenfelder die ntigen Daten zur
;$ Umsetzung des Zeichens als Csound i-Anweisung enthalten. 
;+   Each typographical character is translated into a symbol pointing to a
;+ structure of the desired type, whose slots contain the necessary data to
;+ realize the character as an Csound i statement.

(set (read-from-string (string #\a))                ;[a]
     (make-voy1
      :instrument 'un
      :Pcommuns   (PCOMs-Ps COMMUNs1)               ;accs -> donnes communes 1
      :Pspecifs   '(-20.0 (440.0 . 1231.0) (-1.0 . -2.0))
      ))

(set (read-from-string (string #\e))                ;[e]
     (make-voy1
      :instrument 'un
      :Pcommuns   (PCOMs-Ps COMMUNs1)               ;accs -> donnes communes 1
      :Pspecifs   '(-23.0 (180.2 . 390.5) (-5.3 . -6.4))
      ))

(set (read-from-string (string #\q))                ;[q]
     (make-cons5
      :instrument 'trois
      :Pcommuns   (PCOMs-Ps COMMUNs2)               ;accs -> donnes communes 2
      :Pspecifs  '(-12.0 (10 . 20))
      ))
;etc. / usw. / etc.

;;; Fonctions
;$$ Funktionen
;++ Functions

;;   Dfinition des "fonctions/instruments" capables d'interprter les donnes
;; communes et spcifiques puises dans une structure, afin de raliser un
;; nonc i pour Csound. Exemples : ...
;$   Definition der "Funktionen/Instrumente", die fhig sind, die eingelesenen
;$ gemeinsamen und spezifischen Daten aus einer Struktur zu interpretieren um
;$ eine Csound i-Anweisung zu erzeugen. Beispiele: ...
;+   Definition of the "functions/instruments" capable of interpreting the
;+ common and specific data read from a structure in order to realize a Csound i
;+ statement. Examples: ...

;;   ... fonction/instrument un/1...
;$   ... Funktion/Instrument un/1...
;+   ... function/instrument un/1...

(defun un (donneesC donneesS debut duree carac)
  (let* ((p3  (+ duree (eval (car donneesC))))
         (p4  (+ (car donneesS) (eval (cdr donneesC))))
         (tp1 (cadr donneesS))
         (p5  (+ (car tp1) (random (- (cdr tp1) (car tp1)))))
         (tp2 (caddr donneesS))
         (p6  (* p5 (+ (cdr tp2) (random (abs (- (cdr tp2) (car tp2))))))))
    (format t "~&i1 ~7,3F~9,3F~12,3F~12,3F~12,3F~78T;~A~%"
               debut p3 p4 p5 p6 carac)))

;;   ... et fonction/instrument trois/3.
;$   ... und Funktion/Instrument trois/3.
;+   ... and function/instrument trois/3.

(defun trois (donneesC donneesS debut duree carac)
  (let* ((p3  (+ duree (eval (car donneesC))))
         (p4  (+ (car donneesS) (eval (cdr donneesC))))
         (tp1 (cadr donneesS))
         (p5  (* (+ (car tp1) (random (- (cdr tp1) (car tp1))))
                 (if (zerop (random 2)) 1 -1))))
    (format t "~&i3 ~7,3F~9,3F~12,3F~12D~78T;~A~%"
               debut p3 p4 p5 carac)))
;etc. / usw. / etc.

;;   Fonction accdant aux donnes dfinies pour un caractre ASCII. Elle fait
;; un appel  la fonction/instrument dsigne dans la structure correspondante,
;; avec, comme arguments, les donnes communes et spcifiques -- elles-aussi
;; comprises dans la structure. C'est cette fonction/instrument appele qui
;; ralise concrtement l'nonc i pour Csound. On suppose que la date de dbut
;; et la dure de l'nonc  produire ont dj t dtermines par ailleurs.
;$   Funktion zum Zugriff auf die Daten, welche fr ein ASCII-Zeichen definiert
;$ wurden. Es ruft die in der entsprechenden Struktur bestimmte
;$ Funktion/Instrument mit den gemeinsamen und spezifischen Daten -- welche
;$ selbst auch in der Struktur enthalten sind -- als Aufrufparameter auf. Die
;$ aufgerufene Funktion/Instrument erzeugt die eigentliche Csound i-Anweisung.
;$ Es wird vorausgesetzt, dass die Startzeit und Dauer einer Anweisung bereits
;$ an anderer Stelle festgelegt wurde.
;+   Function accessing to the data defined for an ASCII character. It calls the
;+ function/instrument designated in the corresponding structure with, as
;+ calling arguments, the common and specific data--themselves also contained in
;+ the structure. This called function/instrument concretely realizes the Csound
;+ i statement. The start time and duration of the statement are presumed to
;+ have been already determined elsewhere.

(defun note-Csound (ascii start duration)
  (let* ((carac (read-from-string (string ascii))) ;caractre ASCII => symbole 
         (struc (symbol-value carac))              ;accs direct -> structure
         (typde (string (type-of struc)))          ;radical- fonctions d'accs
         (instr (read-from-string                  ;accs -> fonction  appeler
                  (concatenate 'string typde "-instrument")))
         (donnC (read-from-string                  ;accs -> donnes communes
                   (concatenate 'string typde "-Pcommuns")))
         (donnS (read-from-string                  ;accs -> donnes spcifiques
                   (concatenate 'string typde "-Pspecifs"))))
    ;(dbg carac) (dbg struc) (dbg typde) (dbg instr) (dbg donnC) (dbg donnS)
    (funcall (funcall instr struc) (funcall donnC struc)
                                    (funcall donnS struc)
                                    start
                                    duration
                                    carac)))

;-   Exemples d'appels : / Beispielaufrufe: / Example calls:
;-    (note-Csound <caractere> <date_de_debut> <duree>)

(note-Csound #\a  0.0  2.5)
(note-Csound #\a  3.1  7.7)
(note-Csound #\e  1.8 12.9)
(note-Csound #\e 14.2  9.2)
(note-Csound #\q  3.5 25.6)
(note-Csound #\q 13.8 15.1)
