;$   Strukturdefinition.                                              [d..H.B.]

(defstruct (Schrank)
  Schmucken
  Partituren
  Tabaken
  Struempfen
  Kleidern)

;$   Instanz (english "instance").

(setq Srk1 (make-Schrank :Tabaken '(Davidoff) :Partituren '(Rameau)))

;- 1.

;$   Pures Hinzufgen, ohne Interface: nicht sehr spannend...
;$    (setf (Schrank-Tabaken Srk1) (cons 'Monte-Cristo (Schrank-Tabaken Srk1)))

;- 2.

;$   Gutes Interface: Macrofunktion Fge. Beispiele:
;$    (Fge Beatles zu Partituren in Schrank Srk1 hinzu)
;$    (Fge NikoMax zu Tabaken in Schrank Srk1 hinzu)
;$    (Fge Krone zu Schmucken in Schrank Srk1 hinzu)
;$    (Fge Ballkleid zu Kleidern in Schrank Srk1 hinzu)
;$    (Fge MoonGoo zu Struempfen in Schrank Srk1 hinzu)

(defmacro Fge (was y wo z structure wessen x)
  (let ((access-f (read-from-string
                   (concatenate 'string
                                (string structure)
                                "-"
                                (string wo)))))
    `(setf (,access-f ,wessen)
           (cons (quote ,was) (,access-f ,wessen)))))

;- 3.

;$   Noch besseres Interface: Macrocharakter "!". Beispiele:
;$    !Fge Mozart zu Partituren in Schrank Srk1 hinzu.
;$    !Fge Bach + Brahms u. Zappa + Pope zu Partituren in Schrank Srk1 hinzu.
;$    !Fge Hose und RotenRock mit Hemd zu Kleidern in Schrank Srk1 hinzu.
;$    !Fge HugoBoss mit Esprit u. Carre zu Struempfen in Schrank Srk1 hinzu.
;$    !Fge Szepter u. Wurst mit Schokolade zu Schmucken in Schrank Srk1 hinzu.
;$    !Fge LuckyStrike und Menthol + Drum zu Tabaken in Schrank Srk1 hinzu.
;! ? MAIS dans le Listener on doit avoir un second <return> pour que a marche.
;! En valuant par <ctrl>-E dans l'Editor a marche OK.

(defun HinzuFgen (flux char)
  (labels ((str>sym (listr) ;convertit une liste de strings en liste de symboles
             (if (endp listr) nil
             (cons (read-from-string (car listr)) (str>sym (cdr listr)))))
           (parse (phrase)
             ;(dbg phrase)
             (if (member-or '(und u. + mit) phrase)
                 (let ((lng (length phrase)))
                   (eval (append (subseq phrase 0 1)
                                 (subseq phrase (- lng 7) lng)))
                   (parse (append (subseq phrase 0 (- lng 8))
                                  (subseq phrase (- lng 6) lng))))
               phrase)))
    (parse (str>sym (separer-mots (read-line flux))))))
(set-macro-character #\! #'HinzuFgen)

;;   Rend T si au moins l'un des lments de la liste <its> est prsent dans la
;; liste <li>. NIL autrement.
;$   Ergibt T wenn mindestens ein Element der <its> Liste in der <li> Liste ist.
;$ Ansonsten NIL.
;+   Returns T if at least one of the elements of list <its> is present in list
;+ <li>. NIL otherwise.

(defun member-or (its li)
  (labels ((interne (ts)
             (cond ((endp ts) nil)
                   ((member (car ts) li :test #'equal))
                   (t (member-or (cdr ts) li)))))
    (interne its)))

;$ Nutzfunktion

;;    partir d'une chane de caractres, donne une liste de sous-chanes, ou
;; <mots>. Un <mot> est compos d'un ou plusieurs caractres non-sparateurs,
;; spars par un ou plusieurs caractres sparateurs. Le mot-clef :separateurs
;; permet de modifier la liste des caractres dfinis comme sparateurs. Ex. :
;-    (separer-mots "    ")                   => nil
;-    (separer-mots " C'est lui... Non ? ")   => ("C" "est" "lui..." "Non")
;-    (separer-mots "It's you, and me... as well as many others.")
;-        => ("It" "s" "you" "and" "me..." "as" "well" "as" "many" "others.")
;- Cf. cl.d/utilitaires.d/separer-mots.lisp.
;! Dans la prsente version, le point (.) est considr comme un caractre
;! non-sparateur. Ceci pour permettre en allemand l'abrviation "u." de "und".
;! Fait pour CRIM (Christian Morales-Ossio) dans le cadre de son projet sur des
;! pomes de Vicente Huidobro -- 20/12/97.

(defun separer-mots (cadena
     &key (separateurs
              '(#\space #\newline #\tab #\return #\linefeed #\page       ;blancs
                #\, ;#\.                      ;ponctuations (enlev #\. pour u.)
                #\; #\: #\! #\?                            ;ponctuations (suite)
                #\- #\' #\( #\) #\" #\/                            ;grammaticaux
                ))  )
  (do ( (rez    ()                    )
        (mot    ""                    )
        (caract nil                   )
        (long   (length cadena)       )
        (n      0               (1+ n)) )
      ( (= n long) (if (string= "" mot) rez
                       (append rez (list mot))) )
    (if (member (setq caract (char cadena n)) separateurs)
      (when (not (string= "" mot))
        (setq rez (append rez (list mot)))
        (setq mot ""))
      (setq mot (concatenate 'string mot (string caract))))))