;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Fichier ~/dlo/.clinit.cl (UNIX) : pour servir d'exemples de macro-caractres,
;; de personnalisation d'environnement, etc.
;; - NE PAS CHARGER en Macintosh Common LISP (MCL) ou LispWorks
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;   Exemples de divers macro-caractres utilitaires : cf. ~dlo/.clinit.cl.
;;    Interaction avec UNIX : !, ^E, ^A, ^F.
;; L'usage de macro-caractres a ici pour effet principal d'amliorer le confort
;; de l'utilisateur en le dispensant de rdiger des formes LISP compltes : un
;; macro-caractre permet d'intervenir librement dans le processus top-level
;; normal  la suite de la frappe d'un seul caractre. Ces exemples utilisent
;; cette technique pour crer des fonctions du mme niveau qu'avec la fonction
;; top-level:alias (UNIX : cf. dfinition de l'alias top-level lo ci-dessous).
;;   Le fichier UNIX ~dlo/.clinit.cl est donn ici intgralement. Il n'est
;; cependant que partiellement utilisable en Macintosh Common LISP (MCL) -- cf.
;; fichier ...:DLO:cl.d:+clinit.cl.

;;; ENVIRONNEMENT --------------------------------------------------------------
;;   Path divers utiles...
;;   Gestion du module canons-stochastiques.
(defvar common-lisp::*path-stochastique* (pathname "/home/dlo/cl.d/stoch.d/*"))
(top-level:alias "ld-stoch" (&rest args)
  (if args
    (dolist (type args)
      (top-level:do-command "ld"
        (merge-pathnames (concatenate 'string "canons_"
                                      (string type)
                                      ".cl")
                         common-lisp::*path-stochastique*)))
    (top-level:do-command "ld"
      (merge-pathnames "canons_tous.cl" common-lisp::*path-stochastique*))))
;;   Pour imprimer de longues listes en paix.
(top-level:setq-default top-level:*print-length* nil)
(top-level:setq-default top-level:*print-level* nil)
;;   Pour mieux profiter de l'historique.
(top-level:setq-default top-level:*history* 23)
(top-level:alias "h" (&rest args)
  (top-level:do-command "history" :count 23))
;;   En cas de besoin de la macro loop.
;(load "/usr/local/lang/cl/lib/code/loop.fasl")

;;; FONCTIONS DIVERSES ---------------------------------------------------------
;;   Pour tester rptitivement quelque chose.
;;    continu <form> => boucle continue infinie sur <form>.
;;    pasapas <form> => boucle infinie sur <form>, avec attente d'un <RETURN> 
;;                      chaque pas.
;;    iter <form> &optional (<n> 22) => imprime <n> fois l'valuation de <form>.
;; Dans les deux premiers cas : faire <q><RETURN> pour interrompre.
(defmacro continu (programme)
  (format *debug-io* "~&Taper <RETURN> pour commencer, et <q><RETURN> ~
                      pour finir.~%")
  (read-char *debug-io*)
  `(loop
     ,programme
     (when (char= #\q (read-char-no-hang *debug-io* nil t t))
       (return 'ok))))
(defmacro pasapas (programme)
  (format *debug-io* "~&Taper <RETURN> pour repeter, et <q><RETURN> ~
                      pour finir.~%")
  `(loop
     ,programme
     (when (char= #\q (read-char *debug-io* nil t t))
       (return 'ok))))
(defmacro iter (express &optional (nbr 22))
  `(do ((n 1 (1+ n)))
       ((> n ,nbr) 'ok)
     (prins n " :    " ,express)))

;;   Pour imprimer temporairement une valeur, en cours de mise au point - de
;; debugging.
;;    dbg <forme> => imprime la <forme> et son valuation sur *debug-io*.
(defmacro dbg (forme &optional fonction)
  (if fonction
    `(format *debug-io* "~4Tdbg-> ~S : ~S = ~S~%"
             (quote ,fonction) (quote ,forme) ,forme)
    `(format *debug-io* "~4Tdbg-> ~S = ~S~%" (quote ,forme) ,forme)))

;;   Pour obtenir la documentation-string d'une variable, ou fonction, ou macro-
;; fonction.
;; N.B. : la primitive documentation ne marche pas pour les macro-fonctions sous
;; UNIX.
(defmacro dof (nom)
"
Pour obtenir la documentation-string d'une
variable, ou macro-fonction, ou fonction.
"
  (cond ((boundp nom)
         `(format *debug-io* "~A : variable : ~A" (quote ,nom)
                  (documentation (quote ,nom) 'variable)))
        ((macro-function nom)
         `(format *debug-io* "~A : macro-fonction : ~A" (quote ,nom)
                  (documentation (quote ,nom) 'function)))
        ((fboundp nom)
         `(format *debug-io* "~A : fonction : ~A" (quote ,nom)
                  (documentation (quote ,nom) 'function)))
        ))

;;    partir de 2 points [x1,y1] et [x2,y2], donne 2 values : les coefficients
;; a et b de la fonction linaire y=ax+b passant par ces deux points.
(defun y=ax+b (x1 x2 y1 y2)
  (let ((a (float (/ (- y2 y1) (- x2 x1)))))
    (values a (- y2 (* x2 a)))))

;;   Transformation d'un nombre quelconque en flottant arrondi  4 dcimales au
;; maximum.
;;   Le nombre de dcimales n'est ni un argument d'appel, ni un mot-clef : la
;; fonction n'a pas de souplesse, mais son utilisation est simplifie - par
;; exemple lorsqu'elle doit tre applique  une liste de nombres avec la
;; primitive mapcar. Exemple (le dernier nombre = frquence du Do central) :
;;    (mapcar #'trimfloat4 '(7/3 9321/623463 3.141592653589793 261.62556530059))
;;        => (2.3333 0.015 3.1416 261.6256)
(defun trimfloat4 (num)
  (/ (round (float num) 0.0001) 10000.0))

;;   Mme chose, avec nombre de dcimales en argument optionnel [2 par dfaut].

(defun trimfloatx (num &optional (nn 2))
  (/ (round (float num) (expt 10.0 (- nn))) (expt 10.0 nn)))


;;   Utilit incontestable...
(defmacro setqq (var val)
    `(setq ,var (quote ,val)))

;;   Utilitaire d'impression d'une squence quelconque de formes en une seule
;; ligne de texte.
;;    prins {<forme>}* => impression squentielle de(s) <forme>(s).
;; Exemple d'utilisation : (prins "a = " a " ; b = " b ".").
(defun prins (&rest frms)   ;fonction interface : limination du cas trivial, et
  (if (endp frms) nil       ;emballage du nbr inconnu de forme(s) en une liste
      (labels ((prinsX (li)                             ;fonction effective.
                 (cond ((endp li) (terpri) 'ok)         ;cas terminal
                       (t (princ (car li))              ;imprimer la premire...
                          (prinsX (cdr li))))))         ;... et continuer
        (prinsX frms))))                                ;corps du labels            


;;; FONCTIONS LIES  L'INTERACTION AVEC UNIX ----------------------------------
;;   Pour lancer une commande shell.
;; a. Macro-fonction sh :
;;    sh <commande> [<argument(s)] => appel shell de <commande> [argument(s)].
;; N.B. : supporte mal les caractres UNIX spciaux comme `'"<>() etc. - cf.
;; sh-list-to-string ci-dessous.
(defmacro sh (&rest commande)
  `(excl:run-shell-command ,(sh-list-to-string commande)))

;; b. Macro-caractre "!" :
;;    !<commande> [argument(s)] => appel shell de <commande> [argument(s)].
;; N.B. : supporte, beaucoup mieux que la macro-fonction sh ci-dessus, les
;; caractres UNIX spciaux comme `'"<>() etc.
(defun exclamation (flux caractere)
  `(excl:run-shell-command ,(read-line flux t nil t)))
(set-macro-character #\! #'exclamation)

;;   Utilitaire pour la macro-fonction sh ci-dessus. Transforme une liste
;; d'atomes en une string compose des lments de la liste, spars par des
;; espaces.
;; N.B. : ne supporte pas d'lments ne pouvant tre interprts comme des
;; symboles LISP. En particulier, interprte mal les caractres UNIX spciaux
;; `'"<>() etc., surtout isols.
(defun sh-list-to-string (li)
  (let ((elm (car li))) (cond
    ((endp li) "")
    ((symbolp elm) (let ((tmp (string elm)))
                     (concatenate 'string
                                  tmp
                                  " "
                                  (sh-list-to-string (cdr li)))))
    (t (cerror "continuer en eliminant cet argument"
               "sh-list-to-string : ~
                ~A : ne peut etre traite ici comme un argument de commande."
               elm)
       (sh-list-to-string (cdr li))))))

;;   Alias de la commande top-level :ld avec extension ".cl" par dfaut au(x)
;; nom(s) de fichier(s).
;;    :lo {<fics>}* => :ld de tous les fichiers <fics> s'ils existent, ou de
;;                     fichiers <fics>.cl sinon. Chargement - lecture et
;;                     valuation - du(des) dernier(s) fichier(s) charg(s)
;;                     prcdemment si <fics> est null.
(defvar common-lisp::*derniers-lo* ())
(top-level:alias "lo" (&rest fics)
  (if (endp fics) (setq fics common-lisp::*derniers-lo*)
  (setq common-lisp::*derniers-lo* fics))
  (if (endp fics) (apply #'top-level:do-command "ld" nil)
  (lolo fics)))

;;   Fonction effective appele par le top-level:alias lo ci-dessus.
(defun lolo (listfics) (cond
    ((endp listfics) 'ok)
    (t (let ((nom (string (car listfics))))
         (if (probe-file nom)
         (apply #'top-level:do-command "ld" (list nom))
         (apply #'top-level:do-command
                "ld" (list (concatenate 'string nom ".cl"))))
         (lolo (cdr listfics))))))

;;   Pour charger - lire et valuer - des fichiers. Macro-caractre "^A" :
;;    ^A{<zut>}* => chargement de <zut> s'il existe, ou de <zut>.cl sinon, et
;;                  ainsi de suite pour tous les fichiers nomms.
;;    ^A         => encore chargement du(des) mme(s).
;; Interface de la fonction lolo ci-dessus.
(defun control-A (flux caractere)
  (let ((fics (string-to-list flux)))
    (if (endp fics) (setq fics common-lisp::*derniers-lo*)
    (setq common-lisp::*derniers-lo* fics))
    (if (endp fics) (apply #'top-level:do-command "ld" nil)
    `(lolo (quote ,fics)))))
(set-macro-character #\ #'control-A)

;;   Pour obtenir la date, sous forme d'une chane de caractres
;;    "Jour, tantime/mois/anne heures:minutes:secondes".
(defun jour-date ()
  (let ((time (multiple-value-list (get-decoded-time))))
    (format nil "~A, ~2,'0D/~2,'0D/~4D ~2,'0D:~2,'0D:~2,'0D"
            (nth (nth 6 time) '(    "Lundi"
                                    "Mardi"
                                    "Mercredi"
                                    "Jeudi"
                                    "Vendredi"
                                    "Samedi"
                                    "Dimanche"))
            (nth 3 time)
            (nth 4 time)
            (nth 5 time)
            (nth 2 time)
            (nth 1 time)
            (nth 0 time))))
(format *debug-io* "~&;; Salut, t'es ~A.~%" (jour-date))

;;; FONCTIONS LIES  L'UTILISATION DE VI --------------------------------------
;;   a. Ne mmorise pas le nom de fichier.
;;    vi <zut> => vi du fichier <zut> s'il existe, ou de <zut>.cl sinon.
;; N.B. : ne supporte pas un nom de fichier ne pouvant tre interprt comme
;; un symbole LISP.
(defmacro vi (x) (cond
    ((symbolp x)
     `(excl:run-shell-command
       (if (probe-file (string ',x)) (concatenate 'string    "vi " (string ',x))
       (concatenate 'string "vi " (string ',x) ".cl"))))
    (t (cerror "donner un nouveau nom de fichier"
               "vi : ~A : ne peut etre traite ici comme un nom de fichier."
               x)
       (format *query-io* "~&Nouveau nom de fichier ? ")
       `(vi ,(read *query-io*)))))

;;   b. Mmorise le(s) nom(s) de fichier(s) dans le fichier invisible .vimem.
;;    v {<zut>}* => vi de <zut> s'il existe, ou de <zut>.cl sinon, et ainsi de
;;                  suite pour tous les fichiers nomms.
;;    v          => encore vi du(des) mme(s).
;; N.B. : ne supporte pas des noms de fichiers ne pouvant tre interprts comme
;; des symboles LISP - cf. fonction vi-list-to-string ci-dessous.
(defmacro v (&rest noms &aux a-editer) (cond
    ((null noms)
     (with-open-file (fichier ".vimem" :direction :input)
       (setq a-editer (read-line fichier)))
     `(excl:run-shell-command (concatenate 'string "vi " ,a-editer)))
    (t
     (setq a-editer (vi-list-to-string noms))
     (with-open-file (fichier ".vimem" :direction         :output
                                       :if-does-not-exist :create
                                       :if-exists         :rename-and-delete)
       (princ a-editer fichier) (terpri fichier))
     `(excl:run-shell-command (concatenate 'string "vi " ,a-editer)))))
;;   Alias utile...
(setf (macro-function 'V) (macro-function 'v))

;;   c. Interface de la macro-fonction v ci-dessus. Macro-caractre "^E" :
;;    ^E{<zut>}* => vi de <zut> s'il existe, ou de <zut>.cl sinon, et ainsi de
;;                  suite pour tous les fichiers nomms.
;;    ^E         => encore vi du(des) mme(s).
;; N.B. : ne supporte pas des noms de fichiers ne pouvant tre interprts comme
;; des symboles LISP.
(defun control-E (flux caractere)
  `(v ,@(string-to-list flux)))
(set-macro-character #\ #'control-E)

;;   Utilitaire pour la macro-fonction v ci-dessus. Transforme une liste
;; d'atomes en une string compose des lments de la liste, spars par des
;; espaces. Chaque lment est complt de l'extension ".cl" lorsqu'il ne
;; correspond pas  un fichier dj existant.
;; N.B. : ne supporte pas des lments ne pouvant tre interprts comme des
;; symboles LISP.
(defun vi-list-to-string (li)
  (let ((nom (car li))) (cond
    ((endp li) "")
    ((symbolp nom) (let ((tmp (string nom)))
                     (concatenate 'string
                                  (if (probe-file tmp) tmp
                                      (concatenate 'string tmp ".cl"))
                                  " "
                                  (vi-list-to-string (cdr li)))))
    (t (cerror "continuer en eliminant cet item"
               "vi-list-to-string : ~
                ~A : ne peut etre traite ici comme un nom de fichier."
               nom)
       (vi-list-to-string (cdr li))))))

;;   d. Pour l'dition et le chargement, enchans automatiquement, de la
;; dfinition de fonctions LISP dj connues - en cours de mise au point.
;; Macro-caractre "^F" :
;;    ^F{<fon>}* => vi du fichier contenant la dfinition de la fonction <fon>,
;;                  suivi automatiquemnent du chargement de ce fichier, et
;;                  ainsi de suite pour toutes les fonctions nommes.
;;    ^F         => encore vi et chargement de la (des) mme(s) fonction(s).
;; quivalent manuel en Macintosh Common LISP (MCL), pour une seule fonction :
;; squence de menus, fentre de dialogue et bouton :
;;    Tools > Get Info... > [taper le nom <fon>] > Definition(s) > Find Def.
(defvar common-lisp::*derniers-ed-lo* ())
(defun control-F (flux caractere)
  (let ((fics (string-to-list flux)))
    (cond ((endp fics)
           (if (null common-lisp::*derniers-ed-lo*)
             (error "control-F : ~
                     pas de definition de fonction a editer.")
             (setq fics common-lisp::*derniers-ed-lo*)))
          (t
           (setq common-lisp::*derniers-ed-lo* fics)))
    `(ed-lo (quote ,fics))))
(set-macro-character #\ #'control-F)

;;   Fonction effective appele par la fonction control-F ci-dessus.
(defun ed-lo (listfics)
  (when listfics
    (let* ((fonction (car listfics))
           (fichier (excl:source-file fonction)))
      (if fichier
        (progn (format *debug-io*
                       "; Edition et chargement ~A, ~
                        ~%;~4Tcontenant la definition de ~A ... "
                       fichier fonction)
               (eval (macroexpand-1
                      `(vi ,(read-from-string (namestring fichier)))))
               (load fichier)
               (ed-lo (cdr listfics)))
        (error "ed-lo : ~A : fonction inconnue." fonction))))
  'ok)

;;   Utilitaire pour les fonctions control-A, control-E et control-F ci-dessus.
;; Transforme une string, comportant un nombre quelconque d'lments, disposs
;; sur une ligne et spars par des blancs (espaces ou tabs), en une liste
;; d'atomes. Une string nulle ou vide produit la liste vide nil.
;;   L'utilisation de la primitive listen permet de poursuivre l'valuation sans
;; attendre davantage, au cas o l'utilisateur n'a tap qu'un <return> aprs le
;; macro-caractre qui est  l'origine de l'appel.
(defun string-to-list (flux)
  (do* ((ligne (if (listen flux) (read-line flux t nil t)
                   ""))
        (long (length ligne))
        (pt 0)
        (result nil))
       ((= pt long) (nreverse result))
    (multiple-value-bind (nom indx)
                         (read-from-string ligne nil nil :start pt)
      (setq pt indx)
      (when nom (push nom result)))))

;;   Pour la validation multi-processus des dfinitions de macro-caractres
;; ci-dessus.
(top-level:setq-default common-lisp:*readtable* (copy-readtable))

;;   Environnement SONVS Common LISP sous UNIX - pour terminaux X, clavier 107,
;; fentre xterm.
(top-level:alias "sonvs" (&rest args)
    (format *debug-io* "~%Aide SONVS Common LISP~2%")
    (labels
      ((docdoc (sujets)
         (let ((lesuj (car sujets)))
           (cond 
            ((endp sujets) 'ok)
            ((or (equal 'fenetre lesuj) (equal 1 lesuj))
             (format *debug-io* "1\fenetre~%")
               (format *debug-io*
                 "~4Tcopier/coller~19T~
                  bouton gauche~34Tselectionner et copier ~
                  au curseur de souris~%")
               (format *debug-io*
                 "~19Tbouton milieu~34Tcoller au curseur de texte~%")
               (format *debug-io*
                 "~19Tbouton droite~34Tetendre la selection ~
                  au curseur de souris~%")
               (format *debug-io*
                 "~4Tascenseur~19T~
                  bouton gauche~34Tdescendre (haut=>lent ; bas=>rapide)~%")
               (format *debug-io*
                 "~19T bouton milieu~34Tpositionner directement~%")
               (format *debug-io*
                 "~19Tbouton droite~34Tmonter (haut=>lent ; bas=>rapide)~%")
               (docdoc (cdr sujets)))
            ((or (equal 'macros lesuj) (equal 2 lesuj))
             (format *debug-io*
               "2\macros (et commandes top-level), ~
                cf. /home/dlo/.clinit.cl pour plus de details~%")
               (format *debug-io*
                 "~4T:sonvs [1fenetre] [2macros] [3clavier]~46T=> ~
                  cette aide elle-meme~%")
               (format *debug-io* "~4T:h~46T=> abreviation pour :history~%")
               (format *debug-io*
                 "~4Tcommodites~24T(setqq ...), (prins ...)~%")
               (format *debug-io*
                 "~4Tmise au point~24T~
                  (continu ...), (pasapas ...), (iter ...), (dbg ...)~%")
               (format *debug-io*
                 "~4Tinteraction cshell~24T(sh <com.>)  !<com.>~46T=> ~
                  executer <com.> cshell~%")
               (format *debug-io*
                 "~24T:lo {fic[.cl]}*~46T=> charger le(s) fichier(s)~%")
               (format *debug-io*
                 "~24T^A{fic[.cl]}*~46T=> idem~%")
               (format *debug-io*
                 "~4Tediteur vi~24T(vi fic[.cl])~46T=> edition vi simple~%")
               (format *debug-io*
                 "~24T(v {fic[.cl]}*)~46T=> vi et memoris. nom(s) fic.(s)~%")
               (format *debug-io*
                 "~24T^E{fic[.cl]}*~46T=> idem~%")
               (format *debug-io*
                 "~24T^F{<fonction(s)>}*~46T=> vi et load de fic.(s) def.(s)~%")
               (format *debug-io*
                 "~4Tlib. stochastique~24T:ld-stoch {type}*~46T=> ~
                  charger lib.partie(s) type(s)~%")
               (docdoc (cdr sujets)))
            ((or (equal 'clavier lesuj) (equal 3 lesuj))
             (format *debug-io* "3\clavier~%")
               (format *debug-io*
                 "~4Tabreviations par touches~30T~
                  \"Help\"~46T=> \":sonvs ...\"~%")
               (format *debug-io*
                 "~30T\"= clav. num.\"~46T=> \":lo ...\"~%")
               (format *debug-io*
                 "~30T\"/ clav. num.\"~46T=> \"(v ...\"~%")
               (format *debug-io*
                 "~30T\"* clav. num.\"~46T=> \"(vi ...\"~%")
               (format *debug-io*
                 "~30T\"F1\"~46T=> \"ZZ<cr>\"~%")
               (format *debug-io*
                 "~30T\"F2\"~46T=> clear fenetre~%")
               (format *debug-io*
                 "~30T\"0 clav. num.\"~46T=> \":res<cr>\"~%")
               (format *debug-io*
                 "~30T\"Stop\"~46T=> \":ex<cr>\"~%")
               (docdoc (cdr sujets)))
            (t (format *debug-io* "~A : option inconnue.~%" lesuj))))))
      (docdoc (if args args '(fenetre macros clavier)))
      (format *debug-io* "~%")))
(format *debug-io* "~&;; La commande top-level \":sonvs\" est definie.~%")
