;===============================================================================
;;; 18.1 --> FILTRES SIMPLES (pattern matching)
;$$ 18.1 --> EINFACHE MUSTERERKENNUNG                                 [d..H.B.]
;++ 18.1 --> SIMPLE PATTERN MATCHING
;===============================================================================

;;   Principe de cration d'un <crible>  partir de seux squences <s1> et <s2>,
;; par rcursion non-terminale :
;$   Konstruktionsprinzip eines Siebs <crible> aus zwei Sequenzen <s1> und <s2>
;$ durch nicht-endrekursive Rekursion:
;+   Principle for creating a <crible> sieve from two sequences <s1> and <s2>,
;+ by non tail-recursive recursion:
;-   (defun crible (s1 s2) (cond                                      ;clauses :
;-     ((endp s1) nil)                                                ;terminale
;-     (<predicat> (<action positive> (crible (cdr s1) (cdr s2))))    ;positive
;-     (t          (<action negative> (crible (cdr s1) (cdr s2))))))  ;ngative

;;   Application littrale du principe prcdent pour raliser une fonction
;; quivalente  ETelem1 (cf. ped.13) : on compare deux squences et construit
;; un rsultat comportant un t chaque fois que leurs lments respectifs sont
;; identiques, et un nil autrement. Exemple :
;$   Wortwrtliche Anwendung des vorausgehenden Konstruktionsprinzips um eine
;$ equivalente Funktion zu ETelem1 (vgl. ped.13) zu erhalten: zwei Sequenzen
;$ werden verglichen und das Ergebnis wird aus einem t fr identische
;$ Elemente und einem nil andererseits zusammengebaut. Beispiel:
;+   Literal application of the preceding principle to realize an equivalent
;+ function to ETelem1 (cf. ped.13): two sequences are compared and a result is
;+ built containing a t every time their respective elements are identical, and
;+ a nil otherwise. Example:
;-    (ETelem2 '(a b c d) '(a X c W)) => (T NIL T NIL)

(defun ETelem2 (s1 s2) (cond                                          ;clauses :
   ((endp s1) nil)                                                    ;terminale
   ((equal (car s1) (car s2)) (cons t (ETelem2 (cdr s1) (cdr s2))))   ;positive
   (t (cons nil (ETelem2 (cdr s1) (cdr s2))))))                       ;ngative

;;   Dans le cas prcdent, on peut abrger en combinant les clauses positive et
;; ngative en une seule. On construit ici explicitement l'effet de la primitive
;; mapcar de la version ETelem1 (cf. ped.13). Exemple :
;$   Im vorhergehenden Fall kann man den positiven und negativen Ausdruck in
;$ einem Ausdruck zusammengefassen. Der Effekt des mapcar-Primitives in der
;$ ETelem1-Version wird hier ausdrcklich konstruiert. Beispiel:
;+   In the preceding case, the positive and negative clauses can be combined in
;+ a single one. The effect of the mapcar primitive of the ETelem1 version is
;+ here explicity constructed. Example:
;-    (ETelem3 '(a b c d e) '(a X c W e)) => (T NIL T NIL T)

(defun ETelem3 (s1 s2)                                                ;clauses :
   (if (endp s1) nil                                                  ;terminale
   (cons (equal (car s1) (car s2)) (ETelem3 (cdr s1) (cdr s2)))))     ;active

;;   Application d'un crible, ou masque, <msk>  une squence <seq>, afin d'en
;; slectionner certains lments. La primitive and opre la slection.
;$   Anwendung eines <msk>-Siebes, oder Maske, auf eine Sequenz <seq> zur
;$ Auswahl bestimmter Elemente. Das and-Primitive steuert die Auswahl.
;+   Application of a <msk> sieve, or mask, to a sequence <seq>, in order to
;+ select certain of its elements. The and primitive operates the selection.
;- Exemple : / Beispiel: / Example:
;-    (masque1 '(t nil t t nil) '(1 2 3 4 5)) => (1 NIL 3 4 NIL)

(defun masque1 (msk seq)
  (if (endp msk) nil
  (cons (and (car msk) (car seq)) (masque1 (cdr msk) (cdr seq)))))

;;   Mme application d'un masque <msk>  une squence <seq>, mais sans
;; conserver les nil marquant les liminations d'lments. Exemple :
;$   Dieselbe Anwendung einer Maske <msg> auf die Sequenz <seq>, aber ohne die
;$ nil-Atome, welche die gelschten Elemente markieren. Beispiel:
;+   Same application of a <msk> mask to a sequence <seq>, but not keeping the
;+ nil atoms pointing to eliminated elements. Example:
;-    (masque2 '(t nil t t nil nil) '(1 2 3 4 5 6)) => (1 3 4)
;- quivaut  / Equivalent zu / Equivalent to
;-    (remove nil (masque1 <msk> <seq>))

(defun masque2 (msk seq) (cond
   ((endp msk) nil)
   ((car msk) (cons (car seq) (masque2 (cdr msk) (cdr seq))))
   (t (masque2 (cdr msk) (cdr seq)))))
  
;;   Dsignation de caractres spciaux pour dfinir des lments variables dans
;; un filtre. Slection de caractres de type constituent, mais suffisamment
;; visibles ou mnmoniques, et ne prtant pas  confusion avec les primitives
;; arithmtiques.
;$   Festlegung bestimmter Zeichen zur Markierung variabler Elemente in einem
;$ Muster. Auswahl von Zeichen, die einzeln stehen, aber gengend sichtbar oder
;$ wiedererkennbar sind, und nicht anfllig sind, mit den arithmetischen
;$ Primitives verwechselt zu werden.
;+   Designation of special characters to define variable elements in a pattern.
;+ Selection of characters of type constituent, but sufficiently visible or
;+ mnemonic, and not liable of being confounded with the arithmetic primitives.

;;   Dnombrements sans restrictions :
;;    _         := un et un seul lment quelconque                   _ := [1]
;;    $         := un ou plusieurs lment(s) quelconque(s)           $ := [1,n[
;;    %         := zro ou un lment quelconque                      % := [0,1]
;;    ~         := zro, un ou plusieurs lment(s) quelconque(s)     ~ := [0,n[
;$   Uneingeschrnkte Auswahl:
;$    _         := ein beliebiges Element                             _ := [1]
;$    $         := eines oder mehrere beliebige Elemente              $ := [1,n[
;$    %         := kein oder ein beliebiges Element                   % := [0,1]
;$    ~         := kein oder eines oder mehrere beliebige Elemente    ~ := [0,n[
;+   Unrestricted Numberings:
;+    _         := any one and only element                           _ := [1]
;+    $         := any one or many element(s)                         $ := [1,n[
;+    %         := zero or any one and only element                   % := [0,1]
;+    ~         := zero, or any one or many element(s)                ~ := [0,n[

;;   Dnombrements de possibilits numres :
;;    (?_ <li>) := un et un seul lment parmi ceux de la liste <li>
;;    (?$ <li>) := un ou plusieurs lment(s) parmi ceux de <li>
;;    (?% <li>) := zro ou un lment parmi ceux de <li>
;;    (?~ <li>) := zro, un ou plusieurs lment(s) parmi ceux de <li>
;$   Auswahl aus endlichen Mglichkeiten:
;$    (?_ <li>) := ein einziges Element aus der Liste <li>
;$    (?$ <li>) := eines oder mehrere Elemente aus <li> 
;$    (?% <li>) := kein oder ein einziges Element aus <li>
;$    (?~ <li>) := kein, eines oder mehrere Elemente aus <li> 
;+   Numberings of enumerated possibilities:
;+    (?_ <li>) := one and only element among those of the <li> list
;+    (?$ <li>) := one or many element(s) among those of <li>
;+    (?% <li>) := zero or one and only element among those of <li>
;+    (?~ <li>) := zero, or any one or many element(s) among those of <li>

;;   Point de dpart : dfinition d'une fonction rcursive testant l'galit
;; rigoureuse -- il s'agit de la fonction egal, cf. fichier ped.11.2a.
;;    egal <a> <b> => t si et seulement si <a> et <b> sont des objets de
;;                    structure et contenu identiques. quivalente LISP : equal
;$   Ausgangspunkt: Definition einer rekursiven Funktion, die auf strikte
;$ Gleichheit testet -- es handelt sich um die egal-Funktion, Vgl. Datei
;$ ped.11.2a.
;$    egal <a> <b> => t falls und nur falls <a> und <b> in Struktur und Inhalt
;$                    identische Objekte sind. LISP-Equivalent: equal
;+   Starting point: definition of a recursive function testing strict equality
;+ --namely the egal function, cf. ped.11.2a file.
;+    egal <a> <b> => t if and only if <a> and <b> are identical objects in
;+                    structure and content. LISP equivalent: equal

(defun egal (a b)
  (cond ((atom a) (eq a b))                                    ;cas terminal
        ((atom b) nil)                                         ;cas trivial
        ((egal (car a) (car b))                                ;confirmer...
          (egal (cdr a) (cdr b)))                              ;... par la suite
        (t nil)))                                              ;faux immdiat

;;   Sur ce modle : fonction testant l'galit, mais  une et une seule
;; exception prs. Ralisation du rle du caractre spcial
;$   Nach demselben Prinzip: Funktion, die auf Gleichheit testet, jedoch mit
;$ einer einzigen mglichen Ausnahme. Umsetzung des Verhaltens eines
;$ Sonderzeichens.
;+   On the same model: function testing equality, but with one and only
;+ possible exception. Realization of the role of the special character
;-    _

(defun f1 (f a)
  (cond ((atom f) (eq f a))                           ;cf. egal
        ((eq (car f) '_)                              ;\
          (if (atom a) nil                            ; 3 lignes ajoutes pour _
          (f1 (cdr f) (cdr a))))                      ;/
        ((atom a) nil)                                ;cf. egal
        ((f1 (car f)(car a))                          ;cf. egal
          (f1 (cdr f)(cdr a)))                        ;cf. egal
        (t nil)))                                     ;cf. egal

;;   Ralisation des rles de tous les caractres spciaux de dnombrements sans
;; restrictions
;$   Umsetzung sowohl des Verhaltens aller Sonderzeichen fr uneingeschrnkte
;$ Auswahl
;+   Realization of the roles of all special characters for unrestricted
;+ numberings
;-    _  $  %  ~
;; ainsi que des dnombrements de possibilits numres
;$ als auch derer zur Auswahl aus endlichen Mglichkeiten
;+ as well as the numberings of enumerated possibilities
;-    (?_ <li>)  (?$ <li>)  (?% <li>)  (?~ <li>)

(defun f2 (f a)    (declare (notinline f2))
;(dbg f f2) (dbg a f2)
  (cond ((atom f) (eq f a))                                           ;cf. egal
        ((eq (car f) '_)                                              ;_
          (if (atom a) nil
          (f2 (cdr f) (cdr a))))
        ((eq (car f) '$) (cond                                        ;$
          ((atom a) nil)
          ((f2 (cdr f) (cdr a)))
          ((f2 f (cdr a)))))
        ((eq (car f) '%) (cond                                        ;%
          ((and a (atom a)) nil)
          ((f2 (cdr f) a))
          ((f2 (cdr f) (cdr a)))))
        ((eq (car f) '~) (cond                                        ;~
          ((and a (atom a)) nil)
          ((null (cdr f)) t)
;            ((equal (cadr f) '~)         ;Ensemble de clauses constituant un
;                (f2 (cdr f) a))          ;exemple d'approche erratique et de
;            ((and    (consp (cadr f))    ;bricolage ad hoc : MAUVAISE SOLUTION
;                (equal (caadr f) '?~)    ;...
;                (null a))                ;...
;                    (f2 (cdr f) a))      ;...comportant en outre un danger de
;            ((null a) nil)               ;boucle infinie
;            ((f2 (cdr f) a))             ;...
;            ((f2 f (cdr a)))))           ;...
          ((SEq0n f a))))                 ;BONNE SOLUTION : dlgation  SEq0n !
        ((and (consp (car f)) (eq (caar f) '?_)) (cond                ;(?_ <li>)
          ((atom a) nil)
          ((member (car a) (cdar f) :test #'equal)
            (f2 (cdr f) (cdr a)))))
        ((and (consp (car f)) (eq (caar f) '?$)) (cond                ;(?$ <li>)
          ((atom a) nil)
          ((member (car a) (cdar f) :test #'equal)
            (or (f2 (cdr f) (cdr a))
                (f2 f (cdr a))))))
        ((and (consp (car f)) (eq (caar f) '?%)) (cond                ;(?% <li>)
          ((and a (atom a)) nil)
          ((f2 (cdr f) a))
          ((member (car a) (cdar f) :test #'equal)
            (f2 (cdr f) (cdr a)))))
        ((and (consp (car f)) (eq (caar f) '?~)) (cond                ;(?~ <li>)
          ((and a (atom a)) nil)
          ((f2 (cdr f) a))
          ((member (car a) (cdar f) :test #'equal)
            (f2 f (cdr a)))))
        ((atom a) nil)                                                ;cf. egal
        ((f2 (car f)(car a))                                          ;cf. egal
          (f2 (cdr f)(cdr a)))                                        ;cf. egal
        (t nil)))                                                     ;cf. egal

;;   Gestion spcifique du caractre spcial de filtre
;$   Spezielle Behandlung des Muster-Sonderzeichens
;+   Specific handling of the special pattern character
;-    ~
;; Cette fonction compare des squences de un ou plusieurs caractres spciaux
;; ~ dans le filtre <fi>,  des squences allant de <n>  zro lments de
;; l'assertion <as>.
;$ Diese Funktion vergleicht Folgen von einem oder mehreren ~ Sonderzeichen im
;$ Muster <fi> mit Folgen, die aus <n> bis null Elementen der <as>-Aussage
;$ bestehen.
;+ This function compares sequences of one or many ~ special characters in the
;+ <fi> pattern to sequences ranging from <n> to zero elements of the <as>
;+ assertion.

(defun SEq0n (fi as)
;(dbg fi SEq0n) (dbg as SEq0n)
  (cond (as (if (f2 (cdr fi) as) t
            (progn (pop as)                 ;Clause finale : serait normalement
                   (SEq0n fi as))))         ;suffisante. Mais il vaut mieux
        ;(t nil)))               ;<========= puiser totalement le filtre, pour
        ((f2 (cdr fi) as))))                ;assurer la validit de l'algorithme
                                            ;mme dans le cas d'critures
                                            ;"abusives" du type "(... ~ ~ ~)"
