;;;=============================================================================
;;; 18.2 - Annexe C --> FILTRES LABORS (pattern matching) : liaison d'lments
;;;                     d'assertions    des  variables  de  filtres,  calcul de
;;;                     contraintes et d'lments de filtres
;;;
;;;               c. -> FILTRE f3 PROPREMENT DIT
;;;=============================================================================

;;   Fonction interface d'habillage.
;;    f3 <filtre> <asser> [:rien] => deux values :
;;                               0. t ou nil, selon succs ou echec global
;;                               1. une A-liste reprsentant les variables de
;;                                  filtre et leurs liaisons  l'issue du
;;                                  filtrage
;;   Le mot-clef :rien permet de donner une autre valeur, approprie au
;; contexte,  la variable spciale *rien* [*rien* par dfaut]. La valeur de
;; cette variable spciale *rien* sera attribue  toute variable de filtre
;; pendant qu'elle ne sera lie  aucun lment d'assertion. Si, par exemple,
;; les variables de filtres doivent tre impliques dans des oprations
;; arithmtiques, il pourra tre plus appropri d'attribuer dans ce contexte une
;; valeur numrique  la variable spciale *rien*. Par exemple, avec un appel :
;;    (f3 <filtre> <assertion> :rien -9999999)

(defun f3 (flt ass &key (rien '*rien*))
  (let ((*A-LISTE* nil)
        (*rien* rien))
    (declare (special *rien* *A-LISTE*))
    (values (fX flt ass) (reverse *A-LISTE*))))

;;   Fonction effective.

(defun fX (f a &aux z1 z2) (declare (special *rien* *A-LISTE*))
  (when bugbug (dbg f fX) (dbg a fX) (dbg *A-LISTE* fX) (dbg *rien* fX))
  (cond
    ((atom f) (eq f a))                                                ;cf. egal
    ((and (consp (car f)) (eq (caar f) '|@|))                                 ;@
     (setq z1 (cdar f)) (cond
         ((atom z1)
          (setq z2 (Xassoc z1 *A-LISTE*)) (cond
              ((eq *rien* z2)
               (fX (cdr f) a))
              ((eq '*undef* z2)
               (error "la variable de filtre \"~A\" est \"~A\".~%" z1 z2))
              (t (fX (append (if (consp z2) z2 (list z2))
                             (cdr f))
                     a))))
         (t (setq z2 (escargot z1 *A-LISTE*))
            (when bugbug
              (prins "    ->expr. : " f)
              (prins "    <-expr. : " (append (if (atom z2)(list z2) z2)
                                              (cdr f))))
            (fX (append (if (atom z2) (list z2) z2)
                        (cdr f))
                a))))
    ((and (consp (car f)) (eq (caar f) '|^|))                                ;^
     (setq z2 (if (atom a) a (car a)))
     (when (setq z1 (cadar f))
       (delier z1)
       (setq *A-LISTE* (acons z1 z2 *A-LISTE*)))
     (when (chapeau (cddar f) z2 *A-LISTE*)
       (fX (cdr f) (cdr a))))
    ((and (consp (car f)) (eq (caar f) '|_|)) (cond                           ;_
        ((null (setq z1 (cdar f)))
         (if (atom a) nil
             (fX (cdr f) (cdr a))))
        (t (delier z1)
           (if (atom a) nil
               (progn (setq *A-LISTE* (acons z1 (car a) *A-LISTE*))
                      (fX (cdr f) (cdr a)))))))
    ((and (consp (car f)) (eq (caar f) '|$|)) (cond                           ;$
        ((null (setq z1 (cdar f)))
         (if (atom a) nil
             (SEq1n f a)))
        (t (delier z1)
           (if (atom a) nil
               (progn (setq *A-LISTE* (acons z1 *rien* *A-LISTE*))
                      (SEqv1n z1 f a))))))
    ((and (consp (car f)) (eq (caar f) '|%|)) (cond                           ;%
        ((null (setq z1 (cdar f))) (cond
            ((and a (atom a)) nil)
            ((fX (cdr f) a))
            (t (fX (cdr f) (cdr a)))))
        (t (delier z1) (cond
            ((and a (atom a)) nil)
            (t (setq *A-LISTE* (acons z1 *rien* *A-LISTE*))
               (if (fX (cdr f) a) t
                   (progn (Anconc (assoc z1 *A-LISTE*) (car a))
                          (fX (cdr f) (cdr a)))))))))
    ((and (consp (car f)) (eq (caar f) '|~|)) (cond                           ;~
        ((null (setq z1 (cdar f))) (cond
            ((and a (atom a)) nil)
            ((null (cdr f)) t)
            (t (SEq0n f a))))
        (t (delier z1) (cond
            ((and a (atom a)) nil)
            ((null (cdr f))
             (setq *A-LISTE* (acons z1 (if a a *rien*) *A-LISTE*))
             t)
            (t (setq *A-LISTE* (acons z1 *rien* *A-LISTE*))
               (SEqv0n z1 f a))))))
    ((and (consp (car f)) (equal (caar f) "?_")) (cond               ;("?_" ...)
        ((atom a) nil)
        ((null (setq z1 (cadar f)))
         (ORx1 (cddar f) f a))
        (t (delier z1)
           (setq *A-LISTE* (acons z1 (car a) *A-LISTE*))
           (ORx1 (cddar f) f a))))
    ((and (consp (car f)) (equal (caar f) "?$")) (cond               ;("?$" ...)
        ((atom a) nil)
        ((null (setq z1 (cadar f)))
         (ORx1n (cddar f) f a))
        (t (delier z1)
           (setq *A-LISTE* (acons z1 *rien* *A-LISTE*))
           (ORxv1n z1 (cddar f) f a))))
    ((and (consp (car f)) (equal (caar f) "?%")) (cond               ;("?%" ...)
        ((and a (atom a)) nil)
        ((null (setq z1 (cadar f)))
         (ORx01 (cddar f) f a))
        (t (delier z1)
           (setq *A-LISTE* (acons z1 *rien* *A-LISTE*))
           (ORxv01 z1 (cddar f) f a))))
    ((and (consp (car f)) (equal (caar f) "?~")) (cond               ;("?~" ...)
        ((and a (atom a)) nil)
        ((null (setq z1 (cadar f)))
         (ORx0n (cddar f) f a))
        (t (delier z1)
           (setq *A-LISTE* (acons z1 *rien* *A-LISTE*))
           (ORxv0n z1 (cddar f) f a))))
    ((atom a) nil)                                                     ;cf. egal
    ((fX (car f)(car a)) (fX (cdr f)(cdr a)))))                        ;cf. egal
