;===============================================================================
;;; 19.1b --> PROGRAMMATION OBJET EN CLOS : exemples simples, primitives
;;;           defclass, definstance, etc.
;$$ 19.1b --> OBJEKTORIENTIERTE PROGRAMMIERUNG MIT CLOS: Einfache Beispiele,
;$$           defclass, definstance und andere Primitives             [d..H.B.]
;++ 19.1b --> CLOS OBJECT PROGRAMMING: Simple Examples, defclass, definstance,
;++           and Other Primitives
;===============================================================================

;;   Dfinition d'un ensemble structur de classes d'objets mobiles. Il s'agit
;; d'un exemple classique de programmation par objets -- cf. [Abdulrab:116-123].
;$   Definition eines strukturierten Klassengefges sich bewegender Objekte --
;$ ein klassisches Beispiel fr objektorientierte Programmierung. Vgl.
;$ [Abdulrab:116-123].
;+   Definition of a structured set of classes of moving objects, constituting
;+ a classical instance of object programming--cf. [Abdulrab:116-123].

(defclass objet-mobile
  () 
  ((nom   :initarg :nom)
   (masse :initarg :masse))
  (:default-initargs :nom   nil
                     :masse nil))

(defclass uni-dim
  (objet-mobile)
  ((x-vitesse  :initarg :x-vitesse)
   (x-position :initarg :x-position))
  (:default-initargs :x-vitesse  nil
                     :x-position nil))

(defclass bi-dim
  (uni-dim)
  ((y-vitesse  :initarg :y-vitesse)
   (y-position :initarg :y-position))
  (:default-initargs :y-vitesse  nil
                     :y-position nil))

(defclass tri-dim
  (bi-dim)
  ((z-vitesse  :initarg :z-vitesse)
   (z-position :initarg :z-position))
  (:default-initargs :z-vitesse  nil
                     :z-position nil))

(defclass metro
  (uni-dim)
  ((passagers :initarg :passagers))
  (:default-initargs :passagers 0))

(defclass camion
  (bi-dim)
  ((fret :initarg :fret))
  (:default-initargs :fret nil))

(defclass bateau
  (bi-dim)
  ((fret :initarg :fret))
  (:default-initargs :fret nil))

(defclass aeronef
  (tri-dim)
  ((fret :initarg :fret))
  (:default-initargs :fret nil))

(defclass paquebot
  (bateau)
  ((passagers :initarg :passagers))
  (:default-initargs :passagers 0))

(defclass avion
  (aeronef)
  ((passagers :initarg :passagers))
  (:default-initargs :passagers 0))

;;   Diverses mthodes gnriques permettant de contrler les caractristiques
;; des diffrentes classes d'objets mobiles.
;$   Einigie generische Methoden zur Steuerung der Eigenschaften der
;$ verschiedenen Klassen sich bewegender Objekte.
;+   Various generic methods allowing the control of the characteristics of
;+ different classes of moving objects.

(defmethod positionne ((a1 uni-dim))
  (format t "~10Tx-position~25T=  ~D~%" (slot-value a1 'x-position)))

(defmethod positionne ((a1 bi-dim))
  (format t "~10Ty-position~25T=  ~D~%" (slot-value a1 'y-position))
  (call-next-method a1))

(defmethod positionne ((a1 tri-dim))
  (format t "~10Tz-position~25T=  ~D~%" (slot-value a1 'z-position))
  (call-next-method a1))

(defmethod vitesse ((a1 uni-dim))
  (let ((xvit (slot-value a1 'x-vitesse)) )
    (if xvit (format t "~10Tx-vitesse~25T=  ~D~%" xvit)
    (format t "~10TATTENTION : vitesse inconnue !~%"))))

(defmethod vitesse ((a1 bi-dim))
  (let ((xvit (slot-value a1 'x-vitesse))
        (yvit (slot-value a1 'y-vitesse)) )
    (if (and xvit yvit) (progn
                          (format t "~10Ty-vitesse~25T=  ~D~%" yvit)
                          (format t "~10Tx-vitesse~25T=  ~D~%" xvit)
                          (format t "~10Tvitesse comb.~25T=  ~D~%"
                                  (sqrt (+ (* xvit xvit)
                                           (* yvit yvit)))))
    (format t "~10TATTENTION : vitesse inconnue !~%"))))

(defmethod vitesse ((a1 tri-dim))
  (let ((xvit (slot-value a1 'x-vitesse))
        (yvit (slot-value a1 'y-vitesse))
        (zvit (slot-value a1 'z-vitesse)) )
    (if (and xvit yvit zvit) (progn
                               (format t "~10Tz-vitesse~25T=  ~D~%" zvit)
                               (format t "~10Ty-vitesse~25T=  ~D~%" yvit)
                               (format t "~10Tx-vitesse~25T=  ~D~%" xvit)
                               (format t "~10Tvitesse comb.~25T=  ~D~%"
                                       (sqrt (+ (* xvit xvit)
                                                (* yvit yvit)
                                                (* zvit zvit)))))
    (format t "~10TATTENTION : vitesse inconnue !~%"))))

(defmethod chargement ((a1 objet-mobile))
  (if (slot-exists-p a1 'fret)
    (format t "Transportant du ~A~%" (slot-value a1 'fret))
  (format t "Aucun fret a bord")))

(defmethod accueil-passagers ((a1 metro))
  (format t "La TCL souhaite un bon voyage a ses ~
             ~D passagers !~%"(slot-value a1 'passagers)))

(defmethod accueil-passagers ((a1 paquebot))
  (format t "Le Capitaine Cook souhaite un bon voyage a ses ~
             ~D passagers !~%"(slot-value a1 'passagers)))

(defmethod accueil-passagers ((a1 avion))
  (format t "Jean Mermoz souhaite un bon voyage a ses ~
             ~D passagers !~%"(slot-value a1 'passagers)))

(defmethod identifie ((a1 objet-mobile))
  (format t "Objet-mobile de type ~A :~%" (type-of a1))
  (format t "~10Tnom~25T=  ~A~%" (slot-value a1 'nom))
  (format t "~10Tmasse~25T=  ~A~%" (slot-value a1 'masse))
  (positionne a1)
  (vitesse a1)
  (when (slot-exists-p a1 'fret) (chargement a1))
  (when (slot-exists-p a1 'passagers) (accueil-passagers a1)))

;;   Cration d'objets pour l'exemple, c'est--dire d'instances des classes
;; pr-dfinies.
;$   Erstellen von Beispielobjekten, also von Instanzen der schon definierten
;$ Klassen.
;+   Creation of example objects, namely of instances of the predefined classes.

(setq  ligneD (make-instance    'metro :x-position    9.3
                                       :passagers    27        ))
(setq camion1 (make-instance   'camion :nom        'Chevrolet
                                       :x-position   12.8
                                       :y-position   -4.7
                                       :y-vitesse     0.875
                                       :x-vitesse    -1.2
                                       :fret       'mouron     ))
(setq bateau1 (make-instance   'bateau :nom        'Exxon-Valdez
                                       :masse      'vachement_lourd
                                       :fret       'petrole
                                       :x-position    3.4
                                       :y-position    2        ))
(setq  avion1 (make-instance    'avion :nom        'A-320
                                       :masse      'leger
                                       :x-position    3
                                       :y-position    4
                                       :z-position    0        ))
(setq bateau2 (make-instance 'paquebot :nom        'Titanic
                                       :masse      'tres_lourd
                                       :fret       '(mucho dinero...)
                                       :passagers  3333
                                       :x-position   41.075
                                       :y-position   12.8
                                       :x-vitesse     0.75
                                       :y-vitesse     0.12     ))

;;   Il est possible de modifier les caractristiques d'un objet pr-existant...
;$   Die Eigenschaften eines schon vorhandenen Objekts lassen sich verndern...
;+   It is possible to modify the characteristics of a preexisting object...

;(reinitialize-instance bateau1 :x-vitesse   0
;                               :y-vitesse   0  )
;(reinitialize-instance  avion1 :passagers 128
;                               :y-vitesse   4
;                               :z-vitesse  -0.5)
;(reinitialize-instance  avion1 :passagers  33
;                               :x-vitesse 2.7)
