;===============================================================================
;;; 19.2x.graph --> PROGRAMMATION OBJET EN CLOS :exemple classique du robot
;;;                 manipulateur -- interfaces d'affichage "graphique"
;$$ 19.2x.graph --> OBJEKTORIENTIERTE PROGRAMIERUNG MIT CLOS: Klassisches
;$$                 Robotermanipulator-Beispiel -- "Grafische"        [d..H.B.]
;++ 19.2x.graph --> CLOS OBJECT PROGRAMMING: Classical Manipulating Robot
;++                 Example--"Graphical" Display Interfaces
;===============================================================================

;;;  charger APRS le fichier ped.19.2x.ecran
;$$ Sollte NACH der ped.19.2x.ecran-Datei geladen werden
;++ To be loaded AFTER the ped.19.2x.ecran file

;;   On trouve dans ce fichier une interfaces d'affichage avec la bibliothque
;; LispWorks CAPI.
;;   Il s'agit essentiellement d'une redfinition de la fonction ecran, qui
;; affiche ici la situation des objets dans une fentre "graphique"
;; indpendante. Pour le reste, le fonctionnement de l'ensemble demeure
;; identique.
;$   Diese Datei enthlt eine Anzeigeschnittstelle, welche die LispWorks
;$ CAPI-Bibliothek benutzt.
;$   Sie besteht im Grunde genommen aus einer neuen Definition der ecran-
;$ Funktion, welche jetzt die Lage der Objekte in einem unabhngigen
;$ "grafischen" Fenster anzeigt. Die Arbeitsweise bleibt ansonsten gleich. 
;+   This file contains a display interface using the LispWorks CAPI library.
;+   It essentially consists in a new definition of the ecran function, which
;+ now displays the situation of the objects in an independent "graphical"
;+ window. The operating mode remains otherwise identical.

(defparameter larg-carac 8.4)                           ;largeur d'un caractre
(defparameter epais-lign 13.5)                ;paisseur d'une ligne horizontale
(defparameter taille-fen                             ;taille de la fentre *fen*
  (cons (+ (* 80 larg-carac) 9)
        (+ (* 25 epais-lign) 24)))

;;   Cette version de la fonction ecran affiche une reprsentation "graphique"
;; de la position courante des objets au moment de son appel. Elle peut tre
;; invoque automatiquement par d'autres fonctions, ou manuellement au besoin.
;;   La taille de la fentre *fen* est ajuste pour un affichage en police
;; Monaco 14 pts. On compte 13 pixels par ligne horizontale. Il faut 24 pixels
;; de plus verticalement pour une fentre CAPI (sans doute  cause de la barre
;; de titre).
;$   Diese Version der ecran-Funktion zeigt eine "grafische" Reprsentation der
;$ aktuellen Lage der Objekte im Moment ihres Aufrufs an. Sie kann automatisch
;$ von anderen Funktionen oder wenn ntig manuell aufgerufen werden.
;$   Die Gre des *fen*-Fensters ist an die Schriftart Monaco mit 14pt
;$ angepasst. 13 Pixel werden pro horizontaler Zeile reserviert. 24 Extrapixel
;$ sind fr ein CAPI-Fenster ntig (wahrscheinlich wegen der Titelleiste). 
;+   This version of the ecran function displays a "graphical" representation of
;+   This version of the ecran function displays a "graphical" representation of
;+ the current position of the objects at the moment of its call.  It can be
;+ invoked automatically by other functions, or manually when needed.
;+   The size of the *fen* window is ajusted for a Monaco 14 pts display font.
;+ 13 pixels are allocated per horizontal line. 24 extra pixels are needed for a
;+ CAPI window (probably because of the title bar).

(defun ecran (&aux NomObj)
  (when (not *fen*)
    (setq *fen* (capi::contain
                 (make-instance 'capi::output-pane)
                 :best-width (car taille-fen)
                 :best-height (cdr taille-fen)
                 :title (string (gensym "AZERTYUIOP-"))))
    (gp::set-graphics-state *fen* :font (gp::find-best-font *fen*
                                          (gp:make-font-description
                                           :name "Monaco" 
                                           :size 14
                                           :weight :bold))))
  (capi::apply-in-pane-process *fen*                         ;effacer la fentre
                               'gp::draw-rectangle *fen*
                               0 0 (car taille-fen) (cdr taille-fen)
                               :foreground :white
                               :filled t)
  (capi::apply-in-pane-process *fen*                     ;2 lignes pour la pince
                               'gp::draw-string *fen*
                               (format nil "~39T/@\\")
                               0 epais-lign
                               :foreground :green)
  (capi::apply-in-pane-process *fen*
                               'gp::draw-string *fen*
                               (format nil "~38T[   ]")
                               0 (* 2 epais-lign)
                               :foreground :green)
  (capi::apply-in-pane-process *fen*                     ;le contenu de la pince
                               'gp::draw-string *fen*
                               (format nil "~A" *pris*)
                               (* 39 larg-carac) (* 2 epais-lign)
                               :foreground :red)
  (do ((y (1- *ny*) (1- y)))                            ;tout le tableau *ecran*
      ((minusp y))
    (capi::apply-in-pane-process *fen*
                                 'gp::draw-string *fen*
                                 (format nil " ")
                                 0 (- (- (cdr taille-fen) epais-lign)
                                      (* (+ y 2.5) epais-lign)))
    (dotimes (x *nx*)
      (capi::apply-in-pane-process *fen*
                                   'gp::draw-string *fen*
                                   (format nil "~A"
                                           (setq NomObj (aref *ecran* x y)))
                                   (+ larg-carac (* x 3 larg-carac))
                                   (- (- (cdr taille-fen) epais-lign)
                                      (* (+ y 2.5) epais-lign))
                                   :foreground
                                    (if (and (not (string= NomObj *vide*))
                                             (or (string= NomObj *>pris*)
                                                 (string= NomObj *<pris*)))
                                        :red
                                      :black))))
  (capi::apply-in-pane-process *fen*                                   ;la table
                               'gp::draw-string *fen*
                               *TTT*
                               0 (- (cdr taille-fen) (* 2.5 epais-lign))
                               :foreground :blue)
  (mp::process-wait-with-timeout "pour ralentir...secs" 1.1))      ;<--LispWorks

(setq *fen* nil)