;===============================================================================
;;; 19.2x --> PROGRAMMATION  OBJET  EN  CLOS  : exemple classique du robot
;;;           manipulateur -- interfaces d'affichage
;$$ 19.2x --> OBJEKTORIENTIERTE PROGRAMIERUNG MIT CLOS: Klassisches
;$$           Robotermanipulator-Beispiel -- Anzeigeschnittstellen    [d..H.B.]
;++ 19.2x --> CLOS OBJECT PROGRAMMING: Classical Manipulating Robot Example--
;++           Display Interfaces
;===============================================================================

;;   On trouve dans ce fichier tout ce qui concerne les interfaces d'affichage.
;; Il s'agit de l'impression des actions du <robot> *robot*, avec ou sans
;; visualisation de la situation des objets manipuls, selon la valeur t ou nil
;; de la variable globale ecran.
;$   Man findet in dieser Datei alles die Anzeigeschnittstellen betreffende. Sie
;$ besteht aus der Ausgabe der Aktionen des *robot* <robot>, mit oder ohne
;$ Anzeige der Lage der beeinflussten Objekte, abhngig vom Wert der globalen
;$ Variable ecran.
;+   Everything concerning the display interfaces can be found in this file. It
;+ consists in the printout of the actions of the *robot* <robot>, with or
;+ without display of the situation of the manipulated objects, depending on
;+ the value of the ecran global variable.

;;   L'espace prvu pour la visualisation est de 26 x 20 "pixels" -- positions
;; discrtes : 26 horizontalement sur la *table*, et 20 en hauteur. Les "pixels"
;; sont en ralit des chanes de 3 caractres, contenues dans le tableau
;; *ecran*.
;;   Cette visualisation ne pourrait fonctionner correctement s'il se produisait
;; une situation o il n'y aurait plus de place disponible sur la *table*, ou
;; si une pile dpassait la hauteur de 20 objets. Afin de ne pas compliquer
;; l'exemple, rien n'est prvu ici pour pallier ces dfauts.
;;   Une pince, reprsente au-dessus de l'"cran" de visualisation, contient,
;; s'il y a lieu, l'objet couramment manipul par *robot*.
;$   Die Gre der Anzeige fr die Visualisierung betrgt 26 x 20 "Pixel" --
;$ diskrete Positionen: 26 horizontal auf dem *table*, und 20 vertikal. Die
;$ "Pixel" sind in Wahrheit Zeichenketten mit den im *ecran*-Array enthaltenen
;$ 3 Zeichen.
;$   Diese Visualisierung knnte nicht richtig arbeiten wenn eine Situation
;$ entsteht, in welcher kein Platz mehr auf dem *table* ist, oder ein Stapel
;$ eine Hhe von 20 Objekten berschreitet. Um die Komplexitt dieses Beispiels
;$ zu verringern wird hier nichts getan, um diese Fehler zu beseitigen.
;$   Klammern, die oberhalb des Visualisierungs-"Bildschirms" angezeigt werden,
;$ enthalten, bei Manipulationen, das gerade vom *robot* manipulierte Objekt. 
;+   The display space for the visualization is of 26 x 20 "pixels"--discrete
;+ positions: 26 horizontally on the *table*, and 20 vertically. The "pixels"
;+ are in reality strings of 3 characters contained in the *ecran* array.
;+   This visualization could not work properly if a situation arised in which
;+ there would be no more space available on the *table*, or a stack exceeded
;+ the height of 20 superposed objects. In order to reduce the complexity of
;+ this example, nothing is done here to cure these flaws.
;+   Tongs represented on top of the visualization "screen" contain, as the case
;+ may be, the object currently manipulated by *robot*.

;;; Dfinitions de variables globales
;$$ Definition der globalen Variablen
;++ Definition of Global Variables

(defparameter   *nx*     26)    ;dimension horizontale de *table*
(defparameter   *ny*     20)    ;dimension verticale au-dessus de *table*
(defparameter *vide*  "   ")    ;chane de caractres d'un pixel vide
(defparameter *pris* *vide*)    ;contiendra chane := objet saisi par *robot*
(defparameter *>pris* *vide*)   ;idem := objet qui va tre saisi... (graph.)
(defparameter *<pris* *vide*)   ;idem := objet venant d'tre relch... (graph.)
(defparameter *TTT* (make-string 80 :initial-element #\T))  ;reprsente la table
(defparameter *ecran*         
  (make-array (list *nx* *ny*)  ;prvu pour la visualisation :
              :initial-contents ;tableau *ecran* de 26 x 20 "pixels"
                (make-list *nx*
                  :initial-element
                    (make-list *ny* :initial-element *vide*))))
(defparameter *liXlibr* nil)
(dotimes (i 26) (push i *liXlibr*))    ;liste des emplacement libres sur *table*

;;   Dfinition d'une variable globale pour obtenir ou non une vue des positions
;; courantes des objets. Mettre  t pour mettre en route l'affichage.
;$   Definition einer globalen Variable um eine Anzeige der aktuellen
;$ Objektpositionen zu erhalten oder nicht. Auf t setzen um die Anzeige zu
;$ starten.
;+   Definition of a global variable to display or not a view of the current
;+ positions of the objects. Set to t to start the display.

(defparameter ecran nil)               ;par dfaut : pas de visualisation

;;   Dfinitions d'une variable globale pour la fentre CAPI qui sera utilise
;; ventuellement par une autre version "graphique" de la fonction ecran, cf.
;; fichier ped.19.2x.graph-CAPI. Charger ce dernier fichier pour dfinir la
;; version graphique.
;$   Definition einer globalen Variable fr das CAPI Container-Fenster, das
;$ schlielich von anderen "grafischen" Versionen der ecran Fuktion genutzt
;$ wird, vgl. Datei ped.19.2x.graph-CAPI. Lade letztere Datei um die grafische
;$ Version zu definieren.
;+   Definition of a global variable for the CAPI container-window eventually
;+ used by another "graphical" version of the ecran function, cf.
;+ ped.19.2x.graph-CAPI file. Load the latter file to define the graphical
;+ version.

(defparameter *fen* nil)

;;; Fonctions
;$$ Funktionen
;++ Functions

;;   Cette version de la fonction ecran affiche une vue imprime des positions
;; courantes des objets au moment de son appel. Elle peut tre invoque
;; automatiquement par des fonctions ci-dessous, ou manuellement au besoin.
;$   Diese Version der ecran-Funktion gibt eine gedruckte Anzeige der aktuellen
;$ Objektpositionen zum Zeitpunkt des Aufrufs aus. Sie kann automatisch von
;$ einigen der folgenden Funktionen oder, falls ntig, manuell aufgerufen
;$ werden.
;+   This version of the ecran function displays a printed view of the current
;+ positions of the objects at the time of its call. It can be invoked
;+ automatically by some functions below, or manually when needed.

(defun ecran ()
  (format t "~39T/@\\~%~38T[~A]~%" *pris*)
  (do ((y (1- *ny*) (1- y)))
      ((minusp y))
    (format t " ")
    (dotimes (x *nx*)
      (format t "~A" (aref *ecran* x y)))
    (format t "~%"))
  (format t "~A~%" *TTT*)                                              ;la table
  (mp::process-wait-with-timeout "pour ralentir...secs" 1.5))      ;<--LispWorks

;;   La fonction >>> inscrit la chane de caractres <chk> dans le "pixel" de
;; coordonnes [<x>;<y>].
;$   Die >>>-Funktion schreibt die Zeichenkette <chk> in den "Pixel" mit den
;$ Koordinaten [<x>;<y>].
;+   The >>> function inscribes the character string <chk> in the "pixel" of
;+ coordinates [<x>;<y>].

(defun >>> (chk x y)
  (and (integerp x) (> x -1) (< x *nx*)
       (integerp y) (> y -1) (< y *ny*)
       (setf (aref *ecran* x y) chk)))

;;   La fonction JePrends est appele par la mthode bouge -- cf. ped.19.2a --
;; agissant sur <obj>. Il s'agit de la situation o *robot* saisit <obj>.
;$   Die JePrends-Funktion wird von der bouge-Methode aufgerufen -- Vgl.
;$ ped.19.2a -- und wirkt auf <obj> ein. Diese behandelt die Situation, wenn
;$ *robot* <obj> ergreift.
;+   The JePrends function is called by the bouge method--cf. ped.19.2a--acting
;+ on <obj>. This concerns the situation when *robot* takes hold of <obj>.

(defun JePrends (obj)
  (setq *>pris* (slot-value obj 'ename))
  (when (and ecran *fen*) (ecran))   ;si graph. : "allumer" l'obj. qui sera pris
  (setq *>pris* *vide*)              ;...c'est fait
  (format t "~&~2T@ Je prends ~A.~%" (slot-value obj 'nom))
  (let* ((vxpos (slot-value obj 'eposi))
         (qqpar (consp vxpos))
         (vxx (when qqpar (car vxpos)))
         (vxy (when qqpar (cdr vxpos))))
    (when qqpar
      (>>> *vide* vxx vxy)                     ;vider pixel prcdent s'il avait
      (when (zerop vxy) (push vxx *liXlibr*))));place libre tait sur *table*
  (setq *pris* (slot-value obj 'ename))        ;l'inscrire dans la pince
  (reinitialize-instance obj :eposi '*pris*)   ;noter qu'il est dans la pince
  (when ecran (ecran)))                        ;visualiser ou non

;;   La fonction JePose est appele par la mthode ajoute-sur -- cf. ped.19.2a
;; -- agissant sur <obj> et <sup>. Il s'agit de la situation o *robot* dpose
;; <obj> sur <sup>.
;;   Lorsque <sup> est une <table>, une place libre y est choisie alatoirement
;; dans *liXlibr*.
;$   Die JePose-Funktion wird von der ajoute-sur-Methode aufgerufen --Vgl.
;$ ped.19.2a -- und wirkt auf <obj> und <sup> ein. Diese behandelt die
;$ Situation, wenn *robot* <obj> auf <sup> setzt.
;$   Wenn <sup> ein <table> ist wird ein freier Platz auf ihm zufllig in
;$ *liXlibr* ausgewhlt. 
;+   The JePose function is called by the ajoute-sur method--cf. ped.19.2a--
;+ acting on <obj> and <sup>. This concerns the situation when *robot* puts
;+ <obj> on <sup>.
;+   When <sup> is a <table>, a free space is randomly chosen upon it in
;+ *liXlibr*.

(defun JePose (obj sup)
  (let* ((typsup (type-of sup))
         (possup nil)
         (nx (if (eq 'table typsup)
               (let ((chx (nth (random (length *liXlibr*)) *liXlibr*)))
                 ;(delete chx *liXlibr*)         ;<-anomalie delete -- cf. infra
                 (setq *liXlibr* (remove chx *liXlibr*))
                 chx)
             (car (setq possup (slot-value sup 'eposi)))))
         (ny (if (eq 'table typsup) 0
             (1+ (cdr possup)))))
    (format t "~&~2T@ Je pose ~A sur ~A.~%" (slot-value obj 'nom)
                                            (slot-value sup 'nom))
    (>>> (slot-value obj 'ename) nx ny)              ;inscrire -> nouveau pixel
    (reinitialize-instance obj :eposi (cons nx ny))) ;inscrire position -> objet
  (setq *pris* *vide*)                               ;vider la pince
  (setq *<pris* (slot-value obj 'ename))
  (when ecran (ecran))                               ;visualiser ou non
  (setq *<pris* *vide*)                              ;... c'est fait
  (when (and ecran *fen*) (ecran)))      ;si graph. :  "teindre" l'obj. relch

;;   La fonction BougeAussi est appele par le dmon after de la mthode bouge
;; -- cf. ped.19.2b. Il s'agit de la situation o la mthode bouge agit sur une
;; <plaque> supportant elle-mme un ou plusieurs autre(s) objet(s), numr(s)
;; dans la liste <liobj>.
;$   Die BougeAussi-Funktion wird vom after-Dmon der bouge-Methode aufgerufen
;$ -- Vgl. ped.19.2b. Diese behandelt die Situation wenn die bouge-Methode auf
;$ ein <plaque> einwirkt, welches selbst eines oder mehr andere Objekte
;$ untersttzt, die in der <liobj>-Liste aufgefhrt sind. 
;+   The BougeAussi function is called by the after demon of the bouge method--
;+ cf. ped.19.2b. This concerns the situation when the bouge method acts on a
;+ <plaque>, itself supporting one or more other object(s) enumerated in the
;+ <liobj> list.

(defun BougeAussi (liobj)
  (format t "~&~4T... elle supporte, aussi")
  (dolist (lobjet (mapcar #'(lambda (v) (slot-value v 'nom)) liobj))
    (format t ", ~A" lobjet))
  (format t ".~%")
  (mapcar #'(lambda (obj) (let ((pos (slot-value obj 'eposi)))
                            (>>> *vide* (car pos) (cdr pos))))
          liobj))

;;   La fonction PoseAussi est appele par le dmon after de la mthode
;; ajoute-sur -- cf. ped.19.2b. Il s'agit de la situation o la mthode
;; ajoute-sur agit sur une <plaque> supportant elle-mme un ou plusieurs
;; autre(s) objet(s), numr(s) dans la liste <liobj>.
;$   Die PoseAussi-Funktion wird vom after-Dmon der ajoute-sur-Methode
;$ aufgerufen -- Vgl. ped.19.2b. Diese behandelt die Situation wenn die
;$ ajoute-sur-Methode auf ein <plaque> einwirkt, welches selbst eines oder
;$ mehr andere Objekte untersttzt, die in der <liobj>-Liste aufgefhrt sind. 
;+   The PoseAussi function is called by the after demon of the ajoute-sur
;+ method--cf. ped.19.2b. This concerns the situation when the ajoute-sur method
;+ acts on a <plaque>, itself supporting one or more other object(s) enumerated
;+ in the <liobj> list.

(defun PoseAussi (liobj)
  (dolist (lobjet liobj)
    (let* ((estsur (slot-value (slot-value lobjet 'sur) 'eposi))
           (estx   (car estsur))
           (esty   (1+ (cdr estsur))))
      (>>> (slot-value lobjet 'ename) estx esty)       ;inscrire -> nouveau pix.
      (reinitialize-instance lobjet
                             :eposi (cons estx esty))));inscr. position -> objet
  (format t "~&~4T... avec ce qu'elle porte.~%")
  (when ecran (ecran)))                                ;visualiser ou non

;;   Utilitaire pour debug d'attribution des "pixels" : numrote les positions
;; de la *table*, donne la valeur du champ eposi de tous les objets et l'tat de
;; *liXlibr*.
;$   Hilfsfunktion um die Belegung der "Pixel" zu debuggen: nummeriert die
;$ Positionen des *table* durch, gibt den Wert des eposi-Slots aller Objekte und
;$ den Status von *liXlibr* zurck.
;+   Utility to debug the allocation of "pixels": numbers the positions of the
;+ *table*, gives the value of the eposi slot of all objects and the state of
;+ *liXlibr*.

(defun eposi ()
  (dotimes (i 26) (format t "~3D" i)) (format t "~%")
  (dotimes (i 10)
    (format t " objet~A : ~A~%" i (slot-value (symbol-value
                                  (read-from-string (format nil "*objet~A*" i)))
                                  'eposi)))
  (dotimes (i 10)
    (format t " livre~A : ~A~%" i (slot-value (symbol-value
                                  (read-from-string (format nil "*livre~A*" i)))
                                  'eposi)))
  (dolist (le '("A" "B"))
    (format t "plaque~A : ~A~%"
            le (slot-value (symbol-value
                            (read-from-string (format nil "*plaque~A*" le)))
                           'eposi)))
  (format t "liXlibr : ~A : lng ~A~%" *liXlibr* (length *liXlibr*)))

;;; Note sur une anomalie de la primitive delete
;$$ Anmerkung zu einer Anomalie des delete-Primitives
;++ Note on an Anomaly of the delete Primitive

;;   La forme
;$   Der Ausdruck
;+   The form
;-    (setq <seq> (remove <item> <seq>))
;; est utilise ci-dessus, au lieu de la forme plus concise normalement attendue
;$ wird oben anstelle des normalerweise erwarteten knapperen Ausdrucks genutzt
;+ is used above, instead of the normally expected more succinct form
;-    (delete <item> <seq>)
;;  cause d'une anomalie de la primitive delete -- dj signale dans le
;; fichier ped.14. En effet, lorsque <item> est le premier lment de <seq>, la
;; valeur de cette dernire n'est pas modifie -- bien que la valeur rendue par
;; l'appel soit correcte.
;;   Cette particularit de delete est due  des restrictions de certaines
;; primitives effectuant des "parcours destructifs de structures". Cf. [CLtL2:
;; 178-180, 401].
;;   Exemple de squence d'valuations :
;$ wegen einer Anomalie des delete-Primitives -- wie schon in Datei ped.14
;$ angedeutet. In der Tat wird, wenn <item> das erste Element von <seq> ist, der
;$ Wert des letzteren nicht verndert -- obwohl der beim Aufruf zurckgegebene
;$ Wert stimmt.
;$   Diese Besonderheit deletes besteht wegen Einschrnkungen in bestimmten
;$ Primitives, die eine "destruktive Traversierung der Struktur" durchfhren.
;$ Vgl. [CLtL2: 178-180, 401].
;$   Beispielhafte Aufrufsequenz:
;+ because of an anomaly of the delete primitive--already signaled in the ped.14
;+ file. Indeed, when <item> is the first element of <seq>, the value of the
;+ latter is not modified--although the value returned by the call is correct.
;+   This particularity of delete is due to restrictions in certain primitives
;+ performing "destructive structure traversal". Cf. [CLtL2: 178-180, 401].
;+   Example sequence of evaluations:
;- <a>
;-    (setq x '(1 2 3 4 5 6)) => (1 2 3 4 5 6)
;- <b>
;;    (delete 5 x) => (1 2 3 4 6)   ;correct, et...
;;    x => (1 2 3 4 6)              ;... <seq> est bien modifie
;$    (delete 5 x) => (1 2 3 4 6)   ;richtig, und...
;$    x => (1 2 3 4 6)              ;... <seq> wurde korrekt verndert
;+    (delete 5 x) => (1 2 3 4 6)   ;correct, and...
;+    x => (1 2 3 4 6)              ;... <seq> is rightly modified
;- <c>
;;    (delete 1 x) => (2 3 4 6)     ;correct, MAIS...
;;    x => (1 2 3 4 6)              ;... <seq> n'est PAS modifie !
;$    (delete 1 x) => (2 3 4 6)     ;richtig, ABER...
;$    x => (1 2 3 4 6)              ;... <seq> wurde NICHT verndert!
;+    (delete 1 x) => (2 3 4 6)     ;correct, BUT...
;+    x => (1 2 3 4 6)              ;... <seq> is NOT modified!!
