;===============================================================================
;;; 14 --> ENTRES/SORTIES, LECTURE, CRATION ET CRITURE DE FICHIERS --
;;;        INTERACTIONS AVEC LE SYSTME D'EXPLOITATION :
;;;        notion de input/output-stream, primitives with-open-file, read,
;;;        format -- notions de path, module, package, compilation, etc.
;$$ 14 --> EINGABE/AUSGABE, DATEIEN LESEN, ERSTELLEN UND SCHREIBEN--
;$$        INTERAKTION MIT DEM BETRIEBSSYSTEM: 
;$$        Begriff des Ein-/Ausgabestroms, Primitive with-open-file, read,
;$$        format--Begriffe Pfad, Modul, Packet, Kompilierung, etc.    [d..H.B]
;++ 14 --> INPUT/OUTPUT, READING, CREATING AND WRITING FILES--INTERACTIONS WITH
;++        THE OPERATING SYSTEM:
;++        Notion of Input/Output-Stream, Primitives with-open-file, read,
;++        format--Notions of Path, Module, Package, Compilation, etc.
;===============================================================================

;;   Exemple d'utilisation de primitives de sortie diriges simultanment vers
;; deux destinations (streams) diffrentes :
;;    a. compte-rendu des oprations sur le stream *debug-io*, et
;;    b. criture de rsultats dans un fichier
;;    f0 <n> "<fic>" => calcul  partir de <n> avec impressions sur cran et
;;                      sauvegarde de rsultats dans le fichier <fic>
;;   En Macintosh Common LISP (MCL), les fichiers seront crs par dfaut -- en
;; criture -- ou recherchs -- en lecture -- dans le dossier mme de
;; l'application MCL.
;;   En LispWorks sous Mac OS X, le dossier par dfaut est la racine mme de
;; toute la hirarchie des fichiers -- c'est--dire le directory UNIX / qui est
;; normalement identique au directory /Volumes/<disque_systme>.
;;   Sous UNIX, plus utilement, le directory par dfaut est celui du lancement
;; du processus LISP en cours d'excution.
;;   Avec la notion de path,  la fin du prsent chapitre, on verra comment
;; modifier les prcdents comportements par dfaut.
;$   Anwendungsbeispiel mit auf zwei verschiedene Datenstrme gleichzeitig
;$ gerichteten Ausgabe-Primitives:
;$    a. Protokollierung der Operationen auf dem Datenstrom *debug-io*, und
;$    b. Schreiben des Ergebnisses in eine Datei
;$    f0 <n> "<fic>" => Berechnung in Bezug auf <n> mit Bildschirmausgabe und 
;$                      Speicherung der Ergebnisse in Datei <fic>
;$   In Macintosh Common LISP (MCL) werden die Dateien standardmig im
;$ Verzeichnis des MCL-Programms selbst -- im Schreibmodus -- erstellt oder
;$  -- im Lesemodus -- gesucht.
;$   In LispWorks unter Mac OS X ist das Standardverzeichnis das
;$ Wurzelverzeichnis (root) der Verzeichnisstruktur --  das UNIX-Verzeichnis /,
;$ welches normalerweise dem Verzeichnis /Volumes/<system_disk> entspricht.
;$   Unter UNIX ist das Standardverzeichnis ntzlichererweise das, in welchem
;$ der aktuelle LISP-Prozess gestartet wurde.
;$   Mit dem Begriff des Pfads, am Ende dieses Kapitels, werden wir sehen wie
;$ sich die vorher genannten Standardverhaltensweisen verndern lassen.
;+   Example usage of output primitives simultaneously directed towards two
;+ different streams:
;+   a. log of operations to the *debug-io* stream, and
;+   b. writing of results into a file
;+   f0 <n> "<fic>" => computation on <n> with screen printouts and results
;+                     saved in file <fic>
;+   In Macintosh Common LISP (MCL), the files will be by default created--in
;+ write mode--or searched for--in read mode--in the folder of the MCL
;+ application itself.
;+   In LispWorks under Mac OS X, the default folder is the very root of the
;+ entire file hierarchy--that is the / UNIX directory, normally identical to
;+ the /Volumes/<system_disk> directory.
;+   Under UNIX, more usefully, the default directory is the one from which the
;+ current LISP process has been launched.
;+   With the notion of path, at the end of the present Chapter, we will see how
;+ the preceding default behaviors can be modified.

(defun f0 (n fichier)
  (terpri *debug-io*) (terpri *debug-io*)
  (with-open-file (flux fichier :direction         :output
                                :if-exists         :rename-and-delete
                                :if-does-not-exist :create)
    (f1 n flux))
  (princ "
    ; f0 a fini : c'est dans le fichier " *debug-io*)
  (princ fichier *debug-io*) (princ "...")
  (terpri *debug-io*) (terpri *debug-io*) (terpri *debug-io*)
  'ok)
(defun f1 (s flux)
  (princ "    ; f1 a recu " *debug-io*) (princ s *debug-io*)
  (princ s flux) (princ " " flux)
  (f2 s flux))
(defun f2 (q flux)
  (princ (* q q) flux) (terpri flux)
  (princ  "
    ; f2 le met au carre => " *debug-io*)
  (princ (* q q) *debug-io*))

;;   Interaction avec l'utilisateur : interrogation suivie de lecture de la
;; rponse. Cf. aussi les primitives yes-or-no etc. [CLtL2:609sq.].
;;    ?? "<message>" => impression de <message> suivi d'un "?", sur *query-io*,
;;                      et attente de la rponse tape par l'utilisateur
;$   Interaktion mit dem Benutzer: Abfrage mit anschlieendem Einlesen der
;$ Antwort. Vgl. auch die Primitives yes-or-no etc. [CLtL2:609sq.].
;$    ?? "<message>" => gibt <message> gefolgt von einem "?" auf *query-io* aus
;$                      und erwartet die Eingabe einer Antwort vom Benutzer
;+   Interaction with the user: interrogation followed by reading of the answer.
;+ Cf. also the yes-or-no etc. primitives [CLtL2:609sq.].
;+    ?? "<message>" => printout of <message> followed by a "?", on *query-io*,
;+                      and expectation of the answer typed by the user

(defun ?? (msg)
  (princ msg *query-io*)                                ;impression du <message>
  (princ " ? " *query-io*)                              ;suivi de " ? "
  (read *query-io*))                                    ;valeur lue => rendue

;;   Exemple d'utilisation de la fonction ?? : valuer la forme
;$   Anwendungsbeispiel der Funktion ??: fhre folgenden Ausdruck aus
;+   Example usage of the ?? function: evaluate the form
;-    (patron)
;; pour crer une liste comportant un certain nombre d'lments identiques.
;$ um eine Liste mit einer bestimmten Anzahl gleicher Elemente zu erzeugen.
;+ to create a list including a certain number of identical elements.

(defun patron ()
  (make-list (?? "Combien en faut-il")
             :initial-element (?? "Qu'est-ce qu'on y met")))

;;   Pour imprimer joliment -- "pretty-print" -- une liste de listes <lideli>.
;; La liste de listes est dcompose en lments par deux dolist imbriques.
;; La structure de parenthses est reconstitue par les primitives d'impression
;; format. On obtient une prsentation en matrice, plus lisible. Le mot-clef
;; :largeur [10 caractres par dfaut] permet de contrler l'espacement des
;; colonnes, sparateur(s) blanc(s) inclus.
;$   Eine Liste von Listen <lideli> schn ausgeben -- "pretty-print". Die Liste
;$ von Listen wird durch zwei verschachtelte "dolist"s in Glieder zerlegt. Die
;$ Klammernstruktur wird durch das Ausgabeprimitive format nachgebildet.
;$ Ausgegeben wird eine lesbarere Matrixdarstellung. Das Schlsselwort :largeur
;$ [standardmig 10 Zeichen] erlaubt den Spaltenabstand, inklusive leerer
;$ Trennzeichen, zu bestimmen.
;+   To "pretty-print" a list of lists <lideli>. The list of lists is decomposed
;+ into elements by two nested dolist. The parenthesis structure is rebuilt by
;+ the format printing primitives. The output is a more legible matrix
;+ representation. The :largeur keyword [10 characters by default] allows the
;+ control of the spacing of columns, blank separator(s) included.
;-   Exemple : / Beispiel: / Example:
;-    (setq ll '((A B C D) (1 2 3 4) (* + = !)))
;-    (jolilideli ll :largeur 3) => (
;-                                   (  A  B  C  D)
;-                                   (  1  2  3  4)
;-                                   (  *  +  =  !)
;-                                  )
;;   Aligne par dfaut  droite. Spcifier :gauche t pour aligner  gauche.
;$   Standardausrichtung rechtsbndig. Fr linksbndig :gauche t angeben.
;+   Default alignment to the right. Specify :gauche t to align to the left.
;-   Exemple : / Beispiel: / Example:
;-    (jolilideli ll :largeur 3 :gauche t) => (
;-                                             (A  B  C  D  )
;-                                             (1  2  3  4  )
;-                                             (*  +  =  !  )
;-                                            )

(defun jolilideli (lideli &key (largeur 10) gauche)
  (setq largeur (int>string largeur))
  (let ((cadena (concatenate 'string "~" largeur (if (not gauche) "@" "") "S")))
    ;(format t "~S~%" cadena)                                           ;Dmo...
    (format t "~&(~%")
    (dolist (lali lideli)
      (format t " (")
      (dolist (lele lali)
        (format t cadena lele))
      (format t ")~%"))
    (format t ")~%")))

;;   Utilitaire pour la fonction prcdente : convertit un entier positif en une
;; chane de caractres. quivalente LISP : prin1-to-string -- d'ailleurs plus
;; performante.
;$   Hilfsfunktion fr die vorangegangene Funktion: wandelt eine positive
;$ Ganzzahl in eine Zeichenkette um. LISP-Equivalentd prin1-to-string -- im
;$ brigen effizienter.
;+   Utility for the preceding function: converts a positive integer into a
;+ character string. LISP equivalent: prin1-to-string--also more efficient.
;- Exemple : / Beispiel: / Example:
;-    (int>string 40781) => "40781"
;; La primitive push sera prsente au Chapitre 15.
;$ Das Primitive push wird in Kapitel 15 vorgestellt.
;+ The push primitive will be presented in Chapter 15.

(defun int>string (int)
  (when (or (not (integerp int)) (minusp int))
    (error "int>string : ~A n'est pas un entier >= zro.~%" int))
  (if (zerop int) "0"
  (do ((div 10 (* 10  div))
       (uni  1 (* 10  uni))
       (fin (* 10 int)    )
       (rez               ))
      ((> div fin) (apply #'concatenate (cons 'string rez)))
    (push (string (digit-char (floor (/ (rem int div) uni)))) rez))))

;;   Format labor (!), avec accord de pluriel et expression conditionnelle...
;; Les valeurs de <nbr> et <organe> peuvent tre donnes en arguments d'appel,
;; ou, sinon, seront demandes  l'utilisateur au moyen de la fonction ??.
;$   Ausgeklgeltes format(!), mit Pluralansprache und Konditionalausdrcken...
;$ Die Werte fr <nbr> und <organe> knnen als Aufrufparameter bergeben oder
;$ stattdessen vom Benutzer mit der ??-Funktion abgefragt werden.
;+   Elaborate format(!), with plural agreement and conditional expression...
;+ The values for <nbr> and <organe> can be given as calling arguments, or will
;+ be otherwise asked from the user by the ?? function.

(defparameter mots_interdits '(bras nez front sourcil))
(defun poilau (&optional (nbr    (?? "Combien"))
                         (organe (?? "Au quoi")))
  (format t "Nous avons :~%~12T~12,6F poil~:P au...~%... ~
             ~:[~A.~; !!! ~]"
            nbr (member organe mots_interdits) organe))

;;   Lecture intgrale du contenu d'un fichier. On ne fait pas ici de traitement
;; rel des donnes lues -- uniquement un dcompte et une impression  l'cran.
;; Mais toute autre opration peut tre insre en lieu et place de la forme
;; marque "<--ici--".
;$   Lesen des gesamten Inhalts einer Datei. Die gelesenen Daten werden hier
;$ nicht wirklich verarbeitet -- sie werden nur gezhlt und auf dem Bildschirm
;$ ausgegeben. Jedoch kann jede andere Verarbeitung anstatt des mit "<--ici--"
;$ markierten Ausdrucks eingefgt werden.
;+   Reading of the entire content of a file. No real processing of the data
;+ read is performed here--it is only counted and printed on screen. But any
;+ other process can be inserted instead of the form marked "<--ici--".

(defun lecture (fichier)
  (with-open-file (flux fichier :direction         :input
                                :if-does-not-exist :error)

    ;Lecture et traitement des donnes contenues dans <fichier>... La primitive
    ;read rendra l'atome ok en atteignant la fin de <fichier>

    (do ((cpt 1                   (1+ cpt))
         (lue (read flux nil 'ok)         ) )
        ((equal 'ok lue)
          (format *standard-output* "~2%Fin normale du fichier ~A.~2%" fichier))
        (format *standard-output* "~&forme~7D :~%~15T~A" cpt lue)      ;<--ici--
        (setq lue (read flux nil 'ok))
        )

    ;Fin de lecture et traitement
    ))

;; Utile pour SuperVP ou autres commandes pilotes par des fonctions de contrle
;; dfinies par des points d'inflexion -- break-points -- sous forme de couples
;; de nombres valeur-temps, ou temps-valeur
;$ Ntzlich fr SuperVP oder andere Kommandos, welche durch Kontrollfunktionen
;$ beschrieben durch break-points -- Wert-Zeit oder Zeit-Wert-Paare -- gesteuert
;$ werden.
;+ Useful for SuperVP or other commands driven by control functions described by
;+ break-points--value-time or time-value number couples

;;   Fonction crant un fichier <fic> contenant une fonction de contrle :
;$   Funktion zum Erstellen einer Datei <fic>, die eine Kontrollfunktion
;$ enthlt:
;+   Function creating a <fic> file containing a control function:
;-    faire-controle <fin> "<fic>" <fonction-val> <fonction-tps> <valeur> [:svp]
;;   L'utilisation prvue par dfaut est celle des commandes UNIX transpsf,
;; gainsf, etc., en couples valeur-temps. On peut obtenir des couples
;; temps-valeur en donnant la valeur t au mot-clef :svp [nil par dfaut].
;;   La premire valeur du fichier sera <valeur>. Les valeurs suivantes seront
;; dtermines par des valuations successives de <fonction-val>.
;;   Le premier temps sera 0.0. Les temps suivants seront dtermins par le
;; cumul d'valuations successives de <fonction-tps>. Ce dernier argument doit
;; donc tre conu comme une source d'intervalles de temps.
;;   L'criture de la fonction de contrle dans le fichier <fic> est interrompue
;; lorsque le cumul des intervalles de temps atteint ou dpasse <fin>.
;;   Exemple de lancement  partir d'un shell-script UNIX, destin  prparer
;; une fonction de contrle pour transpsf, afin de raliser un effet de phasing
;; (pour Antoine Escudier, 1997) :
;$   Standardmig fr die Benutzung mit den UNIX-Befehlen transpsf, gainsf,
;$ etc. mit Wert-Zeit-Paaren ausgelegt. Zeit-Wert-Paare knnen durch Setzen des
;$ Schlsselworts :svp auf t [standardmig nil] erhalten werden.
;$   Die ersten Werte in der Datei werden <valuer> sein. Die nchsten Werte
;$ werden durch aufeinanderfolgendes Ausfhren der <fonction-val> bestimmt.
;$   Der erste Zeitpunkt wird 0.0 sein. Die folgenden Zeitpunkte werden durch
;$ die Summe der aufeinanderfolgenden Evaluationen der <fonction-tps> bestimmt.
;$ Das letzte Argument muss daher als Grundlage der Zeitspannen verstanden
;$ werden.
;$   Das Schreiben der Kontrollfunktion in die Datei <fic> wird unterbrochen
;$ wenn die Summe der Zeitspannen <fin> erreicht oder berschreitet.
;$   In einem UNIX-Shell-Skript ausgefhrtes Beispiel zur Vorbereitung einer
;$ Kontrollfunktion fr transpsf um einen Phasing-Effekt zu erzeugen (fr
;$ Antoine Escudier, 1997):
;+   The default usage is intended for the transpsf, gainsf, etc. UNIX commands,
;+ with value-time couples. Time-value couples can be obtained by setting the
;+ :svp key-word to t [nil by default].
;+   The first value in the file will be <valeur>. The following values will be
;+ determined by successive evaluations of <fonction-val>.
;+   The first time will be 0.0. The following times will be determined by the
;+ cumulation of successive evaluations of <fonction-tps>. The latter argument
;+ must thus be conceived as a source of time intervals.
;+   The writing of the control function in the <fic> file is interrupted when
;+ the cumulation of time intervals reaches or passes <fin>.
;+   Example launched in a UNIX shell-script intended to prepare a control
;+ function for transpsf, in order to produce a phasing effect (for Antoine
;+ Escudier, 1997):
;-    ...
;-    DUR=`querysf -d $1`
;-    ...
;-    (faire-controle $DUR (quote $1.fun) (quote (+ 1 (/ (random 1.0) 300.0)))
;-       (quote (random  0.5)) 1.0)
;-    ...
;; Mme exemple, lanc textuellement :
;$ Dasselbe Beispiel, direkt ausgefhrt:
;+ Same example, launched textually:
;-    (faire-controle 20.5 "vcl3.fun" '(+ 1 (/ (random 1.0) 300.0))
;-      '(random  0.5) 1.0)

(defun faire-controle (fin fic fonction-val fonction-tps valeur
                      &key svp &aux (temps 0.0)) 
  (with-open-file (stream fic :direction         :output
                              :if-does-not-exist :create
                              :if-exists         :rename-and-delete)
    (if svp (format stream "~20,6F~20,6F" temps valeur)
        (format stream "~20,6F~20,6F" valeur temps))    
    (format stream "~%")
    (labels ((interne ()
               (setq valeur (eval fonction-val))
               (incf temps (eval fonction-tps))
               (if svp (format stream "~20,6F~20,6F" temps valeur)
                   (format stream "~20,6F~20,6F" valeur temps))    
               (format stream "~%")
               (when (< temps  fin) (interne))))
      (interne)))
  (format t "C'est dans le fichier ~A." fic) 'ok)

;; Utile pour SuperVP
;$ Ntzlich fr SuperVP
;+ Useful for SuperVP

;;   Fonction prparant un fichier contenant des colonnes de paramtres
;; numriques pour les besoins d'un traitement par SuperVP :
;$   Funktion, die eine Datei mit Spalten numerischer Parameter fr einen
;$ SuperVP-Prozess vorbereitet:
;+   Function preparing a file containing columns of numerical parameters for
;+ a SuperVP process:
;-    faire-svp <liste-temps> [<liste-des-v1> ... <liste-des-v<n>>] [:fichier]
;;   Seule une liste des temps est essentielle... mais elle sera suivie d'une ou
;; plusieurs listes de valeurs pour autant de paramtres, en nombre quelconque
;; selon le traitement SuperVP envisag. La vritable utilit de la fonction
;; faire-svp est purement administrative : elle rorganise les listes --
;; horizontales -- des temps et de valeurs, en colonnes -- verticales -- de
;; paramtres numriques
;$   Essentiell ist nur eine Zeitpunktliste... nach ihr aber folgen eine oder
;$ mehrere Listen mit Werten fr lauter Parameter, in beliebiger, vom SuperVP-
;$ Prozess bentigter, Anzahl. Der wahre Nutzen der Funktion faire-svp ist rein
;$ administrativ: sie organisiert die -- horizontalen -- Zeitpunkt- und
;$ Wertlisten in -- vertikale -- Spalten numerischer Parameter
;+   Only the list of times is essential... but it will be followed by one or
;+ many lists of values for as many parameters, in any number required by the
;+ chosen SuperVP process. The real usefulness of the faire-svp function is
;+ purely administrative: it reorganizes the--horizontal--times and values
;+ lists into--vertical--columns of numerical parameters
;-    t1 v11 v12 ... v1<n>
;-    t2 v21 v22 ... v2<n>
;-    ...
;; dans le fichier dsign par le mot-clef :fichier ["svp-fait" par dfaut].
;$ in der vom Schlsselwort :fichier [standardmig "svp-fait"] festgelegten
;$ Datei.
;+ in the file designated by the :fichier key-word ["svp-fait" by default].

(defun faire-svp (liste-temps &rest listdelist-vX
                              &aux (fichier "svp-fait"))
"
 faire-svp <liste-temps> [<liste-des-v1> ... <liste-des-v<n>>] [:fichier]
"
  ;Le mot-clef :fichier est trait ici explicitement, car l'usage normal du
  ;mot-clef &key dans la lambda-liste annulerait l'avantage du recours au
  ;mot-clef &rest, qui est de permettre de donner un nombre quelconque de       
  ;listes de valeurs en arguments d'appel. On peut ainsi spcifier un nombre
  ;quelconque de champs de valeurs v1 ... v<n>
  (let ((oui (member :fichier listdelist-vX)))
    (when oui (setq fichier (cadr oui))))
  ;Utilisation de (setq listdelist-vX (remove ...)), au lieu de (delete ...), 
  ;cause d'une anomalie de la primitive delete, au cas o aucune liste ne
  ;prcderait le mot-clef :fichier. Cette particularit de delete est dcrite
  ;plus compltement  la fin du fichier ped.19.2x.ecran.
  ;Par prcaution, au cas, extrme, o il n'y aurait aucun champ v1 ... v<n>.
  (setq listdelist-vX (remove :fichier listdelist-vX))
  (setq listdelist-vX (remove  fichier listdelist-vX))
  ;
  (with-open-file (flux fichier :direction         :output
                                :if-exists         :rename-and-delete
                                :if-does-not-exist :create)
    (dotimes (num (list-length liste-temps))
      (format flux "~12,5F" (nth num liste-temps))
      (dolist (lali listdelist-vX)
        (format flux "~12,5F" (nth num lali)))
      (format flux "~%")))
  (format t "C'est dans le fichier ~A." fichier) 'ok)

;; Utile pour Csound
;$ Ntzlich fr Csound
;+ Useful for Csound

;;   Cette fonction prpare des squences de notes pour Csound. Les notes sont
;; destines  l'instrument <instr>, commencent  la date <debut>, et ont
;; successivement les dures de <list-durees>. Il y a autant de notes que
;; d'lments dans <list-durees>.
;;   Les arguments d'appel suivants, optionnels et en nombre quelconque, doivent
;; tre des listes de valeurs, chacune pour l'un des champs p4 p5 ... p<n> des
;; notes.
;;   Deux mots-clef, :synchro et :legato, peuvent prendre la valeur nil ou t
;; [nil par dfaut]. Leur effet est dcrit dans le tableau suivant :
;$   Diese Funktion bereitet Notensequenzen fr Csound vor. Die Noten sind fr
;$ das Instrument <instr>, beginnen am Zeitpunkt <debut>, und haben sukzessiv
;$ die Dauern aus <list-durees>. Es gibt genau so viele Noten wie Glieder in
;$ <list-durees>.
;$   Die folgenden Aufrufparameter, optional und in beliebiger Zahl, mssen
;$ Listen mit Werten sein, jeweils fr eines der p4 p5 ... p<n>-Felder einer
;$ Note.
;$   Zwei Schlsselwrter, :synchro und :legato, knnen entweder nil oder t sein
;$ [standardmig nil]. Ihre Wirkung wird in der folgenden Tabelle beschrieben:
;+   This function prepares sequences of notes for Csound. The notes are for the
;+ <instr> instrument, start from time <debut>, and successively have the
;+ durations of <list-durees>. There are as many notes as elements in
;+ <list-durees>.
;+   The following calling arguments, optional and in any number, must be lists
;+ of values, each for one of the p4 p5 ... p<n> fields of the notes.
;+   Two key-words, :synchro and :legato, can take a nil or t value [nil by
;+ default]. Their effect is described in the following table:
;===============================================================================
;- :synchro    :legato    => Effet... / Auswirkung... / Effect...
;-------------------------------------------------------------------------------
;;   nil         nil      => les notes de la squence sont enchanes l'une  la
;;                           suite de l'autre,  partir de la date <debut>, de
;;                           faon "normale" [comportement par dfaut]
;;   nil         t        => les notes de la squence sont enchanes l'une  la
;;                           suite de l'autre,  partir de la date <debut>, en
;;                           mode "legato" (cf. documentation Csound). Une note
;;                           supplmentaire est ajoute  la fin de la squence,
;;                           pour arrter l'activit de l'instrument <instr>
;;   t           nil      => les notes sont toutes attaques ensemble  la date
;;                           <debut>
;;   t           t        => erreur. Cette combinaison n'a pas de sens dans le
;;                           contexte
;$   nil         nil      => Die Noten der Sequenz werden im "normalen"-Modus
;$                           [Standard-Verhalten] eine nach der anderen ab dem
;$                           Zeitpunkt <debut> aneinandergereiht
;$   nil         t        => Die Noten der Sequenz werden im "legato"-Modus
;$                           [vgl. die Csound-Dokumentation] eine nach der
;$                           anderen ab dem Zeitpunkt <debut> aneinandergereiht.
;$                           Eine zustzliche Note wird zum Anhalten des
;$                           Instruments <instr> am Ende der Sequenz eingefgt
;$   t           nil      => Alle Noten fangen zusammen am Zeitpunkt <debut> an
;$   t           t        => Fehler. Diese Kombination macht keinen Sinn in
;$                           diesem Zusammenhang
;+   nil         nil      => the notes of the sequence are chained one after the
;+                           other from time <debut>, in  "normal" mode [default
;+                           behavior]
;+   nil         t        => the notes of the sequence are chained one after the
;+                           other from time <debut>, in  "legato" mode (cf.
;+                           Csound documentation). An extra note is added at
;+                           end of the sequence to stop the performance of
;+                           instrument <instr>
;+   t           nil      => all notes are attacked together at time <debut>
;+   t           t        => error. This combination does not make sense in the
;+                           context
;===============================================================================
;;   Les notes Csound sont crites dans le fichier spcifi au moyen du mot-clef
;; :fichier ["notes-faites.sco" par dfaut].
;$   Die Csound-Noten werden in die Datei geschrieben, die durch das
;$ Schlsselwort :fichier bestimmt wird [standardmig "notes-faites.sco"].
;+   The Csound notes are written in the file specified by the :fichier key-word
;+ ["notes-faites.sco" by default].
;-   Exemple : / Beispiel: / Example:
;-    (faire-notes 3 0 (faire-durees '(1+ (random 10))    100)
;-                     (faire-liste  '(* -1 (random 10.0)) 33)
;-                     :fichier "ped.14.xmpl.sco" :legato t )
;; Les fonctions faire-durees et faire-liste ont t dfinies dans le fichier
;; ped.12.
;$ Die Funktionen faire-durees und faire-liste wurden in Datei ped.12 definiert.
;+ The faire-durees and faire-liste functions have been defined in file ped.12.

(defun faire-notes (instr debut list-durees &rest listdelist-pX
                                            &aux legato synchro (compteur 0)
                                                 (fichier "notes-faites.sco"))
"
 faire-notes <instr> <debut> <list-durees>
    [<liste-p4> ... <liste-p<n>>] [:legato] [:synchro] [:fichier]
"
  ;Les mots-clef :synchro, :legato et :fichier sont traits ici explicitement,
  ;car l'usage normal du mot-clef &key dans la lambda-liste annulerait
  ;l'avantage du recours au mot-clef &rest, qui est de permettre de donner un
  ;nombre quelconque de listes de valeurs en arguments d'appel. On peut ainsi
  ;spcifier un nombre quelconque de champs de paramtres de notes : p4 p5 ...
  ;p<n>
  (setq synchro (cadr (member :synchro listdelist-pX)))
  (setq  legato (cadr (member  :legato listdelist-pX)))
  (and synchro legato
    (error "faire-notes : synchro ET legato impossibles ensemble."))
  (let ((oui (member :fichier listdelist-pX)))
    (when oui (setq fichier (cadr oui))))
  ;Utilisation de (setq listdelist-pX (remove ...))... cf. faire-svp ci-dessus.
  (setq listdelist-pX (remove :synchro listdelist-pX))
  (setq listdelist-pX (remove  :legato listdelist-pX))
  (setq listdelist-pX (remove :fichier listdelist-pX))
  (setq listdelist-pX (remove        t listdelist-pX))
  (setq listdelist-pX (remove      nil listdelist-pX))
  (setq listdelist-pX (remove  fichier listdelist-pX))
  (setq synchro (if synchro "." "+"))
  (setq  legato (if  legato  -1  1 ))
  ;
  (with-open-file (flux fichier :direction         :output
                                :if-exists         :rename-and-delete
                                :if-does-not-exist :create)
    (format flux "i~3D~12,5F~12,5F" instr debut (* legato (car list-durees)))
    (dolist (lali listdelist-pX)
      (format flux "~12,5F" (car lali)))
    (format flux ";~3D~%" compteur)
    (dotimes (num (1- (list-length list-durees)))
      (setq compteur (1+ num))
      (format flux "i~3D~10T~A~16T~12,5F"
                   instr synchro (nth compteur list-durees))
      (dolist (lali listdelist-pX)
        (format flux "~12,5F" (nth compteur lali)))
      (format flux ";~3D~%" compteur))
    (when (= legato -1)
      (format flux "i~3D~10T~A~16T~12,5F" (* legato instr) synchro 0)
      (dolist (lali listdelist-pX)
        (format flux "~12,5F" 1 lali))
      (format flux ";fin_legato~%")))
  (format t "C'est dans le fichier ~A." fichier) 'ok)

;; Utile pour Csound
;$ Ntzlich fr Csound
;+ Useful for Csound

;;   Cette fonction crit un nonc f pour Csound. Elle fait un simple cho de
;; tous ses arguments : elle n'a donc pas d'autre tche que de transcrire des
;; champs p1 p2 ... p<n> dans un fichier, prcds de la lettre "f".
;;   L'nonce f pour Csound est dirig vers le fichier spcifi au moyen du
;; mot-clef :fichier ["notes-faites.sco" par dfaut].
;;   Un autre mot-clef :ajouter [nil par dfaut] peut prendre la valeur t afin
;; de permettre d'ajouter un nonc f  un fichier dj existant.
;$   Diese Funktion schreibt eine Csound f-Anweisung. Sie erzeugt ein simples
;$ Echo aller ihrer Argumente: sie macht nichts anderes als die p1 p2 ...
;$ p<n>-Felder in eine Datei zu schreiben, angefangen mit einem "f".
;$   Die Csound f-Anweisung wird in die Datei geschrieben, welche durch das
;$ Schlsselwort :fichier bestimmt wird [standardmig "notes-faites.sco"].
;$   Ein weiteres Schlsselwort :ajouter [standardmig nil] erlaubt, wenn es
;$ auf t gesetzt ist, eine f-Anweisung einer vorhandenen Datei hinzuzufgen.
;+   This function writes a Csound f statement. It does a simple echo of all its
;+ arguments: it performs no other task than transcribing p1 p2 ... p<n> fields
;+ into a file, preceded by a letter "f".
;+   The Csound f statement is directed into the file specified by the :fichier
;+ key-word ["notes-faites.sco" by default].
;+   Another key-word :ajouter [nil by default] allows to add an f statement in
;+ a pre-existing file when it takes the value t.
;-   Exemples : / Beispiele: / Examples
;-    (fCsEcho 10 0 1024 9 1 1 0 2 .5 0 :fichier "toto.sco" :ajouter t)
;;        => le fichier toto.sco est cr s'il n'existe pas. S'il existe, tout
;;           son contenu antrieur est conserv. La ligne suivante y est
;;           insre :
;$        => die Datei toto.sco wird erstellt falls es sie noch nicht gibt.
;$           Falls sie existiert, wird der gesamte vorhergehende Inhalt
;$           erhalten. Die folgende Zeile wird eingefgt:
;+        => the toto.sco file is created if it does not exist. If it exists,
;+           all its preceding content is preserved. The following line is
;+           inserted:
;-               f10 0 1024 9 1 1 0 2 0.5 0
;-    (fCsEcho 4 0 2049 7 0 512 1 1024 1 512 0)
;;        => le fichier notes-faites.sco est cr s'il n'existe pas. S'il
;;           existe, tout son contenu antrieur est dtruit. La ligne suivante
;;           y est insre :
;$        => die Datei notes-faites.sco wird erstellt falls es sie noch nicht
;$           gibt. Falls es sie gibt, wird der gesamte vorhergehende Inhalt
;$           zerstrt. Die folgende Zeile wird eingefgt:
;+        => the notes-faites.sco file is created if it does not exist. If it
;+           exists, all its preceding content is destroyed. The following line
;+           is inserted:
;-               f4 0 2049 7 0 512 1 1024 1 512 0

(defun fCsEcho (&rest champs
                &aux (fichier "notes-faites.sco")
                     (ajouter nil))
"
 fCsEcho [p1 ... p<n>] [:fichier] [:ajouter]
"
  ;Les mots-clefs :fichier et :ajouter sont traits ici explicitement, car
  ;l'usage normal du mot-clef &key dans la lambda-liste annulerait l'avantage
  ;du recours au mot-clef &rest, qui est de permettre de donner un nombre
  ;quelconque de valeurs en arguments d'appel. On peut ainsi spcifier un nombre
  ;quelconque de champs p1 ... p<n> pour l'nonc f
  (let ((oui (member :fichier champs))) (when oui (setq fichier (cadr oui))))
  (let ((oui (member :ajouter champs))) (when oui (setq ajouter (cadr oui))))
  ;Utilisation de (setq champs (remove ...))... cf. faire-svp ci-dessus.
  (setq champs (remove :fichier champs))
  (setq champs (remove :ajouter champs))
  (setq champs (remove  fichier champs))
  (setq champs (remove  ajouter champs))
  (if ajouter (setq ajouter :append) (setq ajouter :rename-and-delete))
  ;
  (with-open-file (flux fichier :direction         :output
                                :if-exists         ajouter
                                :if-does-not-exist :create)
    (format flux "~&f~D" (pop champs))
    (labels ((interne (chps)
               (if (endp chps) (format flux "~%")
               (progn (format flux " ~D" (car chps))
                      (interne (cdr chps))))))
      (interne champs))) 'ok)

;; Utile pour Csound
;$ Ntzlich fr Csound
;+ Useful for Csound

;;   Cette fonction crit un nonc f pour les GEN 05 ou 07 de Csound. Elle
;; permet d'indiquer les coordonnes horizontales (abscisses) relles des points
;; d'inflexion de la fonction -- et non les nombres de mots physiques (!)
;; sparant les valeurs successives de la fonction. De plus, les coordonnes
;; horizontales peuvent tre spcifies dans une unit quelconque -- entre 0 et
;; 1, par exemple, ou entre 0 et 100, etc. -- sans tenir compte de la longueur
;; physique de la fonction dclare  Csound (!!).
;;   On s'affranchit ainsi de deux contraintes sadiques (cf. ! et !!) de Csound
;; dans ce contexte. Les mmes donnes peuvent donc en outre tre rutilises
;; sans changements pour des fonctions de longueurs diffrentes.
;;   Les 4 premiers arguments sont les p1  p4 normaux d'un nonc f, dont le p4
;; devrait tre 5, 7, -5 ou -7.
;;   Un cinquime argument doit tre une liste de points d'inflexion, de forme
;$   Diese Funktion schreibt eine Csound f-Anweisung fr GEN 05 oder 07. Sie
;$ erlaubt die eigentlichen horizontalen Koordinaten (Abszisse) der Funktions-
;$ Breakpoints zu spezifizieren -- nicht die Anzahl der physikalischen(!)
;$ Datenworte zwischen aufeinanderfolgenden Werten der Funktion. Weiterhin
;$ knnen die horizontalen Koordinaten in einer beliebigen Einheit angegeben
;$ werden -- z. B. zwischen 0 und 1 oder 0 und 100 etc. -- ohne die
;$ physikalische Lnge der in Csound deklarierten Funktion(!!) beachten zu
;$ mssen.
;$   Man ist deshalb in diesem Zusammenhang von diesen beiden sadistischen
;$ Csound- Einschrnkungen (Vgl. ! und !!) befreit. Dieselben Daten knnen
;$ deshalb ohne nderung fr Funktionen unterschiedlicher Lnge wiederverwertet
;$ werden.
;$   Die ersten 4 Argumente sind die normale f-Anweisung p1 bis p4, wovon p4 5,
;$ 7, -5 oder -7 sein sollte.
;$   Ein fnftes Argument muss eine Liste der Breakpoints sein, in der Form
;+   This function writes a Csound f statement for GEN 05 or 07. It allows to
;+ indicate the real horizontal coordinates (abscissae) of the function
;+ break-points--not the numbers of physical(!) words separating the successive
;+ values of the function. Moreover, the horizontal coordinates can be specified
;+ in any arbitrary unit--between 0 and 1, for instance, or between 0 and 100,
;+ etc.--without taking into account the physical length of the function
;+ declared to Csound(!!).
;+   One is thus liberated from the two sadistic Csound constraints (cf. ! and
;+ !!) in this context. The same data can thus be re-used without modifications
;+ for functions of different lengths.
;+   The 4 first arguments are the normal f statement p1 to p4, of which the p4
;+ should be 5, 7, -5 or -7.
;+   A fifth argument must be a list of break-points, in the form
;-    ((<y>1 . 0) (<y>2 . <x>2) ... (<y<n>> . <x<n>>))
;; o les <y> sont les valeurs successives de la fonction, et les <x> sont les
;; coordonnes horizontales des points d'inflexion.
;;   Pour se conformer aux exigences de Csound, les intervalles sparant deux
;; abscisses <x> successives, mis en rapport avec la dernire abscisse <x<n>>,
;; sont arrondis  un nombre proportionnel de mots de la longueur physique de la
;; fonction. Ce dernier calcul tient bien compte du fait que des fonctions
;; Csound sont parfois dclares avec une longueur augmente de 1. Un
;; avertissement est affich lorsque des erreurs d'arrondis font que le cumul
;; des intervalles (en mots) n'est pas exactement gal  la longueur physique de
;; la fonction.
;;   N.B. : aucune vrification n'est faite quant  la validit de l'nonc f
;; rsultant. Csound poserait son propre diagnostic le cas chant.
;;   L'nonc f pour Csound est dirig vers le fichier spcifi au moyen du
;; mot-clef :fichier ["notes-faites.sco" par dfaut].
;;   Un autre mot-clef :ajouter [nil par dfaut] peut prendre la valeur t afin
;; de permettre d'ajouter un nonc f  un fichier dj existant.
;$ wobei die <y>e die aufeinanderfolgenden Funktionswerte und die <x>e die
;$ horizontalen Koordinaten des Breakpoints sind.
;$   Um die Einschrnkungen von Csound einzuhalten, werden die Intervalle, die
;$ zwei aufeinanderfolgende <x>-Abszissen trennen, auf eine Datenwortanzahl
;$ proportional zur physikalischen Lnge der Funktion gerundet. Letztere
;$ Berechnung bezieht den Umstand mit ein, dass Csound-Funktionen manchmal mit
;$ einer um 1 erhhten Lnge deklariert werden. Eine Warnung wird ausgegeben
;$ wenn Rundungsfehler eine Gesamtsumme der Datenwrter erzeugen, die nicht
;$ exakt gleich der physikalischen Lnge der Funktion ist.
;$   N. B.: es wird kein Gltigkeitstest der erzeugten Csound f-Anweisung
;$ durchgefhrt. Csound wrde schlielich seine eigene Diagnose ausgeben.
;$   Die Csound f-Anweisung wird in die Datei geschrieben, welche durch das
;$ Schlsselwort :fichier bestimmt wird [standardmig "notes-faites.sco"].
;$   Ein weiteres Schlsselwort :ajouter [standardmig nil] erlaubt, wenn es
;$ auf t gesetzt ist, eine f-Anweisung einer vorhandenen Datei hinzuzufgen.
;+ where the <y> are the successive values of the function, and the <x> are the
;+ horizontal coordinates of the break-points.
;+   To meet Csound constraints, the intervals separating two successive <x>
;+ abscissae, related to the last <x<n>> abscissa, are rounded to a proportional
;+ number of words of the function's physical length. The latter computation
;+ takes into account the fact that Csound functions are sometimes declared with
;+ a length incremented of 1. A warning is printed when rounding errors produce
;+ a cumulated total of words that is not exactly equal to the functions's
;+ physical length.
;+   N.B.: no validity check of the resulting Csound f statement is performed.
;+ Csound would eventually output its own diagnostic.
;+   The Csound f statement is directed into the file specified by the :fichier
;+ key-word ["notes-faites.sco" by default].
;+   Another key-word :ajouter [nil by default] allows to add an f statement in
;+ a pre-existing file when it takes the value t.
;-   Exemples : / Beispiele: / Examples:
;-    (fCs5/7 3 0 4097 7 '((0 . 0) (1 . 0.25) (1 . 0.75) (0 . 1))
;-     :fichier "toto.sco" :ajouter t)
;;        => le fichier toto.sco est cr s'il n'existe pas. S'il existe, tout
;;           son contenu antrieur est conserv. Les lignes suivantes y sont
;;           insres :
;$        => die Datei toto.sco wird erstellt falls es sie noch nicht gibt.
;$           Falls es sie gibt, wird der gesamte vorhergehende Inhalt erhalten.
;$           Die folgenden Zeilen werden eingefgt:
;+        => the toto.sco file is created if it does not exist. If it exists,
;+           all its preceding content is preserved. The following lines are
;+           inserted:
;-               f3 0 4097 7        0        1024
;-                                  1        2048
;-                                  1        1024
;-                                  0
;-    (fCs5/7 8 0 1024 -5 '((440 . 0) (880 . 25) (220 . 75) (440 . 100)))
;;        => le fichier notes-faites.sco est cr s'il n'existe pas. S'il
;;           existe, tout son contenu antrieur est dtruit. Les lignes
;;           suivantes y sont insres :
;$        => die Datei notes-faites.sco wird erstellt falls es sie noch nicht
;$           gibt. Falls es sie gibt, wird der gesamte vorhergehende Inhalt
;$           zerstrt. Die folgenden Zeilen werden eingefgt:
;+        => the notes-faites.sco file is created if it does not exist. If it
;+           exists, all its preceding content is destroyed. The following lines
;+           are inserted:
;-               f8 0 1024 -5    440        256
;-                               880        512
;-                               220        256    
;-                               440

(defun fCs5/7 (nof dat lng gen liste-pts
                &key (fichier "notes-faites.sco")
                     (ajouter nil)
                &aux xmax lgrl seg (cumul 0))
"
 fCs5/7 p1 p2 p3 p4 ((<y>1 . 0) ... (<y<n>> . <x<n>>)) [:fichier] [:ajouter]
"

  (if ajouter (setq ajouter :append) (setq ajouter :rename-and-delete))
  (setq xmax (cdar (last liste-pts)))
  (setq lgrl (if (let ((ex (log lng 2))) (= ex (truncate ex))) lng (1- lng)))
  (with-open-file (flux fichier :direction         :output
                                :if-exists         ajouter
                                :if-does-not-exist :create)
    (format flux "~&f~D ~D ~D ~D" nof dat lng gen)
    (labels ((interne (lpts)
               (if (endp (cdr lpts)) (format flux "~20T~D~%" (caar lpts))
               (progn (format flux "~20T~D~30T~D~%"
                              (caar lpts)
                              (setq seg (round (/ (* lgrl (- (cdadr lpts)
                                                             (cdar lpts)))
                                                  xmax))))
                      (incf cumul seg)
                      (interne (cdr lpts))))))
      (interne liste-pts)))
  (if (= cumul lgrl) 'ok
  (format nil "Attention f~D : cumul des intervls (~D) diff. de ~
               longr de fonct. (~D)." nof cumul lgrl)))

;; Utile pour Csound
;$ Ntzlich fr Csound
;+ Useful for Csound

;;   La fonction faire-notes ci-dessus tant limite  l'abrviation "+" pour la
;; ralisation de squences de notes Csound, on a besoin d'une autre fonction
;; dans des cas plus complexes o la liste des temps de dpart doit tre gre
;; explicitement : par exemple pour le chevauchement de notes plus ou moins
;; simultanes, etc. D'o la fonction iCs ci-dessous,  la fois plus simple et
;; plus gnrale que faire-notes.
;;   Elle reoit un numro d'instrument, suivi d'un nombre quelconque de listes
;; spcifiant, respectivement, l'ensemble des valeurs successives des p2, puis
;; celles des p3, des p4, etc.
;;   Les listes de valeurs pour les champs p<n> doivent tre de mme longueur --
;; ce qui n'est pas vrifi par la fonction : en tous cas, c'est le nombre de
;; valeurs inclues dans la premire liste, celle des p2, qui fixe le nombre
;; <nbrnotes> de notes  raliser. Ce procd ne risque de poser problme que
;; dans le cas o certaines listes seraient plus courtes que la premire.
;;   La fonction imprime <nbrnotes> lignes d'noncs i de <nbrps>+1 champs, en
;; utilisant le <nonot>-ime terme de chacune des <nop>-imes listes pour crire
;; une <nonot>-ime note.
;;   Les noncs i pour Csound sont dirigs vers le fichier spcifi au moyen du
;; mot-clef :fichier ["notes-faites.sco" par dfaut].
;;   Un autre mot-clef :ajouter [nil par dfaut] peut prendre la valeur t afin
;; de permettre d'ajouter des noncs i  un fichier dj existant.
;$   Die obige Funktion faire-notes war beschrnkt auf die Abkrzung "+" zur
;$ Sequenzierung von Csound-Noten. Eine weitere Funktion ist in komplizierteren
;$ Fllen ntig, in welchen die Liste der Startzeiten explizit verwaltet werden
;$ muss, zum Beispiel bei sich berlappenden und/oder mehr oder weniger
;$ gleichzeitigen Noten, etc. Daher die Funktion iCs unten, die einfacher und
;$ gleichzeitig allgemeingltiger als faire-notes ist.
;$   Sie erwartet eine Instrumentennummer, gefolgt von einer beliebigen Anzahl
;$ Listen, die jeweils alle aufeinanderfolgenden Werte des p2-Felds, dann von
;$ p3, p4, etc. bestimmen.
;$   Die Liste von Werten fr die p<n>-Felder mssen dieselbe Lnge haben --
;$ was in der Funktion nicht berprft wird: auf jeden Fall bestimmt die Anzahl
;$ von Werten in der ersten Liste, der des p2-Felds, die Notenanzahl <nbrnotes>,
;$ die erzeugt werden soll. Dieses Verfahren verursacht nur Probleme wenn
;$ gewisse Listen krzer als die erste sind.
;$   Die Funktion gibt <nbrnotes> Zeilen mit i-Anweisungen mit jeweils <nbrps>+1
;$ Feldern aus und benutzt jeweils das <nonot>-te Glied von jeder der <nop>-ten
;$ Listen um die <nonot>-te Note zu schreiben.
;$   Die Csound i-Anweisung wird in die Datei geschrieben, welche durch das
;$ Schlsselwort :fichier bestimmt wird [standardmig "notes-faites.sco"].
;$   Ein weiteres Schlsselwort :ajouter [standardmig nil] erlaubt, wenn es
;$ auf t gesetzt ist, eine f-Anweisung einer vorhandenen Datei hinzuzufgen.
;+   The above faire-notes function being limited to the "+" abbreviation for
;+ the sequencing of Csound notes, another function is needed in more complex
;+ cases where the list of start times must be explicitly managed, for instance
;+ in overlapping and/or more or less simultaneous notes, etc. Hence the iCs
;+ function below, at once simpler and more general than faire-notes.
;+   It receives an instrument number, followed by any number of lists
;+ respectively specifying all successive values of the p2 field, then of p3, of
;+ p4, etc.
;+   The values lists for the p<n> fields must be of equal length--which is not
;+ checked by the function: in all cases, it is the number of values included in
;+ the first list, that of the p2 field, which determines the <nbrnotes> number
;+ of notes to be realized. This procedure could cause problems only if certain
;+ lists would be shorter than the first one.
;+   The function prints <nbrnotes> lines of i statements of <nbrps>+1 fields
;+ each, using the <nonot>-th term of each of the <nop>-th lists to write the
;+ <nonot>-th note.
;+   The Csound i statements are directed into the file specified by the
;+ :fichier key-word ["notes-faites.sco" by default].
;+   Another key-word :ajouter [nil by default] allows to add i statements in a
;+ pre-existing file when it takes the value t.
;-   Exemples : / Beispiele: / Examples:
;-    (iCs 5 '(0 1 2 3) '(4 5 6 7) '(8 9 10 11) '(12 13 14 15))
;;        => un nouveau fichier notes-faites.sco est cr. Il contient les
;;           quatre notes suivantes :
;$        => eine neue Datei notes-faites.sco wird erstellt. Sie enthlt die
;$           folgenden vier Noten:
;+        => a new notes-faites.sco file is created. It contains the following
;+           four notes:
;-               i5      0.00000      4.00000      8.00000     12.00000;  0
;-               i5      1.00000      5.00000      9.00000     13.00000;  1
;-               i5      2.00000      6.00000     10.00000     14.00000;  2
;-               i5      3.00000      7.00000     11.00000     15.00000;  3
;-    (iCs 5 '(0 1) '(4 5) '(8 9) '(12 13) :fichier "to.sco" :ajouter t)
;;        => s'il n'existe pas, un nouveau fichier to.sco est cr. S'il existe
;;           dj son contenu antrieur est conserv. Les deux lignes suivantes
;;           y sont insres :
;$        => falls es sie noch nicht gibt, wird eine neue to.sco-Datei erstellt.
;$           Falls es sie schon gibt, wird der vorhergehende Inhalt erhalten.
;$           Die folgenden Zeilen werden eingefgt:
;+        => if it does not exist, a new to.sco file is created. If it already
;+           exists, its preceding content is preserved. The two following lines
;+           are inserted:
;-               i5      0.00000      4.00000      8.00000     12.00000;  0
;-               i5      1.00000      5.00000      9.00000     13.00000;  1

(defun iCs (instr &rest list-pX
                  &aux (fichier "notes-faites.sco")
                       (ajouter nil))
"
 iCs <instr> [<liste-de-p2> ... <liste-de-p<n>>] [:fichier] [:ajouter]
"
  ;Les mots-clef :fichier et :ajouter sont traits ici explicitement, etc...
  ;-- cf. fCsEcho ci-dessus
  (let ((oui (member :fichier list-pX))) (when oui (setq fichier (cadr oui))))
  (let ((oui (member :ajouter list-pX))) (when oui (setq ajouter (cadr oui))))
  ;Utilisation de (setq list-pX (remove ...))... cf. faire-svp ci-dessus.
  (setq list-pX (remove :fichier list-pX))
  (setq list-pX (remove :ajouter list-pX))
  (setq list-pX (remove  fichier list-pX))
  (setq list-pX (remove  ajouter list-pX))
  (if ajouter (setq ajouter :append) (setq ajouter :rename-and-delete))
  ;
    (with-open-file (flux fichier :direction         :output
                                  :if-exists         ajouter
                                  :if-does-not-exist :create)
      (let ((nbrnotes (list-length (nth 0 list-pX)))
            (nbrps    (list-length list-pX)))
        (dotimes (nonot nbrnotes)
          (format flux "~&i~D" instr)
          (dotimes (nop nbrps)
            (format flux " ~12,5F" (nth nonot (nth nop list-pX))))
          (format flux ";~3D~%" nonot)))) 'ok)
    
;; Utile pour Csound
;$ Ntzlich fr Csound
;+ Useful for Csound

;;   Cette fonction ajoute un nonc e pour Csound dans un fichier qui doit
;; dj exister. Le fichier est spcifi au moyen du mot-clef :fichier
;; ["notes-faites.sco" par dfaut].
;$   Diese Funktion fgt eine Csound e-Anweisung einer Datei hinzu, die schon
;$ existieren muss. Die Datei wird durch das Schlsselwort :fichier bestimmt
;$ [standardmig "notes-faites.sco"].
;+   This function adds a Csound e statement to a file which must already exist.
;+ The file is specified by the :fichier key-word ["notes-faites.sco" by
;+ default].
;-   Exemples : / Beispiele: / Examples:
;-    (eCs :fichier "toto.sco")
;;        => si le fichier toto.sco existe, un nonc e y est ajout
;$        => falls es die Datei toto.sco gibt, wird eine e-Anweisung hinzugefgt
;+        => if the toto.sco file exists, an e statement is added
;-    (eCs)
;;        => si le fichier notes-faites.sco existe, un nonc e y est ajout
;$        => falls es die Datei notes-faites.sco gibt, wird eine e-Anweisung
;$           hinzugefgt
;+        => if the notes-faites.sco file exists, an e statement is added

(defun eCs (&key (fichier "notes-faites.sco"))
"
 eCs [:fichier]
"
  (with-open-file (flux fichier :direction         :output
                                :if-exists         :append
                                :if-does-not-exist :error)
    (format flux "~&e~%")) 'ok)

;;   Exemple type de prparation d'un fichier notes-faites.sco pour Csound. Une
;; bibliothque stochastique serait pralablement charge (fonctions corect et
;; inrect), ainsi que la fonction integ (cf. fichier ped.12) utilise pour
;; cumuler des intervalles de temps afin de former une liste des dates de dpart
;; des notes successives :
;$   Typisches Beispiel fr die Vorbereitung einer Csound notes-faites.sco
;$ Datei. Eine stochastische Bibliothek sollte vorher geladen worden sein (die
;$ Funktionen corect und inrect), und auch die integ-Funktion (vgl. Datei
;$ ped.12) zur Berechnung der Summe der Zeitintervalle, um eine Liste der
;$ Startzeiten fr aufeinanderfolgende Noten zu erzeugen:
;+   Typical example for the preparation of a Csound notes-faites.sco file. A
;+ stochastic library would have been previously loaded (corect and inrect
;+ functions), as well as the integ function (cf. ped.12 file) used to cumulate
;+ time intervals in order to form a list of start times for the successive
;+ notes:
;-    (setq mesP2 (integ (faire-liste '(corect 0.5 3.3) 20)))
;-    (setq mesP3 (faire-liste '(corect 0.5 2.0) 21))
;-    (setq mesP4 (faire-liste -10.0 21))
;-    (setq mesP5 (faire-liste '(* 220.0 (inrect 1 30)) 21))
;-    (fCsEcho ...)
;-    (fCs5/7 ... :ajouter t)
;-    (fCsEcho ... :ajouter t)
;-    (iCs 3 mesP2 mesP3 mesP4 mesP5 :ajouter t)
;-    (eCs)

;;; Lancement d'une commande shell UNIX : primitive excl:run-shell-command
;$$ Ausfhren eines UNIX-Shell-Befehls: Primitive excl:run-shell-command
;++ Launch of a UNIX shell command: excl:run-shell-command primitive

;-    sh <commande> {<argument(s)>}*
;;        => excution shell de <commande> {<argument(s)>}* et retour  LISP
;;   Cette macro-fonction supporte mal les caractres UNIX spciaux comme
;; `'"<>() etc. :  cf. fonction utilitaire sh-list-to-string ci-dessous. Les
;; macro-fonctions seront dtailles au Chapitre 17.
;$        => Fhre das <commande> {<argument(s)>}* in der Shell aus und kehre
;$           zu LISP zurck
;$   Diese Makro-Funktion kann nicht gut mit manchen UNIX-Sonderzeichen wie
;$ `'"<>() etc. umgehen: vgl. das Utility sh-list-to-string unten.
;$ Makrofunktionen werden in Kapitel 17 genau beschrieben.
;+        => shell execution of <commande> {<argument(s)>}* and return to LISP
;+   This macro-function does not handle well UNIX special characters such as
;+ `'"<>() etc.: cf. the sh-list-to-string utility below. Macro-functions will
;+ be detailed in Chapter 17.

;(defmacro sh (&rest commande)
;  `(excl:run-shell-command ,(sh-list-to-string commande)))

;;   Utilitaire pour la macro-fonction sh ci-dessus. Transforme une liste <li>
;; d'atomes en une chane de caractres compose des lments de <li> spars
;; par des espaces.
;;   N.B. : ne supporte pas d'lments ne pouvant tre interprts comme des
;; symboles LISP. En particulier, interprte mal les caractres UNIX spciaux
;; `'"<>() etc., surtout isols. Refuse donc aussi des lments numriques dans
;; <li>.
;$   Hilfsfunktion fr die Makrofunktion sh oben. Wandelt eine Liste von Atomen
;$ in eine Zeichenkette um, die aus den mit Leerzeichen unterteilten Gliedern
;$ der <li> besteht.
;$   N. B.: kann nicht mit Gliedern umgehen, welche nicht als LISP-Symbole
;$ interpretiert werden knnen. Im einzelnen interpretiert es die UNIX-
;$ Sonderzeichen `'"<>() etc. schlecht, vor allem wenn sie alleine stehen.
;$ Folglich lehnt es auch numerische Glieder in <li> ab.
;+   Utility for the sh macro-function above. Transforms a <li> list of atoms
;+ into a character string consisting in the elements of <li> separated by blank
;+ spaces.
;+   N.B.: can not handle elements which can not be interpreted as LISP symbols.
;+ In particular, interprets poorly the UNIX special characters `'"<>() etc.,
;+ especially isolated. Thus also rejects numerical elements in <li>.
;; Exemples avec caractres d'chappement prservant les lettres minuscules :
;$ Beispiele mit Escape-Schriftzeichen um die Kleinbuchstaben zu schtzen:
;+ Examples with escape characters preserving lower case characters:
;-    (sh-list-to-string '(|ls| |-l| |Ped.10.*|)) => "ls -l Ped.10.* "
;-    (sh-list-to-string '(\l\s -\l P\e\d.10.*)) => "ls -l Ped.10.* "
;! NOTE : La prservation des minuscules ne peut-tre organise  l'intrieur de
;! la fonction, car la liste <li> est dj en majuscules en y entrant ! Sans
;! chappements | ou \ le rsultat est tout en majuscules. Ex. :
;!    (sh-list-to-string '(ls -l Ped.10.*)) => "LS -L PED.10.* "
;! ce que UNIX ne reconnat pas...

(defun sh-list-to-string (li)
  (let ((elm (car li))) (cond
    ((endp li) "")
    ((symbolp elm) (concatenate 'string
                                (string elm)
                                " "
                                (sh-list-to-string (cdr li))))
    (t (cerror "continuer en eliminant cet argument"
               "sh-list-to-string : ~
               ~A : ne peut etre traite ici comme un argument de commande."
               elm)
       (sh-list-to-string (cdr li))))))

;;; Notion de path -- exemples de mise en oeuvre
;$$ Begriff des Pfads -- Anwendungsbeispiele
;++ Notion of path--examples of use

;;   Un path peut tre dfini de diverses faons. Par exemple textuellement,
;; a. en Macintosh Common LISP (MCL) :
;$   Ein Pfad kann auf unterschiedliche Weise definiert werden. Textlich zum
;$ Beispiel
;$ a. in Macintosh Common LISP (MCL):
;+   A path can be defined in different ways. Textually, for instance,
;+ a. in Macintosh Common LISP (MCL):
      (defvar PathDonnM (pathname "DD:1DLO:cl.d:ped.d:essais.d:*"))
      (format t "~%;PathDonnM = ~S" PathDonnM)                        ;pour voir
;; b. en LispWorks sous Mac OS X :
;$ b. in LispWorks unter Mac OS X:
;+ b. in LispWorks under Mac OS X:
      (defvar PathDonnW (pathname "/Volumes/DD/1DLO/cl.d/ped.d/essais.d/*"))
      (format t "~%;PathDonnW = ~S" PathDonnW)                        ;pour voir
;; c. sous UNIX :
;$ c. unter UNIX:
;+ c. under UNIX:
      (defvar PathDonnU (pathname "/home/dlo/cl.d/ped.d/essais.d/*"))
      (format t "~%;PathDonnU = ~S" PathDonnU)                        ;pour voir

;;   En Macintosh Common LISP (MCL) et LispWorks sous Mac OS X, on peut aussi,
;; par exemple, crer une variable contenant le path mme d'un fichier, lors de
;; son propre chargement :
;$   In Macintosh Common LISP (MCL) und LispWorks unter Mac OS X kann man z. B.
;$ auch eine Variable erstellen, die den eigentlichen Pfad einer Datei enthlt
;$ wenn letztere selbst geladen wird:
;+   In Macintosh Common LISP (MCL) and LispWorks under Mac OS X, one can also,
;+ for instance, create a variable containing the very path of a file when it is
;+ itself loaded:
      (defvar PathLocal (make-pathname :directory
                          (pathname-directory *load-truename*)))
      (format t "~%;PathLocal = ~S" PathLocal)                        ;pour voir
;; Cette technique assure le fonctionnement d'un environnement, quel que soit le
;; dossier dans lequel l'utilisateur le recopie et le mette en oeuvre.
;$ Diese Technik stellt das Funktionieren einer Umgebung sicher, egal, in
;$ welchem Ordner auch immer der Benutzer sie kopiert und ausgefhrt haben mag.
;+ This technique insures the functioning of an environment, in whatever folder
;+ the user may copy and launch it.

;;   Dans les directories ou dossiers ainsi dfinis en tant que paths, on peut
;; ensuite accder  un <fichier> particulier grce  la primitive
;$   Man kann in Verzeichnissen und Ordnern, die auf diese Weise als Pfade
;$ definiert wurden, mithilfe des folgenden Primitives auf eine bestimmte Datei
;$ <fichier> zugreifen:
;+   In directories or folders defined in this manner as paths, one can then
;+ access to a particular <fichier> file with the primitive
;-    merge-pathnames "<fichier>" <path-du-directory-ou-dossier>
;; Exemples -- avec impressions, pour voir :
;$ Beispiele -- mit Ausgabe zur Verdeutlichung:
;+ Examples--with printout to see:
      (let ((fichier))
        (setq fichier (merge-pathnames "maki.cl" PathDonnW))
        (format t "~%;~S existe-t-il ??? ~S !" fichier (probe-file fichier))
        (setq fichier (merge-pathnames "totor.3333.poi" PathDonnW))
        (format t "~%;~S existe-t-il ??? ~S !" fichier (probe-file fichier))
        (setq fichier (merge-pathnames "ped.13.lisp" PathLocal))
        (format t "~%;~S existe-t-il ??? ~S !" fichier (probe-file fichier))
        (setq fichier (merge-pathnames "raluk.666.poi" PathLocal))
        (format t "~%;~S existe-t-il ??? ~S !~%" fichier (probe-file fichier))
      )

;;   criture dans un fichier ainsi localis, en reprenant l'un des exemples
;; ci-dessus :
;$   Schreiben in eine solchermaen lokalisierte Datei. Fortfhrung eines der
;$ obigen Beispiele:
;+   Writing into a file thus localized, reusing one of the above examples:
;-    (let ((fic1 (merge-pathnames "pizza.sco" PathLocal)))
;-      (iCs 5 '(0 1 2 3) '(4 5 6 7) '(8 9 10 11) '(12 13 14 15)
;-             :fichier fic1 :ajouter t))
;; Le mieux serait d'intgrer la primitive merge-pathname  la dfinition du
;; fichier, deuxime lment de la liste de paramtrisation du with-open-file, 
;; l'intrieur de la fonction appele -- iCs dans cet exemple.
;$ Ideal wre die Integration des Primitives merge-pathname in die Definition
;$ der Datei, als zweites Glied der Liste der Parameter von with-open-file in
;$ der aufrufenden Funktion -- iCs in diesem Beispiel.
;+ The best would be to integrate the merge-pathname primitive in the definition
;+ of the file, second element of the parameterization list of the with-open-
;+ file inside the called function--iCs in this example.

;;   La fonction utilitaire suivante permet de concatner les lments
;; successifs d'un path menant  un fichier. Un premier argument doit tre un
;; path racine. Il faut au moins un autre argument chane de caractres. Le
;; rsultat dsigne un fichier sous forme de path. Exemples en LispWorks :
;$   Die folgende Hilfsfunktion erlaubt die Verkettung aufeinanderfolgender
;$ Elemente eines Pfads, der zu einer Datei fhrt. Das erste Argument muss der
;$ root-Pfad sein. Mindestens eine weitere Zeichenkette als Argument ist ntig.
;$ Das Ergebnis ist ein Pfad, der eine Datei lokalisiert.
;$ Beispiele in LispWorks:
;+   The following utility allows to concatenate successive elements of a path
;+ leading to a file. A first argument must be a root path. At least one more
;+ character string argument is needed. The result is a path localizing a file.
;+ LispWorks examples:
;-    (cat-pathnames
;-      (make-pathname :directory "Volume1") "DossierA" "DossierB" "FichierN")
;-        => #P"/Volume1/DossierA/DossierB/FichierN"
;-    (defparameter DirLocArsCombi #P"/Volumes/DD/1DLO/cl.d/clce.d/ArsCombi.d")
;-    (cat-pathnames DirLocArsCombi "Je" "Te" "Me" "Le" "Fais")
;-        => #P"/Volumes/DD/1DLO/cl.d/clce.d/ArsCombi.d/Je/Te/Me/Le/Fais"

;dfinition MCL
;(defun cat-pathnames (depart &rest frags)
;  (labels ((interne (frgs ldr)
;             (if (endp (cdr frgs))
;               (merge-pathnames (car frgs) ldr)
;               (interne (cdr frgs)
;                      (merge-pathnames 
;                       (make-pathname
;                        :directory
;                        (namestring (merge-pathnames (car frgs) ldr)))
;                       "*")))))
;    (interne frags depart)))

;dfinition LispWorks
(defun cat-pathnames (depart &rest frags)
  (when (pathnamep depart) (setq depart (namestring depart)))
  (when (char= (elt depart (1- (length depart))) #\/)
    (setq depart (delete #\/ depart :from-end t :count 1)))
  (labels ((interne (frgs ldr)
            (if (endp (cdr frgs))
              (concatenate 'string ldr "/" (car frgs))
            (interne (cdr frgs)
                     (concatenate 'string ldr "/" (car frgs))))))
  (make-pathname :name (interne frags depart))))

;;   Exemple d'utilisation d'un path et d'un module sous UNIX, pour la mise en
;; oeuvre d'une bibliothque de fonctions stochastiques (cf. directory
;; ~dlo/cl.d/stoch.d).
;;    a. L'environnement dfinit un path racine, par exemple avec la forme
;;       suivante insre dans un fichier d'initialisation (cf. ~/.clinit.cl) :
;$   Anwendungsbeispiel eines Pfades und eines Moduls unter UNIX, zur
;$ Implementierung einer stochastischen Funktionsbibliothek (Vgl. Ordner
;$ ~dlo/cl.d/stoch.d).
;$    a. Die Umgebung definiert den root-Pfad, z. B. mit dem folgenden Ausdruck,
;$       der in die Initialisierungsdatei (vgl. ~/.clinit.cl) eingefgt wird:
;+   Example usage of a path and a module under UNIX, for the implementation of
;+ a stochastic functions library (cf. ~dlo/cl.d/stoch.d directory).
;+    a. The environment defines a root path, for instance with the following
;+       form inserted in an initialization file (cf. ~/.clinit.cl):
;-           (defvar common-lisp::*path-stochastique*
;-             (pathname "/home/dlo/cl.d/stoch.d/*"))
;;    b. Le fichier canons_base.cl constitue la base essentielle du
;;       fonctionnement de l'ensemble de la bibliothque de fichiers. On lui
;;       attribue donc le statut de module. Ce fichier se termine par la forme
;;       suivante, qui inscrira "canons-stochastiques" dans la liste *modules*
;;       des modules prsents en mmoire lorsque le fichier aura t charg :
;$    b. Die Datei canons_base.cl stellt die essentielle Basis der gesamten
;$       Bibliothek dar. Sie besitzt deshalb den Status eines Moduls. Sie endet
;$       mit dem folgenden Ausdruck, welcher "canons-stochastiques" in die Liste
;$       *modules* der Module schreibt, die im Speicher vorhanden sind wenn die
;$       Datei geladen wird:
;+    b. The canons_base.cl file constitutes the essential basis of the whole
;+       library. It thus possesses the status of module. It ends with the
;+       following form, which will inscribe "canons-stochastiques" in the
;+       *modules* list of modules present in memory when the file will be
;+       loaded:
;-           (provide "canons-stochastiques")
;;       Dans le cas o l'utilisateur chargerait lui-mme directement ce
;;       fichier, l'environnement mmoriserait ainsi que ce chargement a dj
;;       t effectu
;;    c. D'autres fichiers, nomms canons_????.cl, font tous appel au contenu de
;;       canons_base.cl, donc au module "canons-stochastiques". Ils peuvent tre
;;       chargs individuellement, au besoin. Ils dbutent tous par les formes
;;       suivantes. La premire vrifie la dfinition du path racine. Dans la
;;       seconde, la primitive require vrifie d'abord si "canons-stochastiques"
;;       est dj prsent dans la liste *modules*. Si c'est la cas, elle ne fait
;;       rien. Si, au contraire, ce n'est pas le cas, require charge le fichier
;;       canons_base.cl. De ce fait, comme ci-dessus, la dernire forme de ce
;;       fichier inscrit donc "canons-stochastiques" dans la liste *modules* des
;;       modules prsents en mmoire, et le fichier ne sera plus recharg par la
;;       suite inutilement par d'autres lments de la bibliothque :
;$       Falls der Benutzer diese Datei direkt laden wrde, wrde sich die
;$       Umgebung somit merken, dass dieses Laden schon erledigt ist.
;$    c. Andere Dateien, benannt canons_????.cl, brauchen den Inhalt von
;$       canons_base.cl, welcher im Modul "canons-stochastiques" ist. Sie knnen
;$       individuell je nach Bedarf geladen werden. Sie beginnen alle mit den
;$       folgenden Ausdrcken. Der erste prft die Definition des root-Pfads.
;$       Im zweiten prft das Primitive require zunchst, ob
;$       "canons-stochastiques" schon in der *modules*-Liste vorhanden ist.
;$       Falls das der Fall ist, wird nichts gemacht. Falls es, im Gegenteil,
;$       nicht der Fall ist, dann ldt require canons_base.cl. In diesem Fall
;$       fgt, wie oben, der letzte Ausdruck dieser Datei "canons-stochastiques"
;$       in die Liste *modules* der Module, die im Speicher vorhanden sind, ein
;$       und die Datei wird spter nicht sinnlos von anderen Elementen der
;$       Bibliothek neu geladen:
;+       If the user would directly load this file himself, the environment
;+       would thus memorize that this loading has already been done
;+    c. Other files, named canons_????.cl, all rely on the content of
;+       canons_base.cl, that is on the "canons-stochastiques" module. They can
;+       be loaded individually, as needed. They all begin with the following
;+       forms. The first checks the definition of the root path. In the second,
;+       the require primitive first checks if "canons-stochastiques" is already
;+       present in the *modules* list. If it is, nothing is done. If, on the
;+       contrary, it is not, require then loads canons_base.cl. In this case,
;+       as above, the last form of this file thus inscribes "canons-
;+       stochastiques" in the *modules* list of modules present in memory, and
;+       the file will not be later uselessly reloaded by other elements of the
;+       library:
;-           (when (not (boundp 'common-lisp::*path-stochastique*))
;-             (defvar common-lisp::*path-stochastique* (pathname ""))
;-             (format *error-output*
;-              "Attention : le path common-lisp::*path-stochastique* est nul :~
;-              ~%~12Terreur(s) fichier(s) possible(s)."))
;-           (require "canons-stochastiques"
;-             (merge-pathnames "canons_base.cl"
;-              common-lisp::*path-stochastique*))
