;===============================================================================
;;; 16.4b --> STRUCTURES : second exemple
;$$ 16.4b --> STRUKTUREN: zweites Beispiel                            [d..H.B.]
;++ 16.4b --> STRUCTURES: Second Example
;===============================================================================

;;   Environnement d'analyse/resynthse (pour Juan-Carlos Pampin, 1993-1995).
;$   Analyse-/Resyntheseumgebung (fr Juan-Carlos Pampin, 1993-1995).
;+   Analysis/resynthesis environment (for Juan-Carlos Pampin, 1993-1995).

;;; Dfinition de la structure <son>
;$$ Definition der <son>-Struktur
;++ Definition of the <son> Structure

(defstruct (son
  (:constructor creer-son)
  (:conc-name <-))
  (fichier)
  (correct-ampl 0)
  (correct-freq 0)
  (erreur 0)
  (Npart 0)
  (Nfene 0)
  (temps ())                    ;On utilise des listes parce que ni le nombre de
  (amplitude ())                ;fentres, ni celui de partiels, ne peuvent tre
  (frequence ()))               ;connus  l'avance

;;; Fonctions
;$$ Funktionen
;++ Functions

;;   Lecture du fichier d'analyse, et transcription des donnes lues dans une
;; structure <son> :
;;    read-svp <nom-du-fichier-d'analyse> <nom-de-la-structure-son--crer>
;; Le fichier contient une succession de fentres d'analyse. Une fentre est
;; reprsente par une date suivie d'une numeration des valeurs de <N> partiels
;; (cf. fichier ped.16.4b.ex). Dans le cas prsent, afin d'interprter
;; correctement les donnes, on compte sur la prsence de nombres flottants
;; comme dates de fentres, et de nombres entiers comme numros de partiels.
;; Exemple de contenu de fichier d'analyse :
;$   Einlesen einer Analysedatei und bersetzung der gelesenen Daten in die
;$ <son>-Struktur:
;$    read-svp <Name-der-Analysedatei> <Name-der-zu-erstellenden-son-Struktur>
;$ Die Datei enthlt eine Abfolge von Analysefenstern. Ein Fenster besteht aus
;$ einem Zeitpunkt, gefolgt von einer Aufzhlung der Werte fr <N> Teiltne
;$ (vgl. Datei ped.16.4b.ex). Zur richtigen Interpration der Daten werden im
;$ aktuellen Fall Gleitkommazahlen als Fensterzeitpunkte und Ganzzahlen als
;$ Teiltonnummern erwartet. Beispiel fr den Inhalt einer Analysedatei:
;+   Reading of the analysis file, and transcription of the data read in a <son>
;+ structure:
;+    read-svp <name-of-the-analysis-file> <name-of-the-son-structure-to-create>
;+ The file contains a succession of analysis windows. A window consists in a
;+ date followed by an enumeration of the values of <N> partials (cf.
;+ ped.16.4b.ex file). In the present case, for a correct interpretation of the
;+ data, floating point numbers are expected as window dates, and integer
;+ numbers as partial numbers. Example of analysis file contents:
;-
;-    ...
;-    0.015                      <-date de fentre d'analyse (float)
;-    0   0.000000 0.000000      <-numro, amplitude, frquence : partiel n 0
;-    1   0.000000 1378.630310   (int.) (float)   (float)    :   "     n 1
;-    2   0.000000 2756.690230     "       "         "       :   "     n 2
;-    ...   ...     ...                                                ...
;-    <N> 0.000000 22048.040059    "       "         "       :   "     n <N>
;-    0.030                      <-date de fentre d'analyse (float)
;-    0   0.000000 0.000000      <-numro, amplitude, frquence : partiel n 0
;-    1   0.005432 1378.630310   (int.) (float)   (float)    :   "     n 1
;-    ...   ...     ...                                                ...
;-    <N> 0.006789 22048.040059    "       "         "       :   "     n <N>
;-    0.045                      <-date de fentre d'analyse (float)
;-    ...
;-
;;   Un premier traitement des donnes lues est possible ds ce stade. Certaines
;; erreurs sont dtectes et signales. D'autres anomalies provoqueront des
;; erreurs LISP, soit immdiates, soit lors de tentatives ultrieures
;; d'exploitation de la structure <son>. Par convention, les partiels de
;; frquence nulle sont supprims.
;$   Eine erste Verarbeitung der eingelesenen Daten ist in diesem Schritt schon
;$ mglich. Bestimmte Fehler werden gefunden und angezeigt. Andere Anomalien
;$ erzeugen LISP-Fehler, entweder gleich oder beim Versuch der sptere
;$ Verarbeitung der <son>-Struktur. Per Konvention werden Teiltne mit Frequenz
;$ 0 unterdrckt. 
;+   A first processing of the data read is already possible at this stage.
;+ Certain errors are detected and signaled. Other anomalies will produce LISP
;+ errors, either immediate or when ulterior processing of the <son> structure
;+ will be attempted. By convention, partials of null frequency are suppressed.

(defun read-svp (nom-fichier son)
  (set son (creer-son :fichier nom-fichier))
  (let ((strc (symbol-value son)))
    (with-open-file (strm (string nom-fichier) :direction :input)
      (do ((tp1 (read strm nil "fini") (read strm nil "fini")))
          ((equal "fini" tp1)
           (format *terminal-io* "~%Fin de fichier : ~A~%" nom-fichier)
           (when (< 0 (<-erreur strc))
             (format *terminal-io* "~D erreur~:P (peut-etre fatale~:P) ~
                                    detectee~:P...~%" (<-erreur strc)))
           (when (< 0 (+ (<-correct-ampl strc) (<-correct-freq strc)))
             (format *terminal-io* "~D correction~:P effectuee~:P :~%~4T~
                                    ~D amplitude~:P, ~
                                    ~D frequence~:P~%"
                     (+ (<-correct-ampl strc) (<-correct-freq strc))
                     (<-correct-ampl strc)
                     (<-correct-freq strc))))
        (cond
         ((integerp tp1) 
          (cond ((= tp1 0) (read-line strm )) 
                (t (when (> tp1 (<-Npart strc))
                     (setf (<-Npart strc) tp1))
                   (push (lire/corriger-ampl strm strc)
                         (<-amplitude strc))
                   (push (lire/corriger-freq strm strc)
                         (<-frequence strc)))))
         ((floatp tp1)
          (setf (<-Nfene strc) (1+ (<-Nfene strc)))
          (push tp1 (<-temps strc))
          (princ tp1 *terminal-io*) (princ "... " *terminal-io*))
         (t (setf (<-erreur strc) (1+ (<-erreur strc)))))))
    ;transcription listes -> vecteurs -- plus rapides  consulter ultrieurement
    (setf (<-frequence strc)
          (make-array (* (<-Npart strc) (<-Nfene strc))
                      :initial-contents (reverse (<-frequence strc))))
    (setf (<-amplitude strc)
          (make-array (* (<-Npart strc) (<-Nfene strc))
                      :initial-contents (reverse (<-amplitude strc))))
    (setf (<-temps strc)
          (make-array (<-Nfene strc)
                      :initial-contents (reverse (<-temps strc))))
    ;valeur finale rendue : structure <son>
    son))

;;   Les deux fonctions de lecture et correction suivantes se comportent
;; diffremment selon le champ concern (amplitude ou frquence). Si ncessaire,
;; elles permettent un premier traitement des valeurs brutes lues dans le
;; fichier d'analyse.
;$   Die folgenden beiden Lese-und Korrekturfunktionen verhalten sich je nach
;$ dem betreffenden Feld (Amplitude oder Frequenz) unterschiedlich. Wenn ntig
;$ erlauben sie eine erste Verarbeitung der Rohdaten, die aus der Analysedatei
;$ gelesen wurden. 
;+   The two following reading and correcting functions act differently
;+ according to the concerned field (amplitude or frequency). If necessary, they
;+ allow a first processing of the raw values read from the analysis file.

(defun lire/corriger-ampl (flux struct)
  (let ((tmp (read flux)))
    (cond ((not (floatp tmp))
           (setf (<-erreur struct) (1+ (<-erreur struct)))
           tmp)
          ((minusp tmp)
           (setf (<-correct-ampl struct) (1+ (<-correct-ampl struct)))
           (abs tmp))
          ((zerop tmp) 1.0e-5) ;pour interpolations exponentielles : 0 := -100dB
          (t tmp))))

(defun lire/corriger-freq (flux struct)
  (let ((tmp (read flux)))
    (cond ((or (not (floatp tmp)) (zerop tmp))
           (setf (<-erreur struct) (1+ (<-erreur struct)))
           tmp)
          ((minusp tmp)
           (setf (<-correct-freq struct) (1+ (<-correct-freq struct)))
           (abs tmp))
          (t tmp))))

;;   Accs aux donnes contenues dans une structure <son> :
;;    que <nom-de-la-structure-son> <champ> <n-fentre> [<n-partiel>]
;; On accepte des coordonnes non entires (n de fentre et/ou de partiel).
;$   Zugriff auf die Daten, die in einer <son>-Struktur enthalten sind:
;$    que <Name-der-son-Struktur> <Feld> <Fenster-Nr.> [<Teilton-Nr.>]
;$ Nicht-ganzzahlige Koordinaten werden akzeptiert (Fenster und/oder
;$ Teiltonnummer(n)).
;+   Access to the data contained in a <son> structure:
;+    que <name-of-the-son-structure> <champ> <window-#> [<partial-#>]
;+ Non-integer coordinates are accepted (window and/or partial number(s)).

(defun que (son champ fenetre &optional partiel)
  (let ((strc (symbol-value son)))
    (when (or (minusp fenetre) (> fenetre (1- (<-Nfene strc))))
      (error    "que : no. fenetre hors bornes : ~A" fenetre))
    (multiple-value-bind (Fe Fr) (floor fenetre)
      (cond ((equal 'temps champ)
              (if (zerop Fr) (aref (<-temps strc) Fe)
                  (intrp-lin (aref (<-temps strc) Fe)
                             (aref (<-temps strc) (1+ Fe))
                             Fr)))
            ((null partiel)
              (error "que : no. partiel non specifie"))
            ((or (minusp partiel) (> partiel (1- (<-Npart strc))))
              (error "que : no. partiel hors bornes : ~A" partiel))
            ((equal 'frequence champ)
              (multiple-value-bind (Pe Pr) (floor partiel)
                (chlin-val (<-frequence strc) Fe Fr Pe Pr (<-Npart strc))))
            ((equal 'amplitude champ)
              (multiple-value-bind (Pe Pr) (floor partiel)
               (chexp-val (<-amplitude strc) Fe Fr Pe Pr (<-Npart strc))))
            (t (error "que : champ inconnu : ~A" champ))))))

;;   Recherche d'une valeur dans un tableau bi-dimensionnel, acceptant des
;; coordonnes entires ou non. Dans le cas prsent, le tableau est contenu dans
;; le vecteur uni-dimensionnel <vect>.
;;    <fe> := partie entire de la coordonne dans la dimension 1 (n fentre)
;;    <fr> := partie dcimale de coordonne dans dim. 1 (fraction de fentre)
;;    <pe> := partie entire de la coordonne dans la dimension 2 (n partiel)
;;    <pr> := partie dcimale de coordonne dans dim. 2 (fraction de partiel)
;;    <d2> := taille de la dimension 2 -- pour exploiter <vect> en 2 dimensions
;; Au besoin, on interpole linairement entre des valeurs du tableau.
;$   Suche nach einem Wert in einem zweidimensionalen Array, akzeptiert
;$ ganzzahlige und nicht-ganzzahlige Koordinaten. Im aktuellen Fall wird das
;$ Array in einem eindimensionalen Vektor <vect> gespeichert.
;$    <fe> := ganzzahliger Teil der Koordinaten in Dimension 1 (Fenster-Nr.)
;$    <fr> := dezimaler Teil Teil der Koordinaten in Dimension 1 (Fensterteil)
;$    <pe> := ganzzahliger Teil der Koordinaten in Dimension 2 (Teilton-Nr.)
;$    <pr> := dezimaler Teil Teil der Koordinaten in Dimension 2 (Teiltonteil)
;$    <d2> := Gre der Dimension 2 -- um <vect> in 2 Dimensionen zu lesen
;$ Wenn ntig wird zwischen Arraywerten linear interpoliert.
;+   Search for a value in a bi-dimensional array, accepting integer or non-
;+ integer coordinates. In the present case, the array is stored in the uni-
;+ dimensional vector <vect>.
;+    <fe> := integer part of the coordinate in dimension 1 (window #)
;+    <fr> := decimal part of the coordinate in dimension 1 (window fraction)
;+    <pe> := integer part of the coordinate in dimension 2 (partial #)
;+    <pr> := decimal part of the coordinate in dimension 2 (partial fraction)
;+    <d2> := size of dimension 2--in order to read <vect> in 2 dimensions
;+ When needed, linear interpolations are performed between array values.

(defun chlin-val (vect fe fr pe pr d2)
  (cond ((= 0 fr pr)                           ;fentre et partiel entiers
          (aref2 vect fe pe d2))
        ((zerop fr)                            ;fentre entire, partiel non
          (intrp-lin (aref2 vect fe pe      d2)
                     (aref2 vect fe (1+ pe) d2)
                     pr))
        ((zerop pr)                            ;fentre non-entire, partiel oui
          (intrp-lin (aref2 vect fe      pe d2)
                     (aref2 vect (1+ fe) pe d2)
                     fr))
        (t                                     ;fentre et partiel non-entiers
          (intrp-lin (intrp-lin (aref2 vect fe pe      d2)
                                (aref2 vect fe (1+ pe) d2)
                                pr)    
                     (intrp-lin (aref2 vect (1+ fe) pe      d2)
                                (aref2 vect (1+ fe) (1+ pe) d2)
                                pr)    
                     fr))))

;;   Mme fonction que la prcdente, avec interpolation exponentielle.
;$   Gleich der vorhergehenden Funktion, mit exponentieller Interpolation.
;+   Same as the preceding function, with exponential interpolation.

(defun chexp-val (vect fe fr pe pr d2)
  (cond ((= 0 fr pr)                           ;fentre et partiel entiers
          (aref2 vect fe pe d2))
        ((zerop fr)                            ;fentre entire, partiel non
          (intrp-exp (aref2 vect fe pe      d2)
                     (aref2 vect fe (1+ pe) d2)
                     pr))
        ((zerop pr)                            ;fentre non-entire, partiel oui
          (intrp-exp (aref2 vect fe      pe d2)
                     (aref2 vect (1+ fe) pe d2)
                     fr))
        (t                                     ;fentre et partiel non-entiers
          (intrp-exp (intrp-exp (aref2 vect fe pe      d2)
                                (aref2 vect fe (1+ pe) d2)
                                pr)    
                     (intrp-exp (aref2 vect (1+ fe) pe      d2)
                                (aref2 vect (1+ fe) (1+ pe) d2)
                                pr)    
                     fr))))

;;; Utilitaires arithmtiques
;$$ Arithmetische Hilfsfunktionen
;++ Arithmetic Utilities

;;   Accs bi-dimensionnel [<c1>,<c2>] [fentre,partiel] dans un vecteur.
;;    aref2 <vect> <c1> <c2> <d2>, o <c1> := coordonne dans la dimension 1
;;                                    <c2> := coordonne dans la dimension 2
;;                                    <d2> := taille de la dimension 2
;$   Zweidimensionaler Zugriff [<c1>,<c2>] [Fenster,Teilton] in einem Vektor.
;$    aref2 <vect> <c1> <c2> <d2>, wo <c1> := Koordinate in Dimension 1
;$                                    <c2> := Koordinate in Dimension 2
;$                                    <d2> := Gre der Dimension 2
;+   Bi-dimensional access [<c1>,<c2>] [window,partial] in a vector.
;+    aref2 <vect> <c1> <c2> <d2>, where <c1> := coordinate in dimension 1
;+                                       <c2> := coordinate in dimension 2
;+                                       <d2> := size of dimension 2

(defun aref2 (vect c1 c2 d2)
  (aref vect (+ (* c1 d2) c2)))

;;   Interpolation linaire entre les valeurs <a> et <b>, pour un excs <frac>
;; au-del de l'abscisse de <a>. Par exemple :
;$   Lineare Interpolation zwischen den Werten <a> und <b> bei <frac>
;$ berschreitung der Abszisse von <a> an. Zum Beispiel:
;+   Linear interpolation between the values <a> and <b>, for <frac> in excess
;+ to the abscissa of <a>. For instance:
;-    (intrp-lin 0 100 0.5) => 50.0

(defun intrp-lin (a b frac)
  (+ a (*    (- b a) frac)))

;;   Interpolation exponentielle entre <a> et <b>, pour un excs <frac> au-del
;; de l'abscisse de <a>. Par exemple :
;$   Exponentielle Interpolation zwischen den Werten <a> und <b> bei <frac>
;$ berschreitung der Abszisse von <a> an. Zum Beispiel:
;+   Exponential interpolation between the values <a> and <b>, for <frac> in
;+ excess to the abscissa of <a>. For instance:
;-    (intrp-exp 1 2 0.5) => 1.4142135

(defun intrp-exp (a b frac)
  (* a (expt (/ b a) frac)))
